/*      */ package com.google.code.yanf4j.buffer;
/*      */ 
/*      */ import java.io.EOFException;
/*      */ import java.io.IOException;
/*      */ import java.io.InputStream;
/*      */ import java.io.ObjectInputStream;
/*      */ import java.io.ObjectOutputStream;
/*      */ import java.io.ObjectStreamClass;
/*      */ import java.io.OutputStream;
/*      */ import java.io.StreamCorruptedException;
/*      */ import java.nio.BufferOverflowException;
/*      */ import java.nio.BufferUnderflowException;
/*      */ import java.nio.ByteBuffer;
/*      */ import java.nio.ByteOrder;
/*      */ import java.nio.CharBuffer;
/*      */ import java.nio.DoubleBuffer;
/*      */ import java.nio.FloatBuffer;
/*      */ import java.nio.IntBuffer;
/*      */ import java.nio.LongBuffer;
/*      */ import java.nio.ShortBuffer;
/*      */ import java.nio.charset.CharacterCodingException;
/*      */ import java.nio.charset.Charset;
/*      */ import java.nio.charset.CharsetDecoder;
/*      */ import java.nio.charset.CharsetEncoder;
/*      */ import java.nio.charset.CoderResult;
/*      */ import java.util.EnumSet;
/*      */ import java.util.Set;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public abstract class AbstractIoBuffer
/*      */   extends IoBuffer
/*      */ {
/*      */   private final boolean derived;
/*      */   private boolean autoExpand;
/*      */   private boolean autoShrink;
/*   70 */   private boolean recapacityAllowed = true;
/*      */   
/*      */ 
/*      */ 
/*      */   private int minimumCapacity;
/*      */   
/*      */ 
/*      */ 
/*      */   private static final long BYTE_MASK = 255L;
/*      */   
/*      */ 
/*      */ 
/*      */   private static final long SHORT_MASK = 65535L;
/*      */   
/*      */ 
/*      */   private static final long INT_MASK = 4294967295L;
/*      */   
/*      */ 
/*   88 */   private int mark = -1;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected AbstractIoBuffer(IoBufferAllocator allocator, int initialCapacity)
/*      */   {
/*  100 */     setAllocator(allocator);
/*  101 */     this.recapacityAllowed = true;
/*  102 */     this.derived = false;
/*  103 */     this.minimumCapacity = initialCapacity;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected AbstractIoBuffer(AbstractIoBuffer parent)
/*      */   {
/*  115 */     setAllocator(getAllocator());
/*  116 */     this.recapacityAllowed = false;
/*  117 */     this.derived = true;
/*  118 */     this.minimumCapacity = parent.minimumCapacity;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean isDirect()
/*      */   {
/*  127 */     return buf().isDirect();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean isReadOnly()
/*      */   {
/*  136 */     return buf().isReadOnly();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected abstract void buf(ByteBuffer paramByteBuffer);
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final int minimumCapacity()
/*      */   {
/*  154 */     return this.minimumCapacity;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer minimumCapacity(int minimumCapacity)
/*      */   {
/*  163 */     if (minimumCapacity < 0) {
/*  164 */       throw new IllegalArgumentException("minimumCapacity: " + minimumCapacity);
/*      */     }
/*  166 */     this.minimumCapacity = minimumCapacity;
/*  167 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final int capacity()
/*      */   {
/*  176 */     return buf().capacity();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer capacity(int newCapacity)
/*      */   {
/*  185 */     if (!this.recapacityAllowed) {
/*  186 */       throw new IllegalStateException("Derived buffers and their parent can't be expanded.");
/*      */     }
/*      */     
/*      */ 
/*  190 */     if (newCapacity > capacity())
/*      */     {
/*      */ 
/*  193 */       int pos = position();
/*  194 */       int limit = limit();
/*  195 */       ByteOrder bo = order();
/*      */       
/*      */ 
/*  198 */       ByteBuffer oldBuf = buf();
/*  199 */       ByteBuffer newBuf = getAllocator().allocateNioBuffer(newCapacity, isDirect());
/*  200 */       oldBuf.clear();
/*  201 */       newBuf.put(oldBuf);
/*  202 */       buf(newBuf);
/*      */       
/*      */ 
/*  205 */       buf().limit(limit);
/*  206 */       if (this.mark >= 0) {
/*  207 */         buf().position(this.mark);
/*  208 */         buf().mark();
/*      */       }
/*  210 */       buf().position(pos);
/*  211 */       buf().order(bo);
/*      */     }
/*      */     
/*  214 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean isAutoExpand()
/*      */   {
/*  223 */     return (this.autoExpand) && (this.recapacityAllowed);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean isAutoShrink()
/*      */   {
/*  232 */     return (this.autoShrink) && (this.recapacityAllowed);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean isDerived()
/*      */   {
/*  241 */     return this.derived;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer setAutoExpand(boolean autoExpand)
/*      */   {
/*  250 */     if (!this.recapacityAllowed) {
/*  251 */       throw new IllegalStateException("Derived buffers and their parent can't be expanded.");
/*      */     }
/*  253 */     this.autoExpand = autoExpand;
/*  254 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer setAutoShrink(boolean autoShrink)
/*      */   {
/*  263 */     if (!this.recapacityAllowed) {
/*  264 */       throw new IllegalStateException("Derived buffers and their parent can't be shrinked.");
/*      */     }
/*  266 */     this.autoShrink = autoShrink;
/*  267 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer expand(int expectedRemaining)
/*      */   {
/*  276 */     return expand(position(), expectedRemaining, false);
/*      */   }
/*      */   
/*      */   private IoBuffer expand(int expectedRemaining, boolean autoExpand)
/*      */   {
/*  281 */     return expand(position(), expectedRemaining, autoExpand);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer expand(int pos, int expectedRemaining)
/*      */   {
/*  290 */     return expand(pos, expectedRemaining, false);
/*      */   }
/*      */   
/*      */   private IoBuffer expand(int pos, int expectedRemaining, boolean autoExpand)
/*      */   {
/*  295 */     if (!this.recapacityAllowed) {
/*  296 */       throw new IllegalStateException("Derived buffers and their parent can't be expanded.");
/*      */     }
/*      */     
/*  299 */     int end = pos + expectedRemaining;
/*      */     int newCapacity;
/*  301 */     int newCapacity; if (autoExpand) {
/*  302 */       newCapacity = IoBuffer.normalizeCapacity(end);
/*      */     }
/*      */     else {
/*  305 */       newCapacity = end;
/*      */     }
/*  307 */     if (newCapacity > capacity())
/*      */     {
/*  309 */       capacity(newCapacity);
/*      */     }
/*      */     
/*  312 */     if (end > limit())
/*      */     {
/*  314 */       buf().limit(end);
/*      */     }
/*  316 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer shrink()
/*      */   {
/*  326 */     if (!this.recapacityAllowed) {
/*  327 */       throw new IllegalStateException("Derived buffers and their parent can't be expanded.");
/*      */     }
/*      */     
/*  330 */     int position = position();
/*  331 */     int capacity = capacity();
/*  332 */     int limit = limit();
/*  333 */     if (capacity == limit) {
/*  334 */       return this;
/*      */     }
/*      */     
/*  337 */     int newCapacity = capacity;
/*  338 */     int minCapacity = Math.max(this.minimumCapacity, limit);
/*      */     
/*  340 */     while (newCapacity >>> 1 >= minCapacity)
/*      */     {
/*      */ 
/*  343 */       newCapacity >>>= 1;
/*      */     }
/*      */     
/*  346 */     newCapacity = Math.max(minCapacity, newCapacity);
/*      */     
/*  348 */     if (newCapacity == capacity) {
/*  349 */       return this;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  354 */     ByteOrder bo = order();
/*      */     
/*      */ 
/*  357 */     ByteBuffer oldBuf = buf();
/*  358 */     ByteBuffer newBuf = getAllocator().allocateNioBuffer(newCapacity, isDirect());
/*  359 */     oldBuf.position(0);
/*  360 */     oldBuf.limit(limit);
/*  361 */     newBuf.put(oldBuf);
/*  362 */     buf(newBuf);
/*      */     
/*      */ 
/*  365 */     buf().position(position);
/*  366 */     buf().limit(limit);
/*  367 */     buf().order(bo);
/*  368 */     this.mark = -1;
/*      */     
/*  370 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final int position()
/*      */   {
/*  379 */     return buf().position();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer position(int newPosition)
/*      */   {
/*  388 */     autoExpand(newPosition, 0);
/*  389 */     buf().position(newPosition);
/*  390 */     if (this.mark > newPosition) {
/*  391 */       this.mark = -1;
/*      */     }
/*  393 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final int limit()
/*      */   {
/*  402 */     return buf().limit();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer limit(int newLimit)
/*      */   {
/*  411 */     autoExpand(newLimit, 0);
/*  412 */     buf().limit(newLimit);
/*  413 */     if (this.mark > newLimit) {
/*  414 */       this.mark = -1;
/*      */     }
/*  416 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer mark()
/*      */   {
/*  425 */     buf().mark();
/*  426 */     this.mark = position();
/*  427 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final int markValue()
/*      */   {
/*  436 */     return this.mark;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer reset()
/*      */   {
/*  445 */     buf().reset();
/*  446 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer clear()
/*      */   {
/*  455 */     buf().clear();
/*  456 */     this.mark = -1;
/*  457 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer sweep()
/*      */   {
/*  466 */     clear();
/*  467 */     return fillAndReset(remaining());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer sweep(byte value)
/*      */   {
/*  476 */     clear();
/*  477 */     return fillAndReset(value, remaining());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer flip()
/*      */   {
/*  486 */     buf().flip();
/*  487 */     this.mark = -1;
/*  488 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer rewind()
/*      */   {
/*  497 */     buf().rewind();
/*  498 */     this.mark = -1;
/*  499 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final int remaining()
/*      */   {
/*  508 */     return limit() - position();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final boolean hasRemaining()
/*      */   {
/*  517 */     return limit() > position();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final byte get()
/*      */   {
/*  526 */     return buf().get();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final short getUnsigned()
/*      */   {
/*  535 */     return (short)(get() & 0xFF);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer put(byte b)
/*      */   {
/*  544 */     autoExpand(1);
/*  545 */     buf().put(b);
/*  546 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final byte get(int index)
/*      */   {
/*  555 */     return buf().get(index);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final short getUnsigned(int index)
/*      */   {
/*  564 */     return (short)(get(index) & 0xFF);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer put(int index, byte b)
/*      */   {
/*  573 */     autoExpand(index, 1);
/*  574 */     buf().put(index, b);
/*  575 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer get(byte[] dst, int offset, int length)
/*      */   {
/*  584 */     buf().get(dst, offset, length);
/*  585 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer put(ByteBuffer src)
/*      */   {
/*  594 */     autoExpand(src.remaining());
/*  595 */     buf().put(src);
/*  596 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer put(byte[] src, int offset, int length)
/*      */   {
/*  605 */     autoExpand(length);
/*  606 */     buf().put(src, offset, length);
/*  607 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer compact()
/*      */   {
/*  616 */     int remaining = remaining();
/*  617 */     int capacity = capacity();
/*      */     
/*  619 */     if (capacity == 0) {
/*  620 */       return this;
/*      */     }
/*      */     
/*  623 */     if ((isAutoShrink()) && (remaining <= capacity >>> 2) && (capacity > this.minimumCapacity)) {
/*  624 */       int newCapacity = capacity;
/*  625 */       int minCapacity = Math.max(this.minimumCapacity, remaining << 1);
/*      */       
/*  627 */       while (newCapacity >>> 1 >= minCapacity)
/*      */       {
/*      */ 
/*  630 */         newCapacity >>>= 1;
/*      */       }
/*      */       
/*  633 */       newCapacity = Math.max(minCapacity, newCapacity);
/*      */       
/*  635 */       if (newCapacity == capacity) {
/*  636 */         return this;
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*  641 */       ByteOrder bo = order();
/*      */       
/*      */ 
/*  644 */       if (remaining > newCapacity) {
/*  645 */         throw new IllegalStateException("The amount of the remaining bytes is greater than the new capacity.");
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*  650 */       ByteBuffer oldBuf = buf();
/*  651 */       ByteBuffer newBuf = getAllocator().allocateNioBuffer(newCapacity, isDirect());
/*  652 */       newBuf.put(oldBuf);
/*  653 */       buf(newBuf);
/*      */       
/*      */ 
/*  656 */       buf().order(bo);
/*      */     }
/*      */     else {
/*  659 */       buf().compact();
/*      */     }
/*  661 */     this.mark = -1;
/*  662 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final ByteOrder order()
/*      */   {
/*  671 */     return buf().order();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer order(ByteOrder bo)
/*      */   {
/*  680 */     buf().order(bo);
/*  681 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final char getChar()
/*      */   {
/*  690 */     return buf().getChar();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer putChar(char value)
/*      */   {
/*  699 */     autoExpand(2);
/*  700 */     buf().putChar(value);
/*  701 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final char getChar(int index)
/*      */   {
/*  710 */     return buf().getChar(index);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer putChar(int index, char value)
/*      */   {
/*  719 */     autoExpand(index, 2);
/*  720 */     buf().putChar(index, value);
/*  721 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final CharBuffer asCharBuffer()
/*      */   {
/*  730 */     return buf().asCharBuffer();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final short getShort()
/*      */   {
/*  739 */     return buf().getShort();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer putShort(short value)
/*      */   {
/*  748 */     autoExpand(2);
/*  749 */     buf().putShort(value);
/*  750 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final short getShort(int index)
/*      */   {
/*  759 */     return buf().getShort(index);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer putShort(int index, short value)
/*      */   {
/*  768 */     autoExpand(index, 2);
/*  769 */     buf().putShort(index, value);
/*  770 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final ShortBuffer asShortBuffer()
/*      */   {
/*  779 */     return buf().asShortBuffer();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final int getInt()
/*      */   {
/*  788 */     return buf().getInt();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer putInt(int value)
/*      */   {
/*  797 */     autoExpand(4);
/*  798 */     buf().putInt(value);
/*  799 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final int getInt(int index)
/*      */   {
/*  808 */     return buf().getInt(index);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer putInt(int index, int value)
/*      */   {
/*  817 */     autoExpand(index, 4);
/*  818 */     buf().putInt(index, value);
/*  819 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IntBuffer asIntBuffer()
/*      */   {
/*  828 */     return buf().asIntBuffer();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final long getLong()
/*      */   {
/*  837 */     return buf().getLong();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer putLong(long value)
/*      */   {
/*  846 */     autoExpand(8);
/*  847 */     buf().putLong(value);
/*  848 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final long getLong(int index)
/*      */   {
/*  857 */     return buf().getLong(index);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer putLong(int index, long value)
/*      */   {
/*  866 */     autoExpand(index, 8);
/*  867 */     buf().putLong(index, value);
/*  868 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final LongBuffer asLongBuffer()
/*      */   {
/*  877 */     return buf().asLongBuffer();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final float getFloat()
/*      */   {
/*  886 */     return buf().getFloat();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer putFloat(float value)
/*      */   {
/*  895 */     autoExpand(4);
/*  896 */     buf().putFloat(value);
/*  897 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final float getFloat(int index)
/*      */   {
/*  906 */     return buf().getFloat(index);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer putFloat(int index, float value)
/*      */   {
/*  915 */     autoExpand(index, 4);
/*  916 */     buf().putFloat(index, value);
/*  917 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final FloatBuffer asFloatBuffer()
/*      */   {
/*  926 */     return buf().asFloatBuffer();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final double getDouble()
/*      */   {
/*  935 */     return buf().getDouble();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer putDouble(double value)
/*      */   {
/*  944 */     autoExpand(8);
/*  945 */     buf().putDouble(value);
/*  946 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final double getDouble(int index)
/*      */   {
/*  955 */     return buf().getDouble(index);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer putDouble(int index, double value)
/*      */   {
/*  964 */     autoExpand(index, 8);
/*  965 */     buf().putDouble(index, value);
/*  966 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final DoubleBuffer asDoubleBuffer()
/*      */   {
/*  975 */     return buf().asDoubleBuffer();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer asReadOnlyBuffer()
/*      */   {
/*  984 */     this.recapacityAllowed = false;
/*  985 */     return asReadOnlyBuffer0();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected abstract IoBuffer asReadOnlyBuffer0();
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer duplicate()
/*      */   {
/* 1001 */     this.recapacityAllowed = false;
/* 1002 */     return duplicate0();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected abstract IoBuffer duplicate0();
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer slice()
/*      */   {
/* 1018 */     this.recapacityAllowed = false;
/* 1019 */     return slice0();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer getSlice(int index, int length)
/*      */   {
/* 1028 */     if (length < 0) {
/* 1029 */       throw new IllegalArgumentException("length: " + length);
/*      */     }
/*      */     
/* 1032 */     int limit = limit();
/*      */     
/* 1034 */     if (index > limit) {
/* 1035 */       throw new IllegalArgumentException("index: " + index);
/*      */     }
/*      */     
/* 1038 */     int endIndex = index + length;
/*      */     
/* 1040 */     if (capacity() < endIndex) {
/* 1041 */       throw new IndexOutOfBoundsException("index + length (" + endIndex + ") is greater " + "than capacity (" + capacity() + ").");
/*      */     }
/*      */     
/*      */ 
/* 1045 */     clear();
/* 1046 */     position(index);
/* 1047 */     limit(endIndex);
/*      */     
/* 1049 */     IoBuffer slice = slice();
/* 1050 */     position(index);
/* 1051 */     limit(limit);
/* 1052 */     return slice;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public final IoBuffer getSlice(int length)
/*      */   {
/* 1061 */     if (length < 0) {
/* 1062 */       throw new IllegalArgumentException("length: " + length);
/*      */     }
/* 1064 */     int pos = position();
/* 1065 */     int limit = limit();
/* 1066 */     int nextPos = pos + length;
/* 1067 */     if (limit < nextPos) {
/* 1068 */       throw new IndexOutOfBoundsException("position + length (" + nextPos + ") is greater " + "than limit (" + limit + ").");
/*      */     }
/*      */     
/*      */ 
/* 1072 */     limit(pos + length);
/* 1073 */     IoBuffer slice = slice();
/* 1074 */     position(nextPos);
/* 1075 */     limit(limit);
/* 1076 */     return slice;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected abstract IoBuffer slice0();
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public int hashCode()
/*      */   {
/* 1091 */     int h = 1;
/* 1092 */     int p = position();
/* 1093 */     for (int i = limit() - 1; i >= p; i--) {
/* 1094 */       h = 31 * h + get(i);
/*      */     }
/* 1096 */     return h;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public boolean equals(Object o)
/*      */   {
/* 1105 */     if (!(o instanceof IoBuffer)) {
/* 1106 */       return false;
/*      */     }
/*      */     
/* 1109 */     IoBuffer that = (IoBuffer)o;
/* 1110 */     if (remaining() != that.remaining()) {
/* 1111 */       return false;
/*      */     }
/*      */     
/* 1114 */     int p = position();
/* 1115 */     int i = limit() - 1; for (int j = that.limit() - 1; i >= p; j--) {
/* 1116 */       byte v1 = get(i);
/* 1117 */       byte v2 = that.get(j);
/* 1118 */       if (v1 != v2) {
/* 1119 */         return false;
/*      */       }
/* 1115 */       i--;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1122 */     return true;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public int compareTo(IoBuffer that)
/*      */   {
/* 1130 */     int n = position() + Math.min(remaining(), that.remaining());
/* 1131 */     int i = position(); for (int j = that.position(); i < n; j++) {
/* 1132 */       byte v1 = get(i);
/* 1133 */       byte v2 = that.get(j);
/* 1134 */       if (v1 != v2)
/*      */       {
/*      */ 
/* 1137 */         if (v1 < v2) {
/* 1138 */           return -1;
/*      */         }
/*      */         
/* 1141 */         return 1;
/*      */       }
/* 1131 */       i++;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1143 */     return remaining() - that.remaining();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public String toString()
/*      */   {
/* 1152 */     StringBuilder buf = new StringBuilder();
/* 1153 */     if (isDirect()) {
/* 1154 */       buf.append("DirectBuffer");
/*      */     }
/*      */     else {
/* 1157 */       buf.append("HeapBuffer");
/*      */     }
/* 1159 */     buf.append("[pos=");
/* 1160 */     buf.append(position());
/* 1161 */     buf.append(" lim=");
/* 1162 */     buf.append(limit());
/* 1163 */     buf.append(" cap=");
/* 1164 */     buf.append(capacity());
/* 1165 */     buf.append(": ");
/* 1166 */     buf.append(getHexDump(16));
/* 1167 */     buf.append(']');
/* 1168 */     return buf.toString();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer get(byte[] dst)
/*      */   {
/* 1177 */     return get(dst, 0, dst.length);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer put(IoBuffer src)
/*      */   {
/* 1186 */     return put(src.buf());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer put(byte[] src)
/*      */   {
/* 1195 */     return put(src, 0, src.length);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public int getUnsignedShort()
/*      */   {
/* 1204 */     return getShort() & 0xFFFF;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public int getUnsignedShort(int index)
/*      */   {
/* 1213 */     return getShort(index) & 0xFFFF;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public long getUnsignedInt()
/*      */   {
/* 1222 */     return getInt() & 0xFFFFFFFF;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public int getMediumInt()
/*      */   {
/* 1231 */     byte b1 = get();
/* 1232 */     byte b2 = get();
/* 1233 */     byte b3 = get();
/* 1234 */     if (ByteOrder.BIG_ENDIAN.equals(order())) {
/* 1235 */       return getMediumInt(b1, b2, b3);
/*      */     }
/*      */     
/* 1238 */     return getMediumInt(b3, b2, b1);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public int getUnsignedMediumInt()
/*      */   {
/* 1248 */     int b1 = getUnsigned();
/* 1249 */     int b2 = getUnsigned();
/* 1250 */     int b3 = getUnsigned();
/* 1251 */     if (ByteOrder.BIG_ENDIAN.equals(order())) {
/* 1252 */       return b1 << 16 | b2 << 8 | b3;
/*      */     }
/*      */     
/* 1255 */     return b3 << 16 | b2 << 8 | b1;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public int getMediumInt(int index)
/*      */   {
/* 1265 */     byte b1 = get(index);
/* 1266 */     byte b2 = get(index + 1);
/* 1267 */     byte b3 = get(index + 2);
/* 1268 */     if (ByteOrder.BIG_ENDIAN.equals(order())) {
/* 1269 */       return getMediumInt(b1, b2, b3);
/*      */     }
/*      */     
/* 1272 */     return getMediumInt(b3, b2, b1);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public int getUnsignedMediumInt(int index)
/*      */   {
/* 1282 */     int b1 = getUnsigned(index);
/* 1283 */     int b2 = getUnsigned(index + 1);
/* 1284 */     int b3 = getUnsigned(index + 2);
/* 1285 */     if (ByteOrder.BIG_ENDIAN.equals(order())) {
/* 1286 */       return b1 << 16 | b2 << 8 | b3;
/*      */     }
/*      */     
/* 1289 */     return b3 << 16 | b2 << 8 | b1;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private int getMediumInt(byte b1, byte b2, byte b3)
/*      */   {
/* 1298 */     int ret = b1 << 16 & 0xFF0000 | b2 << 8 & 0xFF00 | b3 & 0xFF;
/*      */     
/* 1300 */     if ((b1 & 0x80) == 128)
/*      */     {
/* 1302 */       ret |= 0xFF000000;
/*      */     }
/* 1304 */     return ret;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer putMediumInt(int value)
/*      */   {
/* 1313 */     byte b1 = (byte)(value >> 16);
/* 1314 */     byte b2 = (byte)(value >> 8);
/* 1315 */     byte b3 = (byte)value;
/*      */     
/* 1317 */     if (ByteOrder.BIG_ENDIAN.equals(order())) {
/* 1318 */       put(b1).put(b2).put(b3);
/*      */     }
/*      */     else {
/* 1321 */       put(b3).put(b2).put(b1);
/*      */     }
/*      */     
/* 1324 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer putMediumInt(int index, int value)
/*      */   {
/* 1333 */     byte b1 = (byte)(value >> 16);
/* 1334 */     byte b2 = (byte)(value >> 8);
/* 1335 */     byte b3 = (byte)value;
/*      */     
/* 1337 */     if (ByteOrder.BIG_ENDIAN.equals(order())) {
/* 1338 */       put(index, b1).put(index + 1, b2).put(index + 2, b3);
/*      */     }
/*      */     else {
/* 1341 */       put(index, b3).put(index + 1, b2).put(index + 2, b1);
/*      */     }
/*      */     
/* 1344 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public long getUnsignedInt(int index)
/*      */   {
/* 1353 */     return getInt(index) & 0xFFFFFFFF;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public InputStream asInputStream()
/*      */   {
/* 1362 */     new InputStream()
/*      */     {
/*      */       public int available() {
/* 1365 */         return AbstractIoBuffer.this.remaining();
/*      */       }
/*      */       
/*      */ 
/*      */       public synchronized void mark(int readlimit)
/*      */       {
/* 1371 */         AbstractIoBuffer.this.mark();
/*      */       }
/*      */       
/*      */ 
/*      */       public boolean markSupported()
/*      */       {
/* 1377 */         return true;
/*      */       }
/*      */       
/*      */ 
/*      */       public int read()
/*      */       {
/* 1383 */         if (AbstractIoBuffer.this.hasRemaining()) {
/* 1384 */           return AbstractIoBuffer.this.get() & 0xFF;
/*      */         }
/*      */         
/* 1387 */         return -1;
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */       public int read(byte[] b, int off, int len)
/*      */       {
/* 1394 */         int remaining = AbstractIoBuffer.this.remaining();
/* 1395 */         if (remaining > 0) {
/* 1396 */           int readBytes = Math.min(remaining, len);
/* 1397 */           AbstractIoBuffer.this.get(b, off, readBytes);
/* 1398 */           return readBytes;
/*      */         }
/*      */         
/* 1401 */         return -1;
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */       public synchronized void reset()
/*      */       {
/* 1408 */         AbstractIoBuffer.this.reset();
/*      */       }
/*      */       
/*      */       public long skip(long n)
/*      */       {
/*      */         int bytes;
/*      */         int bytes;
/* 1415 */         if (n > 2147483647L) {
/* 1416 */           bytes = AbstractIoBuffer.this.remaining();
/*      */         }
/*      */         else {
/* 1419 */           bytes = Math.min(AbstractIoBuffer.this.remaining(), (int)n);
/*      */         }
/* 1421 */         AbstractIoBuffer.this.skip(bytes);
/* 1422 */         return bytes;
/*      */       }
/*      */     };
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public OutputStream asOutputStream()
/*      */   {
/* 1433 */     new OutputStream()
/*      */     {
/*      */       public void write(byte[] b, int off, int len) {
/* 1436 */         AbstractIoBuffer.this.put(b, off, len);
/*      */       }
/*      */       
/*      */ 
/*      */       public void write(int b)
/*      */       {
/* 1442 */         AbstractIoBuffer.this.put((byte)b);
/*      */       }
/*      */     };
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public String getHexDump()
/*      */   {
/* 1453 */     return getHexDump(Integer.MAX_VALUE);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public String getHexDump(int lengthLimit)
/*      */   {
/* 1462 */     return IoBufferHexDumper.getHexdump(this, lengthLimit);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public String getString(CharsetDecoder decoder)
/*      */     throws CharacterCodingException
/*      */   {
/* 1471 */     if (!hasRemaining()) {
/* 1472 */       return "";
/*      */     }
/*      */     
/* 1475 */     boolean utf16 = decoder.charset().name().startsWith("UTF-16");
/*      */     
/* 1477 */     int oldPos = position();
/* 1478 */     int oldLimit = limit();
/* 1479 */     int end = -1;
/*      */     int newPos;
/*      */     int newPos;
/* 1482 */     if (!utf16) {
/* 1483 */       end = indexOf((byte)0);
/* 1484 */       int newPos; if (end < 0) {
/* 1485 */         newPos = end = oldLimit;
/*      */       }
/*      */       else {
/* 1488 */         newPos = end + 1;
/*      */       }
/*      */     }
/*      */     else {
/* 1492 */       int i = oldPos;
/*      */       for (;;) {
/* 1494 */         boolean wasZero = get(i) == 0;
/* 1495 */         i++;
/*      */         
/* 1497 */         if (i >= oldLimit) {
/*      */           break;
/*      */         }
/*      */         
/* 1501 */         if (get(i) != 0) {
/* 1502 */           i++;
/* 1503 */           if (i >= oldLimit)
/*      */           {
/*      */             break;
/*      */ 
/*      */           }
/*      */           
/*      */ 
/*      */         }
/* 1511 */         else if (wasZero) {
/* 1512 */           end = i - 1;
/* 1513 */           break;
/*      */         }
/*      */       }
/*      */       int newPos;
/* 1517 */       if (end < 0) {
/* 1518 */         newPos = end = oldPos + (oldLimit - oldPos & 0xFFFFFFFE);
/*      */       } else {
/*      */         int newPos;
/* 1521 */         if (end + 2 <= oldLimit) {
/* 1522 */           newPos = end + 2;
/*      */         }
/*      */         else {
/* 1525 */           newPos = end;
/*      */         }
/*      */       }
/*      */     }
/*      */     
/* 1530 */     if (oldPos == end) {
/* 1531 */       position(newPos);
/* 1532 */       return "";
/*      */     }
/*      */     
/* 1535 */     limit(end);
/* 1536 */     decoder.reset();
/*      */     
/* 1538 */     int expectedLength = (int)(remaining() * decoder.averageCharsPerByte()) + 1;
/* 1539 */     CharBuffer out = CharBuffer.allocate(expectedLength);
/*      */     for (;;) { CoderResult cr;
/*      */       CoderResult cr;
/* 1542 */       if (hasRemaining()) {
/* 1543 */         cr = decoder.decode(buf(), out, true);
/*      */       }
/*      */       else {
/* 1546 */         cr = decoder.flush(out);
/*      */       }
/*      */       
/* 1549 */       if (cr.isUnderflow()) {
/*      */         break;
/*      */       }
/*      */       
/* 1553 */       if (cr.isOverflow()) {
/* 1554 */         CharBuffer o = CharBuffer.allocate(out.capacity() + expectedLength);
/* 1555 */         out.flip();
/* 1556 */         o.put(out);
/* 1557 */         out = o;
/*      */ 
/*      */ 
/*      */       }
/* 1561 */       else if (cr.isError())
/*      */       {
/* 1563 */         limit(oldLimit);
/* 1564 */         position(oldPos);
/* 1565 */         cr.throwException();
/*      */       }
/*      */     }
/*      */     
/* 1569 */     limit(oldLimit);
/* 1570 */     position(newPos);
/* 1571 */     return out.flip().toString();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public String getString(int fieldSize, CharsetDecoder decoder)
/*      */     throws CharacterCodingException
/*      */   {
/* 1580 */     checkFieldSize(fieldSize);
/*      */     
/* 1582 */     if (fieldSize == 0) {
/* 1583 */       return "";
/*      */     }
/*      */     
/* 1586 */     if (!hasRemaining()) {
/* 1587 */       return "";
/*      */     }
/*      */     
/* 1590 */     boolean utf16 = decoder.charset().name().startsWith("UTF-16");
/*      */     
/* 1592 */     if ((utf16) && ((fieldSize & 0x1) != 0)) {
/* 1593 */       throw new IllegalArgumentException("fieldSize is not even.");
/*      */     }
/*      */     
/* 1596 */     int oldPos = position();
/* 1597 */     int oldLimit = limit();
/* 1598 */     int end = oldPos + fieldSize;
/*      */     
/* 1600 */     if (oldLimit < end) {
/* 1601 */       throw new BufferUnderflowException();
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 1606 */     if (!utf16) {
/* 1607 */       for (int i = oldPos; i < end; i++) {
/* 1608 */         if (get(i) == 0) {
/*      */           break;
/*      */         }
/*      */       }
/*      */       
/* 1613 */       if (i == end) {
/* 1614 */         limit(end);
/*      */       }
/*      */       else {
/* 1617 */         limit(i);
/*      */       }
/*      */     }
/*      */     else {
/* 1621 */       for (int i = oldPos; i < end; i += 2) {
/* 1622 */         if ((get(i) == 0) && (get(i + 1) == 0)) {
/*      */           break;
/*      */         }
/*      */       }
/*      */       
/* 1627 */       if (i == end) {
/* 1628 */         limit(end);
/*      */       }
/*      */       else {
/* 1631 */         limit(i);
/*      */       }
/*      */     }
/*      */     
/* 1635 */     if (!hasRemaining()) {
/* 1636 */       limit(oldLimit);
/* 1637 */       position(end);
/* 1638 */       return "";
/*      */     }
/* 1640 */     decoder.reset();
/*      */     
/* 1642 */     int expectedLength = (int)(remaining() * decoder.averageCharsPerByte()) + 1;
/* 1643 */     CharBuffer out = CharBuffer.allocate(expectedLength);
/*      */     for (;;) { CoderResult cr;
/*      */       CoderResult cr;
/* 1646 */       if (hasRemaining()) {
/* 1647 */         cr = decoder.decode(buf(), out, true);
/*      */       }
/*      */       else {
/* 1650 */         cr = decoder.flush(out);
/*      */       }
/*      */       
/* 1653 */       if (cr.isUnderflow()) {
/*      */         break;
/*      */       }
/*      */       
/* 1657 */       if (cr.isOverflow()) {
/* 1658 */         CharBuffer o = CharBuffer.allocate(out.capacity() + expectedLength);
/* 1659 */         out.flip();
/* 1660 */         o.put(out);
/* 1661 */         out = o;
/*      */ 
/*      */ 
/*      */       }
/* 1665 */       else if (cr.isError())
/*      */       {
/* 1667 */         limit(oldLimit);
/* 1668 */         position(oldPos);
/* 1669 */         cr.throwException();
/*      */       }
/*      */     }
/*      */     
/* 1673 */     limit(oldLimit);
/* 1674 */     position(end);
/* 1675 */     return out.flip().toString();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer putString(CharSequence val, CharsetEncoder encoder)
/*      */     throws CharacterCodingException
/*      */   {
/* 1684 */     if (val.length() == 0) {
/* 1685 */       return this;
/*      */     }
/*      */     
/* 1688 */     CharBuffer in = CharBuffer.wrap(val);
/* 1689 */     encoder.reset();
/*      */     
/* 1691 */     int expandedState = 0;
/*      */     for (;;) {
/*      */       CoderResult cr;
/*      */       CoderResult cr;
/* 1695 */       if (in.hasRemaining()) {
/* 1696 */         cr = encoder.encode(in, buf(), true);
/*      */       }
/*      */       else {
/* 1699 */         cr = encoder.flush(buf());
/*      */       }
/*      */       
/* 1702 */       if (cr.isUnderflow()) {
/*      */         break;
/*      */       }
/* 1705 */       if (cr.isOverflow()) {
/* 1706 */         if (isAutoExpand()) {
/* 1707 */           switch (expandedState) {
/*      */           case 0: 
/* 1709 */             autoExpand((int)Math.ceil(in.remaining() * encoder.averageBytesPerChar()));
/* 1710 */             expandedState++;
/* 1711 */             break;
/*      */           case 1: 
/* 1713 */             autoExpand((int)Math.ceil(in.remaining() * encoder.maxBytesPerChar()));
/* 1714 */             expandedState++;
/* 1715 */             break;
/*      */           default: 
/* 1717 */             throw new RuntimeException("Expanded by " + (int)Math.ceil(in.remaining() * encoder.maxBytesPerChar()) + " but that wasn't enough for '" + val + "'");
/*      */           
/*      */ 
/*      */           }
/*      */           
/*      */         }
/*      */       }
/*      */       else {
/* 1725 */         expandedState = 0;
/*      */       }
/* 1727 */       cr.throwException();
/*      */     }
/* 1729 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer putString(CharSequence val, int fieldSize, CharsetEncoder encoder)
/*      */     throws CharacterCodingException
/*      */   {
/* 1738 */     checkFieldSize(fieldSize);
/*      */     
/* 1740 */     if (fieldSize == 0) {
/* 1741 */       return this;
/*      */     }
/*      */     
/* 1744 */     autoExpand(fieldSize);
/*      */     
/* 1746 */     boolean utf16 = encoder.charset().name().startsWith("UTF-16");
/*      */     
/* 1748 */     if ((utf16) && ((fieldSize & 0x1) != 0)) {
/* 1749 */       throw new IllegalArgumentException("fieldSize is not even.");
/*      */     }
/*      */     
/* 1752 */     int oldLimit = limit();
/* 1753 */     int end = position() + fieldSize;
/*      */     
/* 1755 */     if (oldLimit < end) {
/* 1756 */       throw new BufferOverflowException();
/*      */     }
/*      */     
/* 1759 */     if (val.length() == 0) {
/* 1760 */       if (!utf16) {
/* 1761 */         put((byte)0);
/*      */       }
/*      */       else {
/* 1764 */         put((byte)0);
/* 1765 */         put((byte)0);
/*      */       }
/* 1767 */       position(end);
/* 1768 */       return this;
/*      */     }
/*      */     
/* 1771 */     CharBuffer in = CharBuffer.wrap(val);
/* 1772 */     limit(end);
/* 1773 */     encoder.reset();
/*      */     for (;;) {
/*      */       CoderResult cr;
/*      */       CoderResult cr;
/* 1777 */       if (in.hasRemaining()) {
/* 1778 */         cr = encoder.encode(in, buf(), true);
/*      */       }
/*      */       else {
/* 1781 */         cr = encoder.flush(buf());
/*      */       }
/*      */       
/* 1784 */       if ((cr.isUnderflow()) || (cr.isOverflow())) {
/*      */         break;
/*      */       }
/* 1787 */       cr.throwException();
/*      */     }
/*      */     
/* 1790 */     limit(oldLimit);
/*      */     
/* 1792 */     if (position() < end) {
/* 1793 */       if (!utf16) {
/* 1794 */         put((byte)0);
/*      */       }
/*      */       else {
/* 1797 */         put((byte)0);
/* 1798 */         put((byte)0);
/*      */       }
/*      */     }
/*      */     
/* 1802 */     position(end);
/* 1803 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public String getPrefixedString(CharsetDecoder decoder)
/*      */     throws CharacterCodingException
/*      */   {
/* 1812 */     return getPrefixedString(2, decoder);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public String getPrefixedString(int prefixLength, CharsetDecoder decoder)
/*      */     throws CharacterCodingException
/*      */   {
/* 1832 */     if (!prefixedDataAvailable(prefixLength)) {
/* 1833 */       throw new BufferUnderflowException();
/*      */     }
/*      */     
/* 1836 */     int fieldSize = 0;
/*      */     
/* 1838 */     switch (prefixLength) {
/*      */     case 1: 
/* 1840 */       fieldSize = getUnsigned();
/* 1841 */       break;
/*      */     case 2: 
/* 1843 */       fieldSize = getUnsignedShort();
/* 1844 */       break;
/*      */     case 4: 
/* 1846 */       fieldSize = getInt();
/*      */     }
/*      */     
/*      */     
/* 1850 */     if (fieldSize == 0) {
/* 1851 */       return "";
/*      */     }
/*      */     
/* 1854 */     boolean utf16 = decoder.charset().name().startsWith("UTF-16");
/*      */     
/* 1856 */     if ((utf16) && ((fieldSize & 0x1) != 0)) {
/* 1857 */       throw new BufferDataException("fieldSize is not even for a UTF-16 string.");
/*      */     }
/*      */     
/* 1860 */     int oldLimit = limit();
/* 1861 */     int end = position() + fieldSize;
/*      */     
/* 1863 */     if (oldLimit < end) {
/* 1864 */       throw new BufferUnderflowException();
/*      */     }
/*      */     
/* 1867 */     limit(end);
/* 1868 */     decoder.reset();
/*      */     
/* 1870 */     int expectedLength = (int)(remaining() * decoder.averageCharsPerByte()) + 1;
/* 1871 */     CharBuffer out = CharBuffer.allocate(expectedLength);
/*      */     for (;;) { CoderResult cr;
/*      */       CoderResult cr;
/* 1874 */       if (hasRemaining()) {
/* 1875 */         cr = decoder.decode(buf(), out, true);
/*      */       }
/*      */       else {
/* 1878 */         cr = decoder.flush(out);
/*      */       }
/*      */       
/* 1881 */       if (cr.isUnderflow()) {
/*      */         break;
/*      */       }
/*      */       
/* 1885 */       if (cr.isOverflow()) {
/* 1886 */         CharBuffer o = CharBuffer.allocate(out.capacity() + expectedLength);
/* 1887 */         out.flip();
/* 1888 */         o.put(out);
/* 1889 */         out = o;
/*      */       }
/*      */       else
/*      */       {
/* 1893 */         cr.throwException();
/*      */       }
/*      */     }
/* 1896 */     limit(oldLimit);
/* 1897 */     position(end);
/* 1898 */     return out.flip().toString();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer putPrefixedString(CharSequence in, CharsetEncoder encoder)
/*      */     throws CharacterCodingException
/*      */   {
/* 1907 */     return putPrefixedString(in, 2, 0, encoder);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer putPrefixedString(CharSequence in, int prefixLength, CharsetEncoder encoder)
/*      */     throws CharacterCodingException
/*      */   {
/* 1917 */     return putPrefixedString(in, prefixLength, 0, encoder);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer putPrefixedString(CharSequence in, int prefixLength, int padding, CharsetEncoder encoder)
/*      */     throws CharacterCodingException
/*      */   {
/* 1927 */     return putPrefixedString(in, prefixLength, padding, (byte)0, encoder);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public IoBuffer putPrefixedString(CharSequence val, int prefixLength, int padding, byte padValue, CharsetEncoder encoder)
/*      */     throws CharacterCodingException
/*      */   {
/*      */     int maxLength;
/*      */     
/*      */ 
/* 1938 */     switch (prefixLength) {
/*      */     case 1: 
/* 1940 */       maxLength = 255;
/* 1941 */       break;
/*      */     case 2: 
/* 1943 */       maxLength = 65535;
/* 1944 */       break;
/*      */     case 4: 
/* 1946 */       maxLength = Integer.MAX_VALUE;
/* 1947 */       break;
/*      */     case 3: default: 
/* 1949 */       throw new IllegalArgumentException("prefixLength: " + prefixLength);
/*      */     }
/*      */     
/* 1952 */     if (val.length() > maxLength) {
/* 1953 */       throw new IllegalArgumentException("The specified string is too long.");
/*      */     }
/* 1955 */     if (val.length() == 0) {
/* 1956 */       switch (prefixLength) {
/*      */       case 1: 
/* 1958 */         put((byte)0);
/* 1959 */         break;
/*      */       case 2: 
/* 1961 */         putShort((short)0);
/* 1962 */         break;
/*      */       case 4: 
/* 1964 */         putInt(0);
/*      */       }
/*      */       
/* 1967 */       return this;
/*      */     }
/*      */     
/*      */     int padMask;
/* 1971 */     switch (padding) {
/*      */     case 0: 
/*      */     case 1: 
/* 1974 */       padMask = 0;
/* 1975 */       break;
/*      */     case 2: 
/* 1977 */       padMask = 1;
/* 1978 */       break;
/*      */     case 4: 
/* 1980 */       padMask = 3;
/* 1981 */       break;
/*      */     case 3: default: 
/* 1983 */       throw new IllegalArgumentException("padding: " + padding);
/*      */     }
/*      */     
/* 1986 */     CharBuffer in = CharBuffer.wrap(val);
/* 1987 */     skip(prefixLength);
/* 1988 */     int oldPos = position();
/* 1989 */     encoder.reset();
/*      */     
/* 1991 */     int expandedState = 0;
/*      */     for (;;) {
/*      */       CoderResult cr;
/*      */       CoderResult cr;
/* 1995 */       if (in.hasRemaining()) {
/* 1996 */         cr = encoder.encode(in, buf(), true);
/*      */       }
/*      */       else {
/* 1999 */         cr = encoder.flush(buf());
/*      */       }
/*      */       
/* 2002 */       if (position() - oldPos > maxLength) {
/* 2003 */         throw new IllegalArgumentException("The specified string is too long.");
/*      */       }
/*      */       
/* 2006 */       if (cr.isUnderflow()) {
/*      */         break;
/*      */       }
/* 2009 */       if (cr.isOverflow()) {
/* 2010 */         if (isAutoExpand()) {
/* 2011 */           switch (expandedState) {
/*      */           case 0: 
/* 2013 */             autoExpand((int)Math.ceil(in.remaining() * encoder.averageBytesPerChar()));
/* 2014 */             expandedState++;
/* 2015 */             break;
/*      */           case 1: 
/* 2017 */             autoExpand((int)Math.ceil(in.remaining() * encoder.maxBytesPerChar()));
/* 2018 */             expandedState++;
/* 2019 */             break;
/*      */           default: 
/* 2021 */             throw new RuntimeException("Expanded by " + (int)Math.ceil(in.remaining() * encoder.maxBytesPerChar()) + " but that wasn't enough for '" + val + "'");
/*      */           
/*      */ 
/*      */           }
/*      */           
/*      */         }
/*      */       }
/*      */       else {
/* 2029 */         expandedState = 0;
/*      */       }
/* 2031 */       cr.throwException();
/*      */     }
/*      */     
/*      */ 
/* 2035 */     fill(padValue, padding - (position() - oldPos & padMask));
/* 2036 */     int length = position() - oldPos;
/* 2037 */     switch (prefixLength) {
/*      */     case 1: 
/* 2039 */       put(oldPos - 1, (byte)length);
/* 2040 */       break;
/*      */     case 2: 
/* 2042 */       putShort(oldPos - 2, (short)length);
/* 2043 */       break;
/*      */     case 4: 
/* 2045 */       putInt(oldPos - 4, length);
/*      */     }
/*      */     
/* 2048 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Object getObject()
/*      */     throws ClassNotFoundException
/*      */   {
/* 2057 */     return getObject(Thread.currentThread().getContextClassLoader());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Object getObject(final ClassLoader classLoader)
/*      */     throws ClassNotFoundException
/*      */   {
/* 2066 */     if (!prefixedDataAvailable(4)) {
/* 2067 */       throw new BufferUnderflowException();
/*      */     }
/*      */     
/* 2070 */     int length = getInt();
/* 2071 */     if (length <= 4) {
/* 2072 */       throw new BufferDataException("Object length should be greater than 4: " + length);
/*      */     }
/*      */     
/* 2075 */     int oldLimit = limit();
/* 2076 */     limit(position() + length);
/*      */     try {
/* 2078 */       ObjectInputStream in = new ObjectInputStream(asInputStream())
/*      */       {
/*      */         protected ObjectStreamClass readClassDescriptor() throws IOException, ClassNotFoundException {
/* 2081 */           int type = read();
/* 2082 */           if (type < 0) {
/* 2083 */             throw new EOFException();
/*      */           }
/* 2085 */           switch (type) {
/*      */           case 0: 
/* 2087 */             return super.readClassDescriptor();
/*      */           case 1: 
/* 2089 */             String className = readUTF();
/* 2090 */             Class<?> clazz = Class.forName(className, true, classLoader);
/* 2091 */             return ObjectStreamClass.lookup(clazz);
/*      */           }
/* 2093 */           throw new StreamCorruptedException("Unexpected class descriptor type: " + type);
/*      */         }
/*      */         
/*      */ 
/*      */         protected Class<?> resolveClass(ObjectStreamClass desc)
/*      */           throws IOException, ClassNotFoundException
/*      */         {
/* 2100 */           String name = desc.getName();
/*      */           try {
/* 2102 */             return Class.forName(name, false, classLoader);
/*      */           }
/*      */           catch (ClassNotFoundException ex) {}
/* 2105 */           return super.resolveClass(desc);
/*      */         }
/*      */         
/* 2108 */       };
/* 2109 */       return in.readObject();
/*      */     }
/*      */     catch (IOException e) {
/* 2112 */       throw new BufferDataException(e);
/*      */     }
/*      */     finally {
/* 2115 */       limit(oldLimit);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer putObject(Object o)
/*      */   {
/* 2125 */     int oldPos = position();
/* 2126 */     skip(4);
/*      */     try {
/* 2128 */       ObjectOutputStream out = new ObjectOutputStream(asOutputStream())
/*      */       {
/*      */         protected void writeClassDescriptor(ObjectStreamClass desc) throws IOException {
/* 2131 */           if (desc.forClass().isPrimitive()) {
/* 2132 */             write(0);
/* 2133 */             super.writeClassDescriptor(desc);
/*      */           }
/*      */           else {
/* 2136 */             write(1);
/* 2137 */             writeUTF(desc.getName());
/*      */           }
/*      */         }
/* 2140 */       };
/* 2141 */       out.writeObject(o);
/* 2142 */       out.flush();
/*      */     }
/*      */     catch (IOException e) {
/* 2145 */       throw new BufferDataException(e);
/*      */     }
/*      */     
/*      */ 
/* 2149 */     int newPos = position();
/* 2150 */     position(oldPos);
/* 2151 */     putInt(newPos - oldPos - 4);
/* 2152 */     position(newPos);
/* 2153 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public boolean prefixedDataAvailable(int prefixLength)
/*      */   {
/* 2162 */     return prefixedDataAvailable(prefixLength, Integer.MAX_VALUE);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public boolean prefixedDataAvailable(int prefixLength, int maxDataLength)
/*      */   {
/* 2171 */     if (remaining() < prefixLength) {
/* 2172 */       return false;
/*      */     }
/*      */     
/*      */     int dataLength;
/* 2176 */     switch (prefixLength) {
/*      */     case 1: 
/* 2178 */       dataLength = getUnsigned(position());
/* 2179 */       break;
/*      */     case 2: 
/* 2181 */       dataLength = getUnsignedShort(position());
/* 2182 */       break;
/*      */     case 4: 
/* 2184 */       dataLength = getInt(position());
/* 2185 */       break;
/*      */     case 3: default: 
/* 2187 */       throw new IllegalArgumentException("prefixLength: " + prefixLength);
/*      */     }
/*      */     
/* 2190 */     if ((dataLength < 0) || (dataLength > maxDataLength)) {
/* 2191 */       throw new BufferDataException("dataLength: " + dataLength);
/*      */     }
/*      */     
/* 2194 */     return remaining() - prefixLength >= dataLength;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public int indexOf(byte b)
/*      */   {
/* 2203 */     if (hasArray()) {
/* 2204 */       int arrayOffset = arrayOffset();
/* 2205 */       int beginPos = arrayOffset + position();
/* 2206 */       int limit = arrayOffset + limit();
/* 2207 */       byte[] array = array();
/*      */       
/* 2209 */       for (int i = beginPos; i < limit; i++) {
/* 2210 */         if (array[i] == b) {
/* 2211 */           return i - arrayOffset;
/*      */         }
/*      */       }
/*      */     }
/*      */     else {
/* 2216 */       int beginPos = position();
/* 2217 */       int limit = limit();
/*      */       
/* 2219 */       for (int i = beginPos; i < limit; i++) {
/* 2220 */         if (get(i) == b) {
/* 2221 */           return i;
/*      */         }
/*      */       }
/*      */     }
/*      */     
/* 2226 */     return -1;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer skip(int size)
/*      */   {
/* 2235 */     autoExpand(size);
/* 2236 */     return position(position() + size);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer fill(byte value, int size)
/*      */   {
/* 2245 */     autoExpand(size);
/* 2246 */     int q = size >>> 3;
/* 2247 */     int r = size & 0x7;
/*      */     
/* 2249 */     if (q > 0) {
/* 2250 */       int intValue = value | value << 8 | value << 16 | value << 24;
/* 2251 */       long longValue = intValue;
/* 2252 */       longValue <<= 32;
/* 2253 */       longValue |= intValue;
/*      */       
/* 2255 */       for (int i = q; i > 0; i--) {
/* 2256 */         putLong(longValue);
/*      */       }
/*      */     }
/*      */     
/* 2260 */     q = r >>> 2;
/* 2261 */     r &= 0x3;
/*      */     
/* 2263 */     if (q > 0) {
/* 2264 */       int intValue = value | value << 8 | value << 16 | value << 24;
/* 2265 */       putInt(intValue);
/*      */     }
/*      */     
/* 2268 */     q = r >> 1;
/* 2269 */     r &= 0x1;
/*      */     
/* 2271 */     if (q > 0) {
/* 2272 */       short shortValue = (short)(value | value << 8);
/* 2273 */       putShort(shortValue);
/*      */     }
/*      */     
/* 2276 */     if (r > 0) {
/* 2277 */       put(value);
/*      */     }
/*      */     
/* 2280 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer fillAndReset(byte value, int size)
/*      */   {
/* 2289 */     autoExpand(size);
/* 2290 */     int pos = position();
/*      */     try {
/* 2292 */       fill(value, size);
/*      */     }
/*      */     finally {
/* 2295 */       position(pos);
/*      */     }
/* 2297 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer fill(int size)
/*      */   {
/* 2306 */     autoExpand(size);
/* 2307 */     int q = size >>> 3;
/* 2308 */     int r = size & 0x7;
/*      */     
/* 2310 */     for (int i = q; i > 0; i--) {
/* 2311 */       putLong(0L);
/*      */     }
/*      */     
/* 2314 */     q = r >>> 2;
/* 2315 */     r &= 0x3;
/*      */     
/* 2317 */     if (q > 0) {
/* 2318 */       putInt(0);
/*      */     }
/*      */     
/* 2321 */     q = r >> 1;
/* 2322 */     r &= 0x1;
/*      */     
/* 2324 */     if (q > 0) {
/* 2325 */       putShort((short)0);
/*      */     }
/*      */     
/* 2328 */     if (r > 0) {
/* 2329 */       put((byte)0);
/*      */     }
/*      */     
/* 2332 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer fillAndReset(int size)
/*      */   {
/* 2341 */     autoExpand(size);
/* 2342 */     int pos = position();
/*      */     try {
/* 2344 */       fill(size);
/*      */     }
/*      */     finally {
/* 2347 */       position(pos);
/*      */     }
/*      */     
/* 2350 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> E getEnum(Class<E> enumClass)
/*      */   {
/* 2359 */     return (Enum)toEnum(enumClass, getUnsigned());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> E getEnum(int index, Class<E> enumClass)
/*      */   {
/* 2368 */     return (Enum)toEnum(enumClass, getUnsigned(index));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> E getEnumShort(Class<E> enumClass)
/*      */   {
/* 2377 */     return (Enum)toEnum(enumClass, getUnsignedShort());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> E getEnumShort(int index, Class<E> enumClass)
/*      */   {
/* 2386 */     return (Enum)toEnum(enumClass, getUnsignedShort(index));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> E getEnumInt(Class<E> enumClass)
/*      */   {
/* 2395 */     return (Enum)toEnum(enumClass, getInt());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> E getEnumInt(int index, Class<E> enumClass)
/*      */   {
/* 2404 */     return (Enum)toEnum(enumClass, getInt(index));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer putEnum(Enum<?> e)
/*      */   {
/* 2413 */     if (e.ordinal() > 255L) {
/* 2414 */       throw new IllegalArgumentException(enumConversionErrorMessage(e, "byte"));
/*      */     }
/* 2416 */     return put((byte)e.ordinal());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer putEnum(int index, Enum<?> e)
/*      */   {
/* 2425 */     if (e.ordinal() > 255L) {
/* 2426 */       throw new IllegalArgumentException(enumConversionErrorMessage(e, "byte"));
/*      */     }
/* 2428 */     return put(index, (byte)e.ordinal());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer putEnumShort(Enum<?> e)
/*      */   {
/* 2437 */     if (e.ordinal() > 65535L) {
/* 2438 */       throw new IllegalArgumentException(enumConversionErrorMessage(e, "short"));
/*      */     }
/* 2440 */     return putShort((short)e.ordinal());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer putEnumShort(int index, Enum<?> e)
/*      */   {
/* 2449 */     if (e.ordinal() > 65535L) {
/* 2450 */       throw new IllegalArgumentException(enumConversionErrorMessage(e, "short"));
/*      */     }
/* 2452 */     return putShort(index, (short)e.ordinal());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer putEnumInt(Enum<?> e)
/*      */   {
/* 2461 */     return putInt(e.ordinal());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IoBuffer putEnumInt(int index, Enum<?> e)
/*      */   {
/* 2470 */     return putInt(index, e.ordinal());
/*      */   }
/*      */   
/*      */   private <E> E toEnum(Class<E> enumClass, int i)
/*      */   {
/* 2475 */     E[] enumConstants = enumClass.getEnumConstants();
/* 2476 */     if (i > enumConstants.length) {
/* 2477 */       throw new IndexOutOfBoundsException(String.format("%d is too large of an ordinal to convert to the enum %s", new Object[] { Integer.valueOf(i), enumClass.getName() }));
/*      */     }
/*      */     
/* 2480 */     return enumConstants[i];
/*      */   }
/*      */   
/*      */   private String enumConversionErrorMessage(Enum<?> e, String type)
/*      */   {
/* 2485 */     return String.format("%s.%s has an ordinal value too large for a %s", new Object[] { e.getClass().getName(), e.name(), type });
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> EnumSet<E> getEnumSet(Class<E> enumClass)
/*      */   {
/* 2494 */     return toEnumSet(enumClass, get() & 0xFF);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> EnumSet<E> getEnumSet(int index, Class<E> enumClass)
/*      */   {
/* 2503 */     return toEnumSet(enumClass, get(index) & 0xFF);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> EnumSet<E> getEnumSetShort(Class<E> enumClass)
/*      */   {
/* 2512 */     return toEnumSet(enumClass, getShort() & 0xFFFF);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> EnumSet<E> getEnumSetShort(int index, Class<E> enumClass)
/*      */   {
/* 2521 */     return toEnumSet(enumClass, getShort(index) & 0xFFFF);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> EnumSet<E> getEnumSetInt(Class<E> enumClass)
/*      */   {
/* 2530 */     return toEnumSet(enumClass, getInt() & 0xFFFFFFFF);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> EnumSet<E> getEnumSetInt(int index, Class<E> enumClass)
/*      */   {
/* 2539 */     return toEnumSet(enumClass, getInt(index) & 0xFFFFFFFF);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> EnumSet<E> getEnumSetLong(Class<E> enumClass)
/*      */   {
/* 2548 */     return toEnumSet(enumClass, getLong());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> EnumSet<E> getEnumSetLong(int index, Class<E> enumClass)
/*      */   {
/* 2557 */     return toEnumSet(enumClass, getLong(index));
/*      */   }
/*      */   
/*      */   private <E extends Enum<E>> EnumSet<E> toEnumSet(Class<E> clazz, long vector)
/*      */   {
/* 2562 */     EnumSet<E> set = EnumSet.noneOf(clazz);
/* 2563 */     long mask = 1L;
/* 2564 */     for (E e : (Enum[])clazz.getEnumConstants()) {
/* 2565 */       if ((mask & vector) == mask) {
/* 2566 */         set.add(e);
/*      */       }
/* 2568 */       mask <<= 1;
/*      */     }
/* 2570 */     return set;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> IoBuffer putEnumSet(Set<E> set)
/*      */   {
/* 2579 */     long vector = toLong(set);
/* 2580 */     if ((vector & 0xFFFFFFFFFFFFFF00) != 0L) {
/* 2581 */       throw new IllegalArgumentException("The enum set is too large to fit in a byte: " + set);
/*      */     }
/* 2583 */     return put((byte)(int)vector);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> IoBuffer putEnumSet(int index, Set<E> set)
/*      */   {
/* 2592 */     long vector = toLong(set);
/* 2593 */     if ((vector & 0xFFFFFFFFFFFFFF00) != 0L) {
/* 2594 */       throw new IllegalArgumentException("The enum set is too large to fit in a byte: " + set);
/*      */     }
/* 2596 */     return put(index, (byte)(int)vector);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> IoBuffer putEnumSetShort(Set<E> set)
/*      */   {
/* 2605 */     long vector = toLong(set);
/* 2606 */     if ((vector & 0xFFFFFFFFFFFF0000) != 0L) {
/* 2607 */       throw new IllegalArgumentException("The enum set is too large to fit in a short: " + set);
/*      */     }
/* 2609 */     return putShort((short)(int)vector);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> IoBuffer putEnumSetShort(int index, Set<E> set)
/*      */   {
/* 2618 */     long vector = toLong(set);
/* 2619 */     if ((vector & 0xFFFFFFFFFFFF0000) != 0L) {
/* 2620 */       throw new IllegalArgumentException("The enum set is too large to fit in a short: " + set);
/*      */     }
/* 2622 */     return putShort(index, (short)(int)vector);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> IoBuffer putEnumSetInt(Set<E> set)
/*      */   {
/* 2631 */     long vector = toLong(set);
/* 2632 */     if ((vector & 0xFFFFFFFF00000000) != 0L) {
/* 2633 */       throw new IllegalArgumentException("The enum set is too large to fit in an int: " + set);
/*      */     }
/* 2635 */     return putInt((int)vector);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> IoBuffer putEnumSetInt(int index, Set<E> set)
/*      */   {
/* 2644 */     long vector = toLong(set);
/* 2645 */     if ((vector & 0xFFFFFFFF00000000) != 0L) {
/* 2646 */       throw new IllegalArgumentException("The enum set is too large to fit in an int: " + set);
/*      */     }
/* 2648 */     return putInt(index, (int)vector);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> IoBuffer putEnumSetLong(Set<E> set)
/*      */   {
/* 2657 */     return putLong(toLong(set));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <E extends Enum<E>> IoBuffer putEnumSetLong(int index, Set<E> set)
/*      */   {
/* 2666 */     return putLong(index, toLong(set));
/*      */   }
/*      */   
/*      */   private <E extends Enum<E>> long toLong(Set<E> set)
/*      */   {
/* 2671 */     long vector = 0L;
/* 2672 */     for (E e : set) {
/* 2673 */       if (e.ordinal() >= 64) {
/* 2674 */         throw new IllegalArgumentException("The enum set is too large to fit in a bit vector: " + set);
/*      */       }
/* 2676 */       vector |= 1L << e.ordinal();
/*      */     }
/* 2678 */     return vector;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IoBuffer autoExpand(int expectedRemaining)
/*      */   {
/* 2687 */     if (isAutoExpand()) {
/* 2688 */       expand(expectedRemaining, true);
/*      */     }
/* 2690 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IoBuffer autoExpand(int pos, int expectedRemaining)
/*      */   {
/* 2699 */     if (isAutoExpand()) {
/* 2700 */       expand(pos, expectedRemaining, true);
/*      */     }
/* 2702 */     return this;
/*      */   }
/*      */   
/*      */   private static void checkFieldSize(int fieldSize)
/*      */   {
/* 2707 */     if (fieldSize < 0) {
/* 2708 */       throw new IllegalArgumentException("fieldSize cannot be negative: " + fieldSize);
/*      */     }
/*      */   }
/*      */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\google\code\yanf4j\buffer\AbstractIoBuffer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */