/*      */ package com.googlecode.concurrentlinkedhashmap;
/*      */ 
/*      */ import java.io.InvalidObjectException;
/*      */ import java.io.ObjectInputStream;
/*      */ import java.io.Serializable;
/*      */ import java.util.AbstractCollection;
/*      */ import java.util.AbstractMap;
/*      */ import java.util.AbstractMap.SimpleEntry;
/*      */ import java.util.AbstractQueue;
/*      */ import java.util.AbstractSet;
/*      */ import java.util.Collection;
/*      */ import java.util.Collections;
/*      */ import java.util.HashMap;
/*      */ import java.util.Iterator;
/*      */ import java.util.LinkedHashMap;
/*      */ import java.util.LinkedHashSet;
/*      */ import java.util.List;
/*      */ import java.util.Map;
/*      */ import java.util.Map.Entry;
/*      */ import java.util.Queue;
/*      */ import java.util.Set;
/*      */ import java.util.concurrent.ConcurrentHashMap;
/*      */ import java.util.concurrent.ConcurrentLinkedQueue;
/*      */ import java.util.concurrent.ConcurrentMap;
/*      */ import java.util.concurrent.atomic.AtomicIntegerArray;
/*      */ import java.util.concurrent.atomic.AtomicReference;
/*      */ import java.util.concurrent.locks.Lock;
/*      */ import java.util.concurrent.locks.ReentrantLock;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ @ThreadSafe
/*      */ public final class ConcurrentLinkedHashMap<K, V>
/*      */   extends AbstractMap<K, V>
/*      */   implements ConcurrentMap<K, V>, Serializable
/*      */ {
/*      */   static final long MAXIMUM_CAPACITY = 9223372034707292160L;
/*      */   static final int MAXIMUM_BUFFER_SIZE = 1048576;
/*      */   static final int BUFFER_THRESHOLD = 16;
/*      */   static final int NUMBER_OF_BUFFERS;
/*      */   static final int BUFFER_MASK;
/*      */   static final int AMORTIZED_DRAIN_THRESHOLD;
/*  160 */   static final Queue<?> DISCARDING_QUEUE = new DiscardingQueue();
/*      */   final ConcurrentMap<K, ConcurrentLinkedHashMap<K, V>.Node> data;
/*      */   
/*  163 */   static { int buffers = ceilingNextPowerOfTwo(Runtime.getRuntime().availableProcessors());
/*  164 */     AMORTIZED_DRAIN_THRESHOLD = (1 + buffers) * 16;
/*  165 */     NUMBER_OF_BUFFERS = buffers;
/*  166 */     BUFFER_MASK = buffers - 1;
/*      */   }
/*      */   
/*      */   static int ceilingNextPowerOfTwo(int x)
/*      */   {
/*  171 */     return 1 << 32 - Integer.numberOfLeadingZeros(x - 1);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   static enum DrainStatus
/*      */   {
/*  178 */     IDLE, 
/*      */     
/*      */ 
/*  181 */     REQUIRED, 
/*      */     
/*      */ 
/*  184 */     PROCESSING;
/*      */     
/*      */ 
/*      */     private DrainStatus() {}
/*      */   }
/*      */   
/*      */ 
/*      */   final int concurrencyLevel;
/*      */   
/*      */   @GuardedBy("evictionLock")
/*      */   final LinkedDeque<ConcurrentLinkedHashMap<K, V>.Node> evictionDeque;
/*      */   
/*      */   @GuardedBy("evictionLock")
/*      */   volatile long weightedSize;
/*      */   
/*      */   @GuardedBy("evictionLock")
/*      */   volatile long capacity;
/*      */   
/*      */   volatile int nextOrder;
/*      */   
/*      */   @GuardedBy("evictionLock")
/*      */   int drainedOrder;
/*      */   
/*      */   final Lock evictionLock;
/*      */   
/*      */   final Queue<Task>[] buffers;
/*      */   
/*      */   final AtomicIntegerArray bufferLengths;
/*      */   
/*      */   final AtomicReference<DrainStatus> drainStatus;
/*      */   
/*      */   final EntryWeigher<? super K, ? super V> weigher;
/*      */   final Queue<ConcurrentLinkedHashMap<K, V>.Node> pendingNotifications;
/*      */   final EvictionListener<K, V> listener;
/*      */   transient Set<K> keySet;
/*      */   transient Collection<V> values;
/*      */   transient Set<Map.Entry<K, V>> entrySet;
/*      */   static final long serialVersionUID = 1L;
/*      */   private ConcurrentLinkedHashMap(Builder<K, V> builder)
/*      */   {
/*  224 */     this.concurrencyLevel = builder.concurrencyLevel;
/*  225 */     this.capacity = Math.min(builder.capacity, 9223372034707292160L);
/*  226 */     this.data = new ConcurrentHashMap(builder.initialCapacity, 0.75F, this.concurrencyLevel);
/*      */     
/*      */ 
/*  229 */     this.weigher = builder.weigher;
/*  230 */     this.nextOrder = Integer.MIN_VALUE;
/*  231 */     this.drainedOrder = Integer.MIN_VALUE;
/*  232 */     this.evictionLock = new ReentrantLock();
/*  233 */     this.evictionDeque = new LinkedDeque();
/*  234 */     this.drainStatus = new AtomicReference(DrainStatus.IDLE);
/*      */     
/*  236 */     this.buffers = ((Queue[])new Queue[NUMBER_OF_BUFFERS]);
/*  237 */     this.bufferLengths = new AtomicIntegerArray(NUMBER_OF_BUFFERS);
/*  238 */     for (int i = 0; i < NUMBER_OF_BUFFERS; i++) {
/*  239 */       this.buffers[i] = new ConcurrentLinkedQueue();
/*      */     }
/*      */     
/*      */ 
/*  243 */     this.listener = builder.listener;
/*  244 */     this.pendingNotifications = (this.listener == DiscardingListener.INSTANCE ? DISCARDING_QUEUE : new ConcurrentLinkedQueue());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   static void checkNotNull(Object o)
/*      */   {
/*  251 */     if (o == null) {
/*  252 */       throw new NullPointerException();
/*      */     }
/*      */   }
/*      */   
/*      */   static void checkArgument(boolean expression)
/*      */   {
/*  258 */     if (!expression) {
/*  259 */       throw new IllegalArgumentException();
/*      */     }
/*      */   }
/*      */   
/*      */   static void checkState(boolean expression)
/*      */   {
/*  265 */     if (!expression) {
/*  266 */       throw new IllegalStateException();
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public long capacity()
/*      */   {
/*  278 */     return this.capacity;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void setCapacity(long capacity)
/*      */   {
/*  289 */     checkArgument(capacity >= 0L);
/*  290 */     this.evictionLock.lock();
/*      */     try {
/*  292 */       this.capacity = Math.min(capacity, 9223372034707292160L);
/*  293 */       drainBuffers(AMORTIZED_DRAIN_THRESHOLD);
/*  294 */       evict();
/*      */     } finally {
/*  296 */       this.evictionLock.unlock();
/*      */     }
/*  298 */     notifyListener();
/*      */   }
/*      */   
/*      */   boolean hasOverflowed()
/*      */   {
/*  303 */     return this.weightedSize > this.capacity;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   @GuardedBy("evictionLock")
/*      */   void evict()
/*      */   {
/*  318 */     while (hasOverflowed()) {
/*  319 */       ConcurrentLinkedHashMap<K, V>.Node node = (Node)this.evictionDeque.poll();
/*      */       
/*      */ 
/*      */ 
/*  323 */       if (node == null) {
/*  324 */         return;
/*      */       }
/*      */       
/*      */ 
/*  328 */       if (this.data.remove(node.key, node)) {
/*  329 */         this.pendingNotifications.add(node);
/*      */       }
/*      */       
/*  332 */       node.makeDead();
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   void afterCompletion(Task task)
/*      */   {
/*  342 */     boolean delayable = schedule(task);
/*  343 */     if (shouldDrainBuffers(delayable)) {
/*  344 */       tryToDrainBuffers(AMORTIZED_DRAIN_THRESHOLD);
/*      */     }
/*  346 */     notifyListener();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   boolean schedule(Task task)
/*      */   {
/*  356 */     int index = bufferIndex();
/*  357 */     int buffered = this.bufferLengths.incrementAndGet(index);
/*      */     
/*  359 */     if (task.isWrite()) {
/*  360 */       this.buffers[index].add(task);
/*  361 */       this.drainStatus.set(DrainStatus.REQUIRED);
/*  362 */       return false;
/*      */     }
/*      */     
/*      */ 
/*  366 */     if (buffered <= 1048576) {
/*  367 */       this.buffers[index].add(task);
/*  368 */       return buffered <= 16;
/*      */     }
/*  370 */     this.bufferLengths.decrementAndGet(index);
/*  371 */     return false;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   static int bufferIndex()
/*      */   {
/*  380 */     return (int)Thread.currentThread().getId() & BUFFER_MASK;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   int nextOrdering()
/*      */   {
/*  388 */     return this.nextOrder++;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   boolean shouldDrainBuffers(boolean delayable)
/*      */   {
/*  398 */     DrainStatus status = (DrainStatus)this.drainStatus.get();
/*  399 */     return (status != DrainStatus.PROCESSING ? 1 : 0) & ((!delayable ? 1 : 0) | (status == DrainStatus.REQUIRED ? 1 : 0));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   void tryToDrainBuffers(int maxToDrain)
/*      */   {
/*  409 */     if (this.evictionLock.tryLock()) {
/*      */       try {
/*  411 */         this.drainStatus.set(DrainStatus.PROCESSING);
/*  412 */         drainBuffers(maxToDrain);
/*      */       } finally {
/*  414 */         this.drainStatus.compareAndSet(DrainStatus.PROCESSING, DrainStatus.IDLE);
/*  415 */         this.evictionLock.unlock();
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   @GuardedBy("evictionLock")
/*      */   void drainBuffers(int maxToDrain)
/*      */   {
/*  434 */     Task[] tasks = new Task[maxToDrain];
/*      */     
/*      */ 
/*      */ 
/*  438 */     int maxTaskIndex = moveTasksFromBuffers(tasks);
/*  439 */     runTasks(tasks, maxTaskIndex);
/*  440 */     updateDrainedOrder(tasks, maxTaskIndex);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   @GuardedBy("evictionLock")
/*      */   int moveTasksFromBuffers(Task[] tasks)
/*      */   {
/*  451 */     int maxTaskIndex = -1;
/*  452 */     for (int i = 0; i < this.buffers.length; i++) {
/*  453 */       int maxIndex = moveTasksFromBuffer(tasks, i);
/*  454 */       maxTaskIndex = Math.max(maxIndex, maxTaskIndex);
/*      */     }
/*  456 */     return maxTaskIndex;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   @GuardedBy("evictionLock")
/*      */   int moveTasksFromBuffer(Task[] tasks, int bufferIndex)
/*      */   {
/*  471 */     Queue<Task> buffer = this.buffers[bufferIndex];
/*  472 */     int removedFromBuffer = 0;
/*      */     
/*      */ 
/*  475 */     int maxIndex = -1;
/*  476 */     Task task; while ((task = (Task)buffer.poll()) != null) {
/*  477 */       removedFromBuffer++;
/*      */       
/*      */ 
/*      */ 
/*  481 */       int index = task.getOrder() - this.drainedOrder;
/*  482 */       if (index < 0)
/*      */       {
/*  484 */         task.run();
/*  485 */       } else { if (index >= tasks.length)
/*      */         {
/*      */ 
/*      */ 
/*  489 */           maxIndex = tasks.length - 1;
/*  490 */           addTaskToChain(tasks, task, maxIndex);
/*  491 */           break;
/*      */         }
/*  493 */         maxIndex = Math.max(index, maxIndex);
/*  494 */         addTaskToChain(tasks, task, index);
/*      */       }
/*      */     }
/*  497 */     this.bufferLengths.addAndGet(bufferIndex, -removedFromBuffer);
/*  498 */     return maxIndex;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   @GuardedBy("evictionLock")
/*      */   void addTaskToChain(Task[] tasks, Task task, int index)
/*      */   {
/*  510 */     task.setNext(tasks[index]);
/*  511 */     tasks[index] = task;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   @GuardedBy("evictionLock")
/*      */   void runTasks(Task[] tasks, int maxTaskIndex)
/*      */   {
/*  522 */     for (int i = 0; i <= maxTaskIndex; i++) {
/*  523 */       runTasksInChain(tasks[i]);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   @GuardedBy("evictionLock")
/*      */   void runTasksInChain(Task task)
/*      */   {
/*  534 */     while (task != null) {
/*  535 */       Task current = task;
/*  536 */       task = task.getNext();
/*  537 */       current.setNext(null);
/*  538 */       current.run();
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   @GuardedBy("evictionLock")
/*      */   void updateDrainedOrder(Task[] tasks, int maxTaskIndex)
/*      */   {
/*  550 */     if (maxTaskIndex >= 0) {
/*  551 */       Task task = tasks[maxTaskIndex];
/*  552 */       this.drainedOrder = (task.getOrder() + 1);
/*      */     }
/*      */   }
/*      */   
/*      */   void notifyListener()
/*      */   {
/*      */     ConcurrentLinkedHashMap<K, V>.Node node;
/*  559 */     while ((node = (Node)this.pendingNotifications.poll()) != null) {
/*  560 */       this.listener.onEviction(node.key, node.getValue());
/*      */     }
/*      */   }
/*      */   
/*      */   class ReadTask extends ConcurrentLinkedHashMap.AbstractTask {
/*      */     final ConcurrentLinkedHashMap<K, V>.Node node;
/*      */     
/*      */     ReadTask() {
/*  568 */       super();
/*  569 */       this.node = node;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     @GuardedBy("evictionLock")
/*      */     public void run()
/*      */     {
/*  579 */       if (ConcurrentLinkedHashMap.this.evictionDeque.contains(this.node)) {
/*  580 */         ConcurrentLinkedHashMap.this.evictionDeque.moveToBack(this.node);
/*      */       }
/*      */     }
/*      */     
/*      */     public boolean isWrite()
/*      */     {
/*  586 */       return false;
/*      */     }
/*      */   }
/*      */   
/*      */   final class AddTask extends ConcurrentLinkedHashMap.AbstractTask {
/*      */     final ConcurrentLinkedHashMap<K, V>.Node node;
/*      */     final int weight;
/*      */     
/*      */     AddTask(int node) {
/*  595 */       super();
/*  596 */       this.weight = weight;
/*  597 */       this.node = node;
/*      */     }
/*      */     
/*      */     @GuardedBy("evictionLock")
/*      */     public void run()
/*      */     {
/*  603 */       ConcurrentLinkedHashMap.this.weightedSize += this.weight;
/*      */       
/*      */ 
/*  606 */       if (((ConcurrentLinkedHashMap.WeightedValue)this.node.get()).isAlive()) {
/*  607 */         ConcurrentLinkedHashMap.this.evictionDeque.add(this.node);
/*  608 */         ConcurrentLinkedHashMap.this.evict();
/*      */       }
/*      */     }
/*      */     
/*      */     public boolean isWrite()
/*      */     {
/*  614 */       return true;
/*      */     }
/*      */   }
/*      */   
/*      */   final class RemovalTask extends ConcurrentLinkedHashMap.AbstractTask {
/*      */     final ConcurrentLinkedHashMap<K, V>.Node node;
/*      */     
/*      */     RemovalTask() {
/*  622 */       super();
/*  623 */       this.node = node;
/*      */     }
/*      */     
/*      */ 
/*      */     @GuardedBy("evictionLock")
/*      */     public void run()
/*      */     {
/*  630 */       ConcurrentLinkedHashMap.this.evictionDeque.remove(this.node);
/*  631 */       this.node.makeDead();
/*      */     }
/*      */     
/*      */     public boolean isWrite()
/*      */     {
/*  636 */       return true;
/*      */     }
/*      */   }
/*      */   
/*      */   final class UpdateTask extends ConcurrentLinkedHashMap.ReadTask
/*      */   {
/*      */     final int weightDifference;
/*      */     
/*      */     public UpdateTask(int node) {
/*  645 */       super(node);
/*  646 */       this.weightDifference = weightDifference;
/*      */     }
/*      */     
/*      */     @GuardedBy("evictionLock")
/*      */     public void run()
/*      */     {
/*  652 */       super.run();
/*  653 */       ConcurrentLinkedHashMap.this.weightedSize += this.weightDifference;
/*  654 */       ConcurrentLinkedHashMap.this.evict();
/*      */     }
/*      */     
/*      */     public boolean isWrite()
/*      */     {
/*  659 */       return true;
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public boolean isEmpty()
/*      */   {
/*  667 */     return this.data.isEmpty();
/*      */   }
/*      */   
/*      */   public int size()
/*      */   {
/*  672 */     return this.data.size();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public long weightedSize()
/*      */   {
/*  681 */     return Math.max(0L, this.weightedSize);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public void clear()
/*      */   {
/*  688 */     this.evictionLock.lock();
/*      */     try {
/*      */       ConcurrentLinkedHashMap<K, V>.Node node;
/*  691 */       while ((node = (Node)this.evictionDeque.poll()) != null) {
/*  692 */         this.data.remove(node.key, node);
/*  693 */         node.makeDead();
/*      */       }
/*      */       
/*      */ 
/*  697 */       for (int i = 0; i < this.buffers.length; i++) {
/*  698 */         Queue<Task> buffer = this.buffers[i];
/*  699 */         int removed = 0;
/*      */         Task task;
/*  701 */         while ((task = (Task)buffer.poll()) != null) {
/*  702 */           if (task.isWrite()) {
/*  703 */             task.run();
/*      */           }
/*  705 */           removed++;
/*      */         }
/*  707 */         this.bufferLengths.addAndGet(i, -removed);
/*      */       }
/*      */     } finally {
/*  710 */       this.evictionLock.unlock();
/*      */     }
/*      */   }
/*      */   
/*      */   public boolean containsKey(Object key)
/*      */   {
/*  716 */     return this.data.containsKey(key);
/*      */   }
/*      */   
/*      */   public boolean containsValue(Object value)
/*      */   {
/*  721 */     checkNotNull(value);
/*      */     
/*  723 */     for (ConcurrentLinkedHashMap<K, V>.Node node : this.data.values()) {
/*  724 */       if (node.getValue().equals(value)) {
/*  725 */         return true;
/*      */       }
/*      */     }
/*  728 */     return false;
/*      */   }
/*      */   
/*      */   public V get(Object key)
/*      */   {
/*  733 */     ConcurrentLinkedHashMap<K, V>.Node node = (Node)this.data.get(key);
/*  734 */     if (node == null) {
/*  735 */       return null;
/*      */     }
/*  737 */     afterCompletion(new ReadTask(node));
/*  738 */     return (V)node.getValue();
/*      */   }
/*      */   
/*      */   public V put(K key, V value)
/*      */   {
/*  743 */     return (V)put(key, value, false);
/*      */   }
/*      */   
/*      */   public V putIfAbsent(K key, V value)
/*      */   {
/*  748 */     return (V)put(key, value, true);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   V put(K key, V value, boolean onlyIfAbsent)
/*      */   {
/*  762 */     checkNotNull(value);
/*      */     
/*  764 */     int weight = this.weigher.weightOf(key, value);
/*  765 */     WeightedValue<V> weightedValue = new WeightedValue(value, weight);
/*  766 */     ConcurrentLinkedHashMap<K, V>.Node node = new Node(key, weightedValue);
/*      */     for (;;)
/*      */     {
/*  769 */       ConcurrentLinkedHashMap<K, V>.Node prior = (Node)this.data.putIfAbsent(node.key, node);
/*  770 */       if (prior == null) {
/*  771 */         afterCompletion(new AddTask(node, weight));
/*  772 */         return null; }
/*  773 */       if (onlyIfAbsent) {
/*  774 */         afterCompletion(new ReadTask(prior));
/*  775 */         return (V)prior.getValue();
/*      */       }
/*      */       for (;;) {
/*  778 */         WeightedValue<V> oldWeightedValue = (WeightedValue)prior.get();
/*  779 */         if (!oldWeightedValue.isAlive()) {
/*      */           break;
/*      */         }
/*      */         
/*  783 */         if (prior.compareAndSet(oldWeightedValue, weightedValue)) {
/*  784 */           int weightedDifference = weight - oldWeightedValue.weight;
/*  785 */           Task task = weightedDifference == 0 ? new ReadTask(prior) : new UpdateTask(prior, weightedDifference);
/*      */           
/*      */ 
/*  788 */           afterCompletion(task);
/*  789 */           return (V)oldWeightedValue.value;
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   public V remove(Object key)
/*      */   {
/*  797 */     ConcurrentLinkedHashMap<K, V>.Node node = (Node)this.data.remove(key);
/*  798 */     if (node == null) {
/*  799 */       return null;
/*      */     }
/*      */     
/*  802 */     node.makeRetired();
/*  803 */     afterCompletion(new RemovalTask(node));
/*  804 */     return (V)node.getValue();
/*      */   }
/*      */   
/*      */   public boolean remove(Object key, Object value)
/*      */   {
/*  809 */     ConcurrentLinkedHashMap<K, V>.Node node = (Node)this.data.get(key);
/*  810 */     if ((node == null) || (value == null)) {
/*  811 */       return false;
/*      */     }
/*      */     
/*  814 */     WeightedValue<V> weightedValue = (WeightedValue)node.get();
/*      */     do {
/*  816 */       if (!weightedValue.hasValue(value)) break;
/*  817 */       if (node.tryToRetire(weightedValue)) {
/*  818 */         if (!this.data.remove(key, node)) break;
/*  819 */         afterCompletion(new RemovalTask(node));
/*  820 */         return true;
/*      */       }
/*      */       
/*  823 */       weightedValue = (WeightedValue)node.get();
/*  824 */     } while (weightedValue.isAlive());
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  831 */     return false;
/*      */   }
/*      */   
/*      */ 
/*      */   public V replace(K key, V value)
/*      */   {
/*  837 */     checkNotNull(value);
/*      */     
/*  839 */     int weight = this.weigher.weightOf(key, value);
/*  840 */     WeightedValue<V> weightedValue = new WeightedValue(value, weight);
/*      */     
/*  842 */     ConcurrentLinkedHashMap<K, V>.Node node = (Node)this.data.get(key);
/*  843 */     if (node == null) {
/*  844 */       return null;
/*      */     }
/*      */     for (;;) {
/*  847 */       WeightedValue<V> oldWeightedValue = (WeightedValue)node.get();
/*  848 */       if (!oldWeightedValue.isAlive()) {
/*  849 */         return null;
/*      */       }
/*  851 */       if (node.compareAndSet(oldWeightedValue, weightedValue)) {
/*  852 */         int weightedDifference = weight - oldWeightedValue.weight;
/*  853 */         Task task = weightedDifference == 0 ? new ReadTask(node) : new UpdateTask(node, weightedDifference);
/*      */         
/*      */ 
/*  856 */         afterCompletion(task);
/*  857 */         return (V)oldWeightedValue.value;
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   public boolean replace(K key, V oldValue, V newValue)
/*      */   {
/*  864 */     checkNotNull(oldValue);
/*  865 */     checkNotNull(newValue);
/*      */     
/*  867 */     int weight = this.weigher.weightOf(key, newValue);
/*  868 */     WeightedValue<V> newWeightedValue = new WeightedValue(newValue, weight);
/*      */     
/*  870 */     ConcurrentLinkedHashMap<K, V>.Node node = (Node)this.data.get(key);
/*  871 */     if (node == null) {
/*  872 */       return false;
/*      */     }
/*      */     for (;;) {
/*  875 */       WeightedValue<V> weightedValue = (WeightedValue)node.get();
/*  876 */       if ((!weightedValue.isAlive()) || (!weightedValue.hasValue(oldValue))) {
/*  877 */         return false;
/*      */       }
/*  879 */       if (node.compareAndSet(weightedValue, newWeightedValue)) {
/*  880 */         int weightedDifference = weight - weightedValue.weight;
/*  881 */         Task task = weightedDifference == 0 ? new ReadTask(node) : new UpdateTask(node, weightedDifference);
/*      */         
/*      */ 
/*  884 */         afterCompletion(task);
/*  885 */         return true;
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   public Set<K> keySet()
/*      */   {
/*  892 */     Set<K> ks = this.keySet;
/*  893 */     return ks == null ? (this.keySet = new KeySet()) : ks;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Set<K> ascendingKeySet()
/*      */   {
/*  910 */     return orderedKeySet(true, Integer.MAX_VALUE);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Set<K> ascendingKeySetWithLimit(int limit)
/*      */   {
/*  929 */     return orderedKeySet(true, limit);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Set<K> descendingKeySet()
/*      */   {
/*  946 */     return orderedKeySet(false, Integer.MAX_VALUE);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Set<K> descendingKeySetWithLimit(int limit)
/*      */   {
/*  965 */     return orderedKeySet(false, limit);
/*      */   }
/*      */   
/*      */   Set<K> orderedKeySet(boolean ascending, int limit) {
/*  969 */     checkArgument(limit >= 0);
/*  970 */     this.evictionLock.lock();
/*      */     try {
/*  972 */       drainBuffers(AMORTIZED_DRAIN_THRESHOLD);
/*      */       
/*  974 */       int initialCapacity = this.weigher == Weighers.singleton() ? Math.min(limit, (int)weightedSize()) : 16;
/*      */       
/*      */ 
/*  977 */       Set<K> keys = new LinkedHashSet(initialCapacity);
/*  978 */       Iterator<ConcurrentLinkedHashMap<K, V>.Node> iterator = ascending ? this.evictionDeque.iterator() : this.evictionDeque.descendingIterator();
/*      */       
/*      */ 
/*  981 */       while ((iterator.hasNext()) && (limit > keys.size())) {
/*  982 */         keys.add(((Node)iterator.next()).key);
/*      */       }
/*  984 */       return Collections.unmodifiableSet(keys);
/*      */     } finally {
/*  986 */       this.evictionLock.unlock();
/*      */     }
/*      */   }
/*      */   
/*      */   public Collection<V> values()
/*      */   {
/*  992 */     Collection<V> vs = this.values;
/*  993 */     return vs == null ? (this.values = new Values()) : vs;
/*      */   }
/*      */   
/*      */   public Set<Map.Entry<K, V>> entrySet()
/*      */   {
/*  998 */     Set<Map.Entry<K, V>> es = this.entrySet;
/*  999 */     return es == null ? (this.entrySet = new EntrySet()) : es;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Map<K, V> ascendingMap()
/*      */   {
/* 1017 */     return orderedMap(true, Integer.MAX_VALUE);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Map<K, V> ascendingMapWithLimit(int limit)
/*      */   {
/* 1037 */     return orderedMap(true, limit);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Map<K, V> descendingMap()
/*      */   {
/* 1055 */     return orderedMap(false, Integer.MAX_VALUE);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Map<K, V> descendingMapWithLimit(int limit)
/*      */   {
/* 1075 */     return orderedMap(false, limit);
/*      */   }
/*      */   
/*      */   Map<K, V> orderedMap(boolean ascending, int limit) {
/* 1079 */     checkArgument(limit >= 0);
/* 1080 */     this.evictionLock.lock();
/*      */     try {
/* 1082 */       drainBuffers(AMORTIZED_DRAIN_THRESHOLD);
/*      */       
/* 1084 */       int initialCapacity = this.weigher == Weighers.singleton() ? Math.min(limit, (int)weightedSize()) : 16;
/*      */       
/*      */ 
/* 1087 */       Map<K, V> map = new LinkedHashMap(initialCapacity);
/* 1088 */       Iterator<ConcurrentLinkedHashMap<K, V>.Node> iterator = ascending ? this.evictionDeque.iterator() : this.evictionDeque.descendingIterator();
/*      */       
/*      */       ConcurrentLinkedHashMap<K, V>.Node node;
/* 1091 */       while ((iterator.hasNext()) && (limit > map.size())) {
/* 1092 */         node = (Node)iterator.next();
/* 1093 */         map.put(node.key, node.getValue());
/*      */       }
/* 1095 */       return Collections.unmodifiableMap(map);
/*      */     } finally {
/* 1097 */       this.evictionLock.unlock();
/*      */     }
/*      */   }
/*      */   
/*      */   @Immutable
/*      */   static final class WeightedValue<V>
/*      */   {
/*      */     final int weight;
/*      */     final V value;
/*      */     
/*      */     WeightedValue(V value, int weight) {
/* 1108 */       this.weight = weight;
/* 1109 */       this.value = value;
/*      */     }
/*      */     
/*      */     boolean hasValue(Object o) {
/* 1113 */       return (o == this.value) || (this.value.equals(o));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */     boolean isAlive()
/*      */     {
/* 1120 */       return this.weight > 0;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */     boolean isRetired()
/*      */     {
/* 1128 */       return this.weight < 0;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */     boolean isDead()
/*      */     {
/* 1136 */       return this.weight == 0;
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   final class Node
/*      */     extends AtomicReference<ConcurrentLinkedHashMap.WeightedValue<V>>
/*      */     implements Linked<ConcurrentLinkedHashMap<K, V>.Node>
/*      */   {
/*      */     final K key;
/*      */     
/*      */     @GuardedBy("evictionLock")
/*      */     ConcurrentLinkedHashMap<K, V>.Node prev;
/*      */     @GuardedBy("evictionLock")
/*      */     ConcurrentLinkedHashMap<K, V>.Node next;
/*      */     
/*      */     Node(ConcurrentLinkedHashMap.WeightedValue<V> key)
/*      */     {
/* 1154 */       super();
/* 1155 */       this.key = key;
/*      */     }
/*      */     
/*      */     @GuardedBy("evictionLock")
/*      */     public ConcurrentLinkedHashMap<K, V>.Node getPrevious()
/*      */     {
/* 1161 */       return this.prev;
/*      */     }
/*      */     
/*      */     @GuardedBy("evictionLock")
/*      */     public void setPrevious(ConcurrentLinkedHashMap<K, V>.Node prev)
/*      */     {
/* 1167 */       this.prev = prev;
/*      */     }
/*      */     
/*      */     @GuardedBy("evictionLock")
/*      */     public ConcurrentLinkedHashMap<K, V>.Node getNext()
/*      */     {
/* 1173 */       return this.next;
/*      */     }
/*      */     
/*      */     @GuardedBy("evictionLock")
/*      */     public void setNext(ConcurrentLinkedHashMap<K, V>.Node next)
/*      */     {
/* 1179 */       this.next = next;
/*      */     }
/*      */     
/*      */     V getValue()
/*      */     {
/* 1184 */       return (V)((ConcurrentLinkedHashMap.WeightedValue)get()).value;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     boolean tryToRetire(ConcurrentLinkedHashMap.WeightedValue<V> expect)
/*      */     {
/* 1195 */       if (expect.isAlive()) {
/* 1196 */         ConcurrentLinkedHashMap.WeightedValue<V> retired = new ConcurrentLinkedHashMap.WeightedValue(expect.value, -expect.weight);
/* 1197 */         return compareAndSet(expect, retired);
/*      */       }
/* 1199 */       return false;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */     void makeRetired()
/*      */     {
/*      */       for (;;)
/*      */       {
/* 1208 */         ConcurrentLinkedHashMap.WeightedValue<V> current = (ConcurrentLinkedHashMap.WeightedValue)get();
/* 1209 */         if (!current.isAlive()) {
/* 1210 */           return;
/*      */         }
/* 1212 */         ConcurrentLinkedHashMap.WeightedValue<V> retired = new ConcurrentLinkedHashMap.WeightedValue(current.value, -current.weight);
/* 1213 */         if (compareAndSet(current, retired)) {
/* 1214 */           return;
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */     @GuardedBy("evictionLock")
/*      */     void makeDead()
/*      */     {
/*      */       for (;;)
/*      */       {
/* 1226 */         ConcurrentLinkedHashMap.WeightedValue<V> current = (ConcurrentLinkedHashMap.WeightedValue)get();
/* 1227 */         ConcurrentLinkedHashMap.WeightedValue<V> dead = new ConcurrentLinkedHashMap.WeightedValue(current.value, 0);
/* 1228 */         if (compareAndSet(current, dead)) {
/* 1229 */           ConcurrentLinkedHashMap.this.weightedSize -= Math.abs(current.weight);
/* 1230 */           return;
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   final class KeySet extends AbstractSet<K>
/*      */   {
/* 1238 */     final ConcurrentLinkedHashMap<K, V> map = ConcurrentLinkedHashMap.this;
/*      */     
/*      */     KeySet() {}
/*      */     
/* 1242 */     public int size() { return this.map.size(); }
/*      */     
/*      */ 
/*      */     public void clear()
/*      */     {
/* 1247 */       this.map.clear();
/*      */     }
/*      */     
/*      */     public Iterator<K> iterator()
/*      */     {
/* 1252 */       return new ConcurrentLinkedHashMap.KeyIterator(ConcurrentLinkedHashMap.this);
/*      */     }
/*      */     
/*      */     public boolean contains(Object obj)
/*      */     {
/* 1257 */       return ConcurrentLinkedHashMap.this.containsKey(obj);
/*      */     }
/*      */     
/*      */     public boolean remove(Object obj)
/*      */     {
/* 1262 */       return this.map.remove(obj) != null;
/*      */     }
/*      */     
/*      */     public Object[] toArray()
/*      */     {
/* 1267 */       return this.map.data.keySet().toArray();
/*      */     }
/*      */     
/*      */     public <T> T[] toArray(T[] array)
/*      */     {
/* 1272 */       return this.map.data.keySet().toArray(array);
/*      */     }
/*      */   }
/*      */   
/*      */   final class KeyIterator implements Iterator<K>
/*      */   {
/* 1278 */     final Iterator<K> iterator = ConcurrentLinkedHashMap.this.data.keySet().iterator();
/*      */     K current;
/*      */     
/*      */     KeyIterator() {}
/*      */     
/* 1283 */     public boolean hasNext() { return this.iterator.hasNext(); }
/*      */     
/*      */ 
/*      */     public K next()
/*      */     {
/* 1288 */       this.current = this.iterator.next();
/* 1289 */       return (K)this.current;
/*      */     }
/*      */     
/*      */     public void remove()
/*      */     {
/* 1294 */       ConcurrentLinkedHashMap.checkState(this.current != null);
/* 1295 */       ConcurrentLinkedHashMap.this.remove(this.current);
/* 1296 */       this.current = null;
/*      */     }
/*      */   }
/*      */   
/*      */   final class Values extends AbstractCollection<V>
/*      */   {
/*      */     Values() {}
/*      */     
/*      */     public int size() {
/* 1305 */       return ConcurrentLinkedHashMap.this.size();
/*      */     }
/*      */     
/*      */     public void clear()
/*      */     {
/* 1310 */       ConcurrentLinkedHashMap.this.clear();
/*      */     }
/*      */     
/*      */     public Iterator<V> iterator()
/*      */     {
/* 1315 */       return new ConcurrentLinkedHashMap.ValueIterator(ConcurrentLinkedHashMap.this);
/*      */     }
/*      */     
/*      */     public boolean contains(Object o)
/*      */     {
/* 1320 */       return ConcurrentLinkedHashMap.this.containsValue(o);
/*      */     }
/*      */   }
/*      */   
/*      */   final class ValueIterator implements Iterator<V>
/*      */   {
/* 1326 */     final Iterator<ConcurrentLinkedHashMap<K, V>.Node> iterator = ConcurrentLinkedHashMap.this.data.values().iterator();
/*      */     ConcurrentLinkedHashMap<K, V>.Node current;
/*      */     
/*      */     ValueIterator() {}
/*      */     
/* 1331 */     public boolean hasNext() { return this.iterator.hasNext(); }
/*      */     
/*      */ 
/*      */     public V next()
/*      */     {
/* 1336 */       this.current = ((ConcurrentLinkedHashMap.Node)this.iterator.next());
/* 1337 */       return (V)this.current.getValue();
/*      */     }
/*      */     
/*      */     public void remove()
/*      */     {
/* 1342 */       ConcurrentLinkedHashMap.checkState(this.current != null);
/* 1343 */       ConcurrentLinkedHashMap.this.remove(this.current.key);
/* 1344 */       this.current = null;
/*      */     }
/*      */   }
/*      */   
/*      */   final class EntrySet extends AbstractSet<Map.Entry<K, V>>
/*      */   {
/* 1350 */     final ConcurrentLinkedHashMap<K, V> map = ConcurrentLinkedHashMap.this;
/*      */     
/*      */     EntrySet() {}
/*      */     
/* 1354 */     public int size() { return this.map.size(); }
/*      */     
/*      */ 
/*      */     public void clear()
/*      */     {
/* 1359 */       this.map.clear();
/*      */     }
/*      */     
/*      */     public Iterator<Map.Entry<K, V>> iterator()
/*      */     {
/* 1364 */       return new ConcurrentLinkedHashMap.EntryIterator(ConcurrentLinkedHashMap.this);
/*      */     }
/*      */     
/*      */     public boolean contains(Object obj)
/*      */     {
/* 1369 */       if (!(obj instanceof Map.Entry)) {
/* 1370 */         return false;
/*      */       }
/* 1372 */       Map.Entry<?, ?> entry = (Map.Entry)obj;
/* 1373 */       ConcurrentLinkedHashMap<K, V>.Node node = (ConcurrentLinkedHashMap.Node)this.map.data.get(entry.getKey());
/* 1374 */       return (node != null) && (node.getValue().equals(entry.getValue()));
/*      */     }
/*      */     
/*      */     public boolean add(Map.Entry<K, V> entry)
/*      */     {
/* 1379 */       return this.map.putIfAbsent(entry.getKey(), entry.getValue()) == null;
/*      */     }
/*      */     
/*      */     public boolean remove(Object obj)
/*      */     {
/* 1384 */       if (!(obj instanceof Map.Entry)) {
/* 1385 */         return false;
/*      */       }
/* 1387 */       Map.Entry<?, ?> entry = (Map.Entry)obj;
/* 1388 */       return this.map.remove(entry.getKey(), entry.getValue());
/*      */     }
/*      */   }
/*      */   
/*      */   final class EntryIterator implements Iterator<Map.Entry<K, V>>
/*      */   {
/* 1394 */     final Iterator<ConcurrentLinkedHashMap<K, V>.Node> iterator = ConcurrentLinkedHashMap.this.data.values().iterator();
/*      */     ConcurrentLinkedHashMap<K, V>.Node current;
/*      */     
/*      */     EntryIterator() {}
/*      */     
/* 1399 */     public boolean hasNext() { return this.iterator.hasNext(); }
/*      */     
/*      */ 
/*      */     public Map.Entry<K, V> next()
/*      */     {
/* 1404 */       this.current = ((ConcurrentLinkedHashMap.Node)this.iterator.next());
/* 1405 */       return new ConcurrentLinkedHashMap.WriteThroughEntry(ConcurrentLinkedHashMap.this, this.current);
/*      */     }
/*      */     
/*      */     public void remove()
/*      */     {
/* 1410 */       ConcurrentLinkedHashMap.checkState(this.current != null);
/* 1411 */       ConcurrentLinkedHashMap.this.remove(this.current.key);
/* 1412 */       this.current = null;
/*      */     }
/*      */   }
/*      */   
/*      */   final class WriteThroughEntry extends AbstractMap.SimpleEntry<K, V>
/*      */   {
/*      */     static final long serialVersionUID = 1L;
/*      */     
/*      */     WriteThroughEntry() {
/* 1421 */       super(node.getValue());
/*      */     }
/*      */     
/*      */     public V setValue(V value)
/*      */     {
/* 1426 */       ConcurrentLinkedHashMap.this.put(getKey(), value);
/* 1427 */       return (V)super.setValue(value);
/*      */     }
/*      */     
/*      */     Object writeReplace() {
/* 1431 */       return new AbstractMap.SimpleEntry(this);
/*      */     }
/*      */   }
/*      */   
/*      */   static final class BoundedEntryWeigher<K, V> implements EntryWeigher<K, V>, Serializable
/*      */   {
/*      */     static final long serialVersionUID = 1L;
/*      */     final EntryWeigher<? super K, ? super V> weigher;
/*      */     
/*      */     BoundedEntryWeigher(EntryWeigher<? super K, ? super V> weigher) {
/* 1441 */       ConcurrentLinkedHashMap.checkNotNull(weigher);
/* 1442 */       this.weigher = weigher;
/*      */     }
/*      */     
/*      */     public int weightOf(K key, V value)
/*      */     {
/* 1447 */       int weight = this.weigher.weightOf(key, value);
/* 1448 */       ConcurrentLinkedHashMap.checkArgument(weight >= 1);
/* 1449 */       return weight;
/*      */     }
/*      */     
/*      */     Object writeReplace() {
/* 1453 */       return this.weigher;
/*      */     }
/*      */   }
/*      */   
/*      */   static final class DiscardingQueue
/*      */     extends AbstractQueue<Object> {
/* 1459 */     public boolean add(Object e) { return true; }
/* 1460 */     public boolean offer(Object e) { return true; }
/* 1461 */     public Object poll() { return null; }
/* 1462 */     public Object peek() { return null; }
/* 1463 */     public int size() { return 0; }
/* 1464 */     public Iterator<Object> iterator() { return Collections.emptyList().iterator(); }
/*      */   }
/*      */   
/*      */   static enum DiscardingListener implements EvictionListener<Object, Object>
/*      */   {
/* 1469 */     INSTANCE;
/*      */     
/*      */     private DiscardingListener() {}
/*      */     
/*      */     public void onEviction(Object key, Object value) {}
/*      */   }
/*      */   
/*      */   static abstract interface Task
/*      */     extends Runnable
/*      */   {
/*      */     public abstract int getOrder();
/*      */     
/*      */     public abstract boolean isWrite();
/*      */     
/*      */     public abstract Task getNext();
/*      */     
/*      */     public abstract void setNext(Task paramTask);
/*      */   }
/*      */   
/*      */   abstract class AbstractTask
/*      */     implements ConcurrentLinkedHashMap.Task
/*      */   {
/*      */     final int order;
/*      */     ConcurrentLinkedHashMap.Task task;
/*      */     
/*      */     AbstractTask()
/*      */     {
/* 1496 */       this.order = ConcurrentLinkedHashMap.this.nextOrdering();
/*      */     }
/*      */     
/*      */     public int getOrder()
/*      */     {
/* 1501 */       return this.order;
/*      */     }
/*      */     
/*      */     public ConcurrentLinkedHashMap.Task getNext()
/*      */     {
/* 1506 */       return this.task;
/*      */     }
/*      */     
/*      */     public void setNext(ConcurrentLinkedHashMap.Task task)
/*      */     {
/* 1511 */       this.task = task;
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   Object writeReplace()
/*      */   {
/* 1520 */     return new SerializationProxy(this);
/*      */   }
/*      */   
/*      */   private void readObject(ObjectInputStream stream) throws InvalidObjectException {
/* 1524 */     throw new InvalidObjectException("Proxy required");
/*      */   }
/*      */   
/*      */ 
/*      */   static final class SerializationProxy<K, V>
/*      */     implements Serializable
/*      */   {
/*      */     final EntryWeigher<? super K, ? super V> weigher;
/*      */     
/*      */     final EvictionListener<K, V> listener;
/*      */     
/*      */     final int concurrencyLevel;
/*      */     final Map<K, V> data;
/*      */     final long capacity;
/*      */     static final long serialVersionUID = 1L;
/*      */     
/*      */     SerializationProxy(ConcurrentLinkedHashMap<K, V> map)
/*      */     {
/* 1542 */       this.concurrencyLevel = map.concurrencyLevel;
/* 1543 */       this.data = new HashMap(map);
/* 1544 */       this.capacity = map.capacity;
/* 1545 */       this.listener = map.listener;
/* 1546 */       this.weigher = map.weigher;
/*      */     }
/*      */     
/*      */     Object readResolve() {
/* 1550 */       ConcurrentLinkedHashMap<K, V> map = new ConcurrentLinkedHashMap.Builder().concurrencyLevel(this.concurrencyLevel).maximumWeightedCapacity(this.capacity).listener(this.listener).weigher(this.weigher).build();
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1556 */       map.putAll(this.data);
/* 1557 */       return map;
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public static final class Builder<K, V>
/*      */   {
/*      */     static final int DEFAULT_CONCURRENCY_LEVEL = 16;
/*      */     
/*      */ 
/*      */     static final int DEFAULT_INITIAL_CAPACITY = 16;
/*      */     
/*      */ 
/*      */     EvictionListener<K, V> listener;
/*      */     
/*      */ 
/*      */     EntryWeigher<? super K, ? super V> weigher;
/*      */     
/*      */ 
/*      */     int concurrencyLevel;
/*      */     
/*      */ 
/*      */     int initialCapacity;
/*      */     
/*      */ 
/*      */     long capacity;
/*      */     
/*      */ 
/*      */ 
/*      */     public Builder()
/*      */     {
/* 1589 */       this.capacity = -1L;
/* 1590 */       this.weigher = Weighers.entrySingleton();
/* 1591 */       this.initialCapacity = 16;
/* 1592 */       this.concurrencyLevel = 16;
/* 1593 */       this.listener = ConcurrentLinkedHashMap.DiscardingListener.INSTANCE;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     public Builder<K, V> initialCapacity(int initialCapacity)
/*      */     {
/* 1606 */       ConcurrentLinkedHashMap.checkArgument(initialCapacity >= 0);
/* 1607 */       this.initialCapacity = initialCapacity;
/* 1608 */       return this;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     public Builder<K, V> maximumWeightedCapacity(long capacity)
/*      */     {
/* 1620 */       ConcurrentLinkedHashMap.checkArgument(capacity >= 0L);
/* 1621 */       this.capacity = capacity;
/* 1622 */       return this;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     public Builder<K, V> concurrencyLevel(int concurrencyLevel)
/*      */     {
/* 1636 */       ConcurrentLinkedHashMap.checkArgument(concurrencyLevel > 0);
/* 1637 */       this.concurrencyLevel = concurrencyLevel;
/* 1638 */       return this;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     public Builder<K, V> listener(EvictionListener<K, V> listener)
/*      */     {
/* 1649 */       ConcurrentLinkedHashMap.checkNotNull(listener);
/* 1650 */       this.listener = listener;
/* 1651 */       return this;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     public Builder<K, V> weigher(Weigher<? super V> weigher)
/*      */     {
/* 1663 */       this.weigher = (weigher == Weighers.singleton() ? Weighers.entrySingleton() : new ConcurrentLinkedHashMap.BoundedEntryWeigher(Weighers.asEntryWeigher(weigher)));
/*      */       
/*      */ 
/* 1666 */       return this;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     public Builder<K, V> weigher(EntryWeigher<? super K, ? super V> weigher)
/*      */     {
/* 1678 */       this.weigher = (weigher == Weighers.entrySingleton() ? Weighers.entrySingleton() : new ConcurrentLinkedHashMap.BoundedEntryWeigher(weigher));
/*      */       
/*      */ 
/* 1681 */       return this;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     public ConcurrentLinkedHashMap<K, V> build()
/*      */     {
/* 1691 */       ConcurrentLinkedHashMap.checkState(this.capacity >= 0L);
/* 1692 */       return new ConcurrentLinkedHashMap(this, null);
/*      */     }
/*      */   }
/*      */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\googlecode\concurrentlinkedhashmap\ConcurrentLinkedHashMap.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */