/*     */ package com.netapp.asup.data.function;
/*     */ 
/*     */ import com.netapp.asup.common.AsupLogger;
/*     */ import com.netapp.asup.data.store.BaseContainer;
/*     */ import com.netapp.asup.data.store.StoreContainer;
/*     */ import com.netapp.asup.exception.ARMAFunctionRegistrationException;
/*     */ import com.netapp.asup.exception.ARMAParserException;
/*     */ import java.io.InputStream;
/*     */ import java.util.HashMap;
/*     */ import java.util.Map;
/*     */ import java.util.Set;
/*     */ import javax.xml.parsers.SAXParser;
/*     */ import javax.xml.parsers.SAXParserFactory;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.Schema.Parser;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class FunctionRegistryImpl
/*     */   implements FunctionRegistry
/*     */ {
/*  23 */   private static final AsupLogger LOG = AsupLogger.getLogger(FunctionRegistryImpl.class);
/*     */   
/*     */ 
/*  26 */   private final Map<String, FunctionRegistoryObject<?, ?>> registeredFunctions = new HashMap();
/*     */   
/*     */   private static final String FUNCTIONS_XML = "Functions.xml";
/*     */   
/*  30 */   private final Map<String, Schema> schemaCache = new HashMap();
/*  31 */   private final Map<String, FunctionInputHandler<?>> inputHandlerCache = new HashMap();
/*  32 */   private final Map<String, FunctionOutputHandler<?>> outputHandlerCache = new HashMap();
/*     */   
/*     */   public FunctionRegistryImpl()
/*     */   {
/*  36 */     InputStream is = ClassLoader.getSystemResourceAsStream("Functions.xml");
/*  37 */     if (is != null) {
/*     */       try {
/*  39 */         addFunctionDefinition(is);
/*     */       } catch (ARMAFunctionRegistrationException e) {
/*  41 */         LOG.warn("-9999", "Problem encountered in fucntion registry:" + e.getMessage(), e);
/*     */       }
/*     */       
/*     */     }
/*     */     else
/*     */     {
/*  47 */       LOG.warn("-9999", "No functions are registered");
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public Set<FunctionRegistoryObject<?, ?>> addFunctionDefinition(InputStream is)
/*     */     throws ARMAFunctionRegistrationException
/*     */   {
/*     */     try
/*     */     {
/*  57 */       SAXParser parser = getParser();
/*  58 */       FunctionHandler handler = new FunctionHandler();
/*  59 */       parser.parse(is, handler);
/*  60 */       Set<FunctionRegistoryObject<?, ?>> regFunctions = handler.getFunctions();
/*     */       
/*     */ 
/*  63 */       for (FunctionRegistoryObject<?, ?> regFunction : regFunctions) {
/*     */         try {
/*  65 */           buildCache(regFunction);
/*  66 */           FunctionValidator.validateandInitialize(regFunction, this);
/*  67 */           getRegisteredFunctions().put(regFunction.getFunctionName(), regFunction);
/*     */         }
/*     */         catch (ARMAFunctionRegistrationException e)
/*     */         {
/*  71 */           LOG.warn("-9999", "Problem encountered in registrying function:" + regFunction + "Error:" + e.getMessage(), e);
/*     */         }
/*     */       }
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*  78 */       return regFunctions;
/*     */     } catch (Exception e) {
/*  80 */       throw new ARMAFunctionRegistrationException(LOG, "Problem Registering the function:" + e.getMessage(), e);
/*     */     }
/*     */   }
/*     */   
/*     */   private void buildCache(FunctionRegistoryObject<?, ?> regFunction)
/*     */     throws ARMAFunctionRegistrationException
/*     */   {
/*  87 */     buildSchemaCache(regFunction);
/*  88 */     buildInputHandlerCache(regFunction);
/*  89 */     buildOutputHandlerCache(regFunction);
/*     */   }
/*     */   
/*     */   private <I, O> void buildInputHandlerCache(FunctionRegistoryObject<I, O> regFunction)
/*     */     throws ARMAFunctionRegistrationException
/*     */   {
/*  95 */     String className = regFunction.getAttr(FuncAttributes.INPUTHANDLERCLASSNAME);
/*     */     
/*  97 */     if ((className != null) && 
/*  98 */       (this.inputHandlerCache.get(className) == null)) {
/*  99 */       this.inputHandlerCache.put(className, initializeInputHandler(className));
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   private <I, O> void buildOutputHandlerCache(FunctionRegistoryObject<I, O> regFunction)
/*     */     throws ARMAFunctionRegistrationException
/*     */   {
/* 108 */     String className = regFunction.getAttr(FuncAttributes.OUTPUTHANDLERCLASSNAME);
/*     */     
/* 110 */     if ((className != null) && 
/* 111 */       (this.outputHandlerCache.get(className) == null)) {
/* 112 */       this.outputHandlerCache.put(className, initializeOutputHandler(className));
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public <I> FunctionInputHandler<I> getInputHandler(String className)
/*     */   {
/* 120 */     return (FunctionInputHandler)this.inputHandlerCache.get(className);
/*     */   }
/*     */   
/*     */   public <O> FunctionOutputHandler<O> getOutputHandler(String className)
/*     */   {
/* 125 */     return (FunctionOutputHandler)this.outputHandlerCache.get(className);
/*     */   }
/*     */   
/*     */ 
/*     */   private <I, O> void buildSchemaCache(FunctionRegistoryObject<I, O> regFunction)
/*     */     throws ARMAFunctionRegistrationException
/*     */   {
/* 132 */     String inputSchema = regFunction.getAttr(FuncAttributes.INPUTSCHEMAFILE);
/*     */     
/* 134 */     if (this.schemaCache.get(inputSchema) == null) {
/* 135 */       this.schemaCache.put(inputSchema, initializeSchema(inputSchema, regFunction));
/*     */     }
/* 137 */     String outputSchema = regFunction.getAttr(FuncAttributes.OUTPUTSCHEMAFILE);
/*     */     
/* 139 */     if (this.schemaCache.get(outputSchema) == null) {
/* 140 */       this.schemaCache.put(outputSchema, initializeSchema(outputSchema, regFunction));
/*     */     }
/*     */   }
/*     */   
/*     */   private SAXParser getParser() throws ARMAParserException {
/* 145 */     SAXParserFactory factory = null;
/*     */     try {
/* 147 */       factory = SAXParserFactory.newInstance();
/*     */       
/* 149 */       factory.setValidating(true);
/*     */       
/* 151 */       factory.setFeature("http://apache.org/xml/features/validation/schema", true);
/* 152 */       return factory.newSAXParser();
/*     */     } catch (Exception ex) {
/* 154 */       String errMsg = "Problem initializing the parser factory";
/* 155 */       throw new ARMAParserException(LOG, "-9999", errMsg, ex);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public <I, O> RegisteredFunction<I, O> getRegisteredFunction(String functionName)
/*     */   {
/* 165 */     return (RegisteredFunction)this.registeredFunctions.get(functionName);
/*     */   }
/*     */   
/*     */   public Map<String, FunctionRegistoryObject<?, ?>> getRegisteredFunctions()
/*     */   {
/* 170 */     return this.registeredFunctions;
/*     */   }
/*     */   
/*     */   private <I, O> Schema initializeSchema(String schemaFile, FunctionRegistoryObject<I, O> regFunction)
/*     */     throws ARMAFunctionRegistrationException
/*     */   {
/* 176 */     if ((null == schemaFile) || (schemaFile.length() == 0)) {
/* 177 */       return null;
/*     */     }
/*     */     
/* 180 */     if (schemaFile.startsWith("hdfs")) {
/*     */       try {
/* 182 */         StoreContainer container = BaseContainer.getInstance().getStoreContainer();
/* 183 */         return container.getSchemaFromHDFS(schemaFile);
/*     */       }
/*     */       catch (Exception ex) {
/* 186 */         throw new ARMAFunctionRegistrationException(LOG, "unable to load schema from HDFS for : " + schemaFile + " for function : " + regFunction.toString() + " caused by : " + ex.getMessage(), ex);
/*     */       }
/*     */     }
/*     */     
/* 190 */     InputStream stream = ClassLoader.getSystemResourceAsStream(schemaFile);
/*     */     try {
/* 192 */       if (null != stream) {
/* 193 */         Schema.Parser parser = new Schema.Parser();
/* 194 */         return parser.parse(stream);
/*     */       }
/* 196 */       throw new ARMAFunctionRegistrationException(LOG, "Unable to initialize schema :" + schemaFile + " for function :" + regFunction.toString() + " due to unavailability of the file in the classpath");
/*     */ 
/*     */ 
/*     */ 
/*     */     }
/*     */     catch (Exception e)
/*     */     {
/*     */ 
/*     */ 
/* 205 */       throw new ARMAFunctionRegistrationException(LOG, "Unable to initialize schema :" + schemaFile + " for function :" + regFunction.toString() + " caused by :" + e.getMessage(), e);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   private Schema initializeSchema(String schemaFile)
/*     */     throws ARMAFunctionRegistrationException
/*     */   {
/* 215 */     if ((null == schemaFile) || (schemaFile.length() == 0)) {
/* 216 */       return null;
/*     */     }
/* 218 */     InputStream stream = ClassLoader.getSystemResourceAsStream(schemaFile);
/*     */     try {
/* 220 */       if (null != stream) {
/* 221 */         Schema.Parser parser = new Schema.Parser();
/* 222 */         return parser.parse(stream);
/*     */       }
/* 224 */       throw new ARMAFunctionRegistrationException(LOG, "Unable to initialize schema :" + schemaFile + " ,due to unavailability of the file in the classpath");
/*     */ 
/*     */ 
/*     */     }
/*     */     catch (Exception e)
/*     */     {
/*     */ 
/* 231 */       throw new ARMAFunctionRegistrationException(LOG, "Unable to initialize schema :" + schemaFile + " caused by :" + e.getMessage(), e);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   private <T> FunctionInputHandler<T> initializeInputHandler(String className)
/*     */     throws ARMAFunctionRegistrationException
/*     */   {
/*     */     try
/*     */     {
/* 241 */       Class<T> inputHandlerClass = Class.forName(className, true, ClassLoader.getSystemClassLoader());
/*     */       
/* 243 */       return (FunctionInputHandler)inputHandlerClass.newInstance();
/*     */     }
/*     */     catch (ClassNotFoundException e)
/*     */     {
/* 247 */       throw new ARMAFunctionRegistrationException(LOG, "Unable to initialize function :" + toString() + " caused by :" + e.getMessage(), e);
/*     */     }
/*     */     catch (InstantiationException e)
/*     */     {
/* 251 */       throw new ARMAFunctionRegistrationException(LOG, "Unable to initialize function :" + toString() + " caused by :" + e.getMessage(), e);
/*     */     }
/*     */     catch (IllegalAccessException e)
/*     */     {
/* 255 */       throw new ARMAFunctionRegistrationException(LOG, "Unable to initialize function :" + toString() + " caused by :" + e.getMessage(), e);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   private <T> FunctionOutputHandler<T> initializeOutputHandler(String className)
/*     */     throws ARMAFunctionRegistrationException
/*     */   {
/*     */     try
/*     */     {
/* 265 */       Class<T> outputHandlerClass = Class.forName(className, true, ClassLoader.getSystemClassLoader());
/*     */       
/* 267 */       return (FunctionOutputHandler)outputHandlerClass.newInstance();
/*     */     }
/*     */     catch (ClassNotFoundException e)
/*     */     {
/* 271 */       throw new ARMAFunctionRegistrationException(LOG, "Unable to initialize function :" + toString() + " caused by :" + e.getMessage(), e);
/*     */     }
/*     */     catch (InstantiationException e)
/*     */     {
/* 275 */       throw new ARMAFunctionRegistrationException(LOG, "Unable to initialize function :" + toString() + " caused by :" + e.getMessage(), e);
/*     */     }
/*     */     catch (IllegalAccessException e)
/*     */     {
/* 279 */       throw new ARMAFunctionRegistrationException(LOG, "Unable to initialize function :" + toString() + " caused by :" + e.getMessage(), e);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public Schema getSchema(String fileName)
/*     */   {
/* 288 */     Schema schema = (Schema)this.schemaCache.get(fileName);
/* 289 */     if (null == schema) {
/*     */       try {
/* 291 */         schema = initializeSchema(fileName);
/*     */       } catch (ARMAFunctionRegistrationException e) {
/* 293 */         return null;
/*     */       }
/* 295 */       this.schemaCache.put(fileName, schema);
/*     */     }
/* 297 */     return schema;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asup\data\function\FunctionRegistryImpl.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */