/*     */ package org.apache.avro.io;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.io.OutputStream;
/*     */ import java.util.Arrays;
/*     */ import org.apache.avro.AvroTypeException;
/*     */ import org.apache.avro.Schema.Type;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class BlockingBinaryEncoder
/*     */   extends BufferedBinaryEncoder
/*     */ {
/*     */   private byte[] buf;
/*     */   private int pos;
/*     */   private BlockedValue[] blockStack;
/*     */   
/*     */   private static class BlockedValue
/*     */   {
/*     */     public Schema.Type type;
/*     */     public State state;
/*     */     public int start;
/*     */     public int lastFullItem;
/*     */     public int items;
/*     */     public long itemsLeftToWrite;
/*     */     
/*     */     public static enum State
/*     */     {
/*  75 */       ROOT, 
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  84 */       REGULAR, 
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  95 */       OVERFLOW;
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       private State() {}
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public BlockedValue()
/*     */     {
/* 124 */       this.type = null;
/* 125 */       this.state = State.ROOT;
/* 126 */       this.start = (this.lastFullItem = 0);
/* 127 */       this.items = 1;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     public boolean check(BlockedValue prev, int pos)
/*     */     {
/* 135 */       assert ((this.state != State.ROOT) || (this.type == null));
/* 136 */       assert ((this.state == State.ROOT) || (this.type == Schema.Type.ARRAY) || (this.type == Schema.Type.MAP));
/*     */       
/*     */ 
/* 139 */       assert (0 <= this.items);
/* 140 */       assert ((0 != this.items) || (this.start == pos));
/* 141 */       assert ((1 < this.items) || (this.start == this.lastFullItem));
/* 142 */       assert ((this.items <= 1) || (this.start <= this.lastFullItem));
/* 143 */       assert (this.lastFullItem <= pos);
/*     */       
/* 145 */       switch (BlockingBinaryEncoder.1.$SwitchMap$org$apache$avro$io$BlockingBinaryEncoder$BlockedValue$State[this.state.ordinal()]) {
/*     */       case 1: 
/* 147 */         assert (this.start == 0);
/* 148 */         if ((!$assertionsDisabled) && (prev != null)) throw new AssertionError();
/*     */         break;
/*     */       case 2: 
/* 151 */         assert (this.start >= 0);
/* 152 */         assert (prev.lastFullItem <= this.start);
/* 153 */         if ((!$assertionsDisabled) && (1 > prev.items)) throw new AssertionError();
/*     */         break;
/*     */       case 3: 
/* 156 */         assert (this.start == 0);
/* 157 */         assert (this.items == 1);
/* 158 */         assert ((prev.state == State.ROOT) || (prev.state == State.OVERFLOW));
/*     */       }
/*     */       
/* 161 */       return false;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 180 */   private int stackTop = -1;
/*     */   
/*     */ 
/*     */   private static final int STACK_STEP = 10;
/*     */   
/* 185 */   private byte[] headerBuffer = new byte[12];
/*     */   
/*     */   private boolean check() {
/* 188 */     assert (this.buf != null);
/* 189 */     assert (0 <= this.pos);
/* 190 */     assert (this.pos <= this.buf.length) : (this.pos + " " + this.buf.length);
/*     */     
/* 192 */     assert (this.blockStack != null);
/* 193 */     BlockedValue prev = null;
/* 194 */     for (int i = 0; i <= this.stackTop; i++) {
/* 195 */       BlockedValue v = this.blockStack[i];
/* 196 */       v.check(prev, this.pos);
/* 197 */       prev = v;
/*     */     }
/* 199 */     return true;
/*     */   }
/*     */   
/*     */   BlockingBinaryEncoder(OutputStream out, int blockBufferSize, int binaryEncoderBufferSize)
/*     */   {
/* 204 */     super(out, binaryEncoderBufferSize);
/* 205 */     this.buf = new byte[blockBufferSize];
/* 206 */     this.pos = 0;
/* 207 */     this.blockStack = new BlockedValue[0];
/* 208 */     expandStack();
/* 209 */     BlockedValue bv = this.blockStack[(++this.stackTop)];
/* 210 */     bv.type = null;
/* 211 */     bv.state = BlockingBinaryEncoder.BlockedValue.State.ROOT;
/* 212 */     bv.start = (bv.lastFullItem = 0);
/* 213 */     bv.items = 1;
/*     */     
/* 215 */     assert (check());
/*     */   }
/*     */   
/*     */   private void expandStack() {
/* 219 */     int oldLength = this.blockStack.length;
/* 220 */     this.blockStack = ((BlockedValue[])Arrays.copyOf(this.blockStack, this.blockStack.length + 10));
/*     */     
/* 222 */     for (int i = oldLength; i < this.blockStack.length; i++) {
/* 223 */       this.blockStack[i] = new BlockedValue();
/*     */     }
/*     */   }
/*     */   
/*     */   BlockingBinaryEncoder configure(OutputStream out, int blockBufferSize, int binaryEncoderBufferSize)
/*     */   {
/* 229 */     super.configure(out, binaryEncoderBufferSize);
/* 230 */     this.pos = 0;
/* 231 */     this.stackTop = 0;
/* 232 */     if ((null == this.buf) || (this.buf.length != blockBufferSize)) {
/* 233 */       this.buf = new byte[blockBufferSize];
/*     */     }
/*     */     
/* 236 */     assert (check());
/* 237 */     return this;
/*     */   }
/*     */   
/*     */   public void flush() throws IOException
/*     */   {
/* 242 */     BlockedValue bv = this.blockStack[this.stackTop];
/* 243 */     if (bv.state == BlockingBinaryEncoder.BlockedValue.State.ROOT) {
/* 244 */       super.writeFixed(this.buf, 0, this.pos);
/* 245 */       this.pos = 0;
/*     */     } else {
/* 247 */       while (bv.state != BlockingBinaryEncoder.BlockedValue.State.OVERFLOW) {
/* 248 */         compact();
/*     */       }
/*     */     }
/* 251 */     super.flush();
/*     */     
/* 253 */     assert (check());
/*     */   }
/*     */   
/*     */   public void writeBoolean(boolean b) throws IOException
/*     */   {
/* 258 */     ensureBounds(1);
/* 259 */     this.pos += BinaryData.encodeBoolean(b, this.buf, this.pos);
/*     */   }
/*     */   
/*     */   public void writeInt(int n) throws IOException
/*     */   {
/* 264 */     ensureBounds(5);
/* 265 */     this.pos += BinaryData.encodeInt(n, this.buf, this.pos);
/*     */   }
/*     */   
/*     */   public void writeLong(long n) throws IOException
/*     */   {
/* 270 */     ensureBounds(10);
/* 271 */     this.pos += BinaryData.encodeLong(n, this.buf, this.pos);
/*     */   }
/*     */   
/*     */   public void writeFloat(float f) throws IOException
/*     */   {
/* 276 */     ensureBounds(4);
/* 277 */     this.pos += BinaryData.encodeFloat(f, this.buf, this.pos);
/*     */   }
/*     */   
/*     */   public void writeDouble(double d) throws IOException
/*     */   {
/* 282 */     ensureBounds(8);
/* 283 */     this.pos += BinaryData.encodeDouble(d, this.buf, this.pos);
/*     */   }
/*     */   
/*     */   public void writeFixed(byte[] bytes, int start, int len) throws IOException
/*     */   {
/* 288 */     doWriteBytes(bytes, start, len);
/*     */   }
/*     */   
/*     */   protected void writeZero() throws IOException
/*     */   {
/* 293 */     ensureBounds(1);
/* 294 */     this.buf[(this.pos++)] = 0;
/*     */   }
/*     */   
/*     */   public void writeArrayStart() throws IOException
/*     */   {
/* 299 */     if (this.stackTop + 1 == this.blockStack.length) {
/* 300 */       expandStack();
/*     */     }
/*     */     
/* 303 */     BlockedValue bv = this.blockStack[(++this.stackTop)];
/* 304 */     bv.type = Schema.Type.ARRAY;
/* 305 */     bv.state = BlockingBinaryEncoder.BlockedValue.State.REGULAR;
/* 306 */     bv.start = (bv.lastFullItem = this.pos);
/* 307 */     bv.items = 0;
/*     */     
/* 309 */     assert (check());
/*     */   }
/*     */   
/*     */   public void setItemCount(long itemCount) throws IOException
/*     */   {
/* 314 */     BlockedValue v = this.blockStack[this.stackTop];
/* 315 */     assert ((v.type == Schema.Type.ARRAY) || (v.type == Schema.Type.MAP));
/* 316 */     assert (v.itemsLeftToWrite == 0L);
/* 317 */     v.itemsLeftToWrite = itemCount;
/*     */     
/* 319 */     assert (check());
/*     */   }
/*     */   
/*     */   public void startItem() throws IOException
/*     */   {
/* 324 */     if (this.blockStack[this.stackTop].state == BlockingBinaryEncoder.BlockedValue.State.OVERFLOW) {
/* 325 */       finishOverflow();
/*     */     }
/* 327 */     BlockedValue t = this.blockStack[this.stackTop];
/* 328 */     t.items += 1;
/* 329 */     t.lastFullItem = this.pos;
/* 330 */     t.itemsLeftToWrite -= 1L;
/*     */     
/* 332 */     assert (check());
/*     */   }
/*     */   
/*     */   public void writeArrayEnd() throws IOException
/*     */   {
/* 337 */     BlockedValue top = this.blockStack[this.stackTop];
/* 338 */     if (top.type != Schema.Type.ARRAY) {
/* 339 */       throw new AvroTypeException("Called writeArrayEnd outside of an array.");
/*     */     }
/* 341 */     if (top.itemsLeftToWrite != 0L) {
/* 342 */       throw new AvroTypeException("Failed to write expected number of array elements.");
/*     */     }
/* 344 */     endBlockedValue();
/*     */     
/* 346 */     assert (check());
/*     */   }
/*     */   
/*     */   public void writeMapStart() throws IOException
/*     */   {
/* 351 */     if (this.stackTop + 1 == this.blockStack.length) {
/* 352 */       expandStack();
/*     */     }
/*     */     
/* 355 */     BlockedValue bv = this.blockStack[(++this.stackTop)];
/* 356 */     bv.type = Schema.Type.MAP;
/* 357 */     bv.state = BlockingBinaryEncoder.BlockedValue.State.REGULAR;
/* 358 */     bv.start = (bv.lastFullItem = this.pos);
/* 359 */     bv.items = 0;
/*     */     
/* 361 */     assert (check());
/*     */   }
/*     */   
/*     */   public void writeMapEnd() throws IOException
/*     */   {
/* 366 */     BlockedValue top = this.blockStack[this.stackTop];
/* 367 */     if (top.type != Schema.Type.MAP) {
/* 368 */       throw new AvroTypeException("Called writeMapEnd outside of a map.");
/*     */     }
/* 370 */     if (top.itemsLeftToWrite != 0L) {
/* 371 */       throw new AvroTypeException("Failed to read write expected number of array elements.");
/*     */     }
/* 373 */     endBlockedValue();
/*     */     
/* 375 */     assert (check());
/*     */   }
/*     */   
/*     */   public void writeIndex(int unionIndex) throws IOException
/*     */   {
/* 380 */     ensureBounds(5);
/* 381 */     this.pos += BinaryData.encodeInt(unionIndex, this.buf, this.pos);
/*     */   }
/*     */   
/*     */   public int bytesBuffered()
/*     */   {
/* 386 */     return this.pos + super.bytesBuffered();
/*     */   }
/*     */   
/*     */   private void endBlockedValue() throws IOException {
/*     */     for (;;) {
/* 391 */       assert (check());
/* 392 */       BlockedValue t = this.blockStack[this.stackTop];
/* 393 */       assert (t.state != BlockingBinaryEncoder.BlockedValue.State.ROOT);
/* 394 */       if (t.state == BlockingBinaryEncoder.BlockedValue.State.OVERFLOW) {
/* 395 */         finishOverflow();
/*     */       }
/* 397 */       assert (t.state == BlockingBinaryEncoder.BlockedValue.State.REGULAR);
/* 398 */       if (0 >= t.items) break;
/* 399 */       int byteCount = this.pos - t.start;
/* 400 */       if ((t.start == 0) && (this.blockStack[(this.stackTop - 1)].state != BlockingBinaryEncoder.BlockedValue.State.REGULAR))
/*     */       {
/*     */ 
/* 403 */         super.writeInt(-t.items);
/* 404 */         super.writeInt(byteCount); break;
/*     */       }
/* 406 */       int headerSize = 0;
/* 407 */       headerSize += BinaryData.encodeInt(-t.items, this.headerBuffer, headerSize);
/* 408 */       headerSize += BinaryData.encodeInt(byteCount, this.headerBuffer, headerSize);
/* 409 */       if (this.buf.length >= this.pos + headerSize) {
/* 410 */         this.pos += headerSize;
/* 411 */         int m = t.start;
/* 412 */         System.arraycopy(this.buf, m, this.buf, m + headerSize, byteCount);
/* 413 */         System.arraycopy(this.headerBuffer, 0, this.buf, m, headerSize);
/* 414 */         break; }
/* 415 */       compact();
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 420 */     this.stackTop -= 1;
/* 421 */     ensureBounds(1);
/* 422 */     this.buf[(this.pos++)] = 0;
/* 423 */     assert (check());
/* 424 */     if (this.blockStack[this.stackTop].state == BlockingBinaryEncoder.BlockedValue.State.ROOT) {
/* 425 */       flush();
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private void finishOverflow()
/*     */     throws IOException
/*     */   {
/* 438 */     BlockedValue s = this.blockStack[this.stackTop];
/* 439 */     if (s.state != BlockingBinaryEncoder.BlockedValue.State.OVERFLOW) {
/* 440 */       throw new IllegalStateException("Not an overflow block");
/*     */     }
/* 442 */     assert (check());
/*     */     
/*     */ 
/* 445 */     super.writeFixed(this.buf, 0, this.pos);
/* 446 */     this.pos = 0;
/*     */     
/*     */ 
/* 449 */     s.state = BlockingBinaryEncoder.BlockedValue.State.REGULAR;
/* 450 */     s.start = (s.lastFullItem = 0);
/* 451 */     s.items = 0;
/* 452 */     assert (check());
/*     */   }
/*     */   
/*     */   private void ensureBounds(int l) throws IOException {
/* 456 */     while (this.buf.length < this.pos + l) {
/* 457 */       if (this.blockStack[this.stackTop].state == BlockingBinaryEncoder.BlockedValue.State.REGULAR) {
/* 458 */         compact();
/*     */       } else {
/* 460 */         super.writeFixed(this.buf, 0, this.pos);
/* 461 */         this.pos = 0;
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   private void doWriteBytes(byte[] bytes, int start, int len) throws IOException
/*     */   {
/* 468 */     if (len < this.buf.length) {
/* 469 */       ensureBounds(len);
/* 470 */       System.arraycopy(bytes, start, this.buf, this.pos, len);
/* 471 */       this.pos += len;
/*     */     } else {
/* 473 */       ensureBounds(this.buf.length);
/* 474 */       assert ((this.blockStack[this.stackTop].state == BlockingBinaryEncoder.BlockedValue.State.ROOT) || (this.blockStack[this.stackTop].state == BlockingBinaryEncoder.BlockedValue.State.OVERFLOW));
/*     */       
/* 476 */       write(bytes, start, len);
/*     */     }
/*     */   }
/*     */   
/*     */   private void write(byte[] b, int off, int len) throws IOException {
/* 481 */     if (this.blockStack[this.stackTop].state == BlockingBinaryEncoder.BlockedValue.State.ROOT) {
/* 482 */       super.writeFixed(b, off, len);
/*     */     } else {
/* 484 */       assert (check());
/* 485 */       while (this.buf.length < this.pos + len) {
/* 486 */         if (this.blockStack[this.stackTop].state == BlockingBinaryEncoder.BlockedValue.State.REGULAR) {
/* 487 */           compact();
/*     */         } else {
/* 489 */           super.writeFixed(this.buf, 0, this.pos);
/* 490 */           this.pos = 0;
/* 491 */           if (this.buf.length <= len) {
/* 492 */             super.writeFixed(b, off, len);
/* 493 */             len = 0;
/*     */           }
/*     */         }
/*     */       }
/* 497 */       System.arraycopy(b, off, this.buf, this.pos, len);
/* 498 */       this.pos += len;
/*     */     }
/* 500 */     assert (check());
/*     */   }
/*     */   
/*     */   private void compact() throws IOException
/*     */   {
/* 505 */     assert (check());
/*     */     
/*     */ 
/* 508 */     BlockedValue s = null;
/*     */     
/* 510 */     for (int i = 1; i <= this.stackTop; i++) {
/* 511 */       s = this.blockStack[i];
/* 512 */       if (s.state == BlockingBinaryEncoder.BlockedValue.State.REGULAR) break;
/*     */     }
/* 514 */     assert (s != null);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 525 */     super.writeFixed(this.buf, 0, s.start);
/*     */     
/*     */ 
/* 528 */     if (1 < s.items) {
/* 529 */       super.writeInt(-(s.items - 1));
/* 530 */       super.writeInt(s.lastFullItem - s.start);
/* 531 */       super.writeFixed(this.buf, s.start, s.lastFullItem - s.start);
/* 532 */       s.start = s.lastFullItem;
/* 533 */       s.items = 1;
/*     */     }
/*     */     
/*     */ 
/* 537 */     super.writeInt(1);
/*     */     
/*     */ 
/*     */ 
/* 541 */     BlockedValue n = i + 1 <= this.stackTop ? this.blockStack[(i + 1)] : null;
/*     */     
/* 543 */     int end = n == null ? this.pos : n.start;
/* 544 */     super.writeFixed(this.buf, s.lastFullItem, end - s.lastFullItem);
/*     */     
/*     */ 
/* 547 */     System.arraycopy(this.buf, end, this.buf, 0, this.pos - end);
/* 548 */     for (int j = i + 1; j <= this.stackTop; j++) {
/* 549 */       n = this.blockStack[j];
/* 550 */       n.start -= end;
/* 551 */       n.lastFullItem -= end;
/*     */     }
/* 553 */     this.pos -= end;
/*     */     
/* 555 */     assert (s.items == 1);
/* 556 */     s.start = (s.lastFullItem = 0);
/* 557 */     s.state = BlockingBinaryEncoder.BlockedValue.State.OVERFLOW;
/*     */     
/* 559 */     assert (check());
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\io\BlockingBinaryEncoder.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */