/*     */ package org.codehaus.jackson.impl;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.math.BigDecimal;
/*     */ import java.math.BigInteger;
/*     */ import org.codehaus.jackson.JsonParseException;
/*     */ import org.codehaus.jackson.JsonParser.NumberType;
/*     */ import org.codehaus.jackson.JsonToken;
/*     */ import org.codehaus.jackson.io.IOContext;
/*     */ import org.codehaus.jackson.io.NumberInput;
/*     */ import org.codehaus.jackson.util.TextBuffer;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class JsonNumericParserBase
/*     */   extends JsonParserBase
/*     */ {
/*     */   protected static final int NR_UNKNOWN = 0;
/*     */   protected static final int NR_INT = 1;
/*     */   protected static final int NR_LONG = 2;
/*     */   protected static final int NR_BIGINT = 4;
/*     */   protected static final int NR_DOUBLE = 8;
/*     */   protected static final int NR_BIGDECIMAL = 16;
/*  41 */   static final BigDecimal BD_MIN_LONG = new BigDecimal(Long.MIN_VALUE);
/*  42 */   static final BigDecimal BD_MAX_LONG = new BigDecimal(Long.MAX_VALUE);
/*     */   
/*  44 */   static final BigDecimal BD_MIN_INT = new BigDecimal(Long.MIN_VALUE);
/*  45 */   static final BigDecimal BD_MAX_INT = new BigDecimal(Long.MAX_VALUE);
/*     */   
/*     */   static final long MIN_INT_L = -2147483648L;
/*     */   
/*     */   static final long MAX_INT_L = 2147483647L;
/*     */   
/*     */   static final double MIN_LONG_D = -9.223372036854776E18D;
/*     */   
/*     */   static final double MAX_LONG_D = 9.223372036854776E18D;
/*     */   
/*     */   static final double MIN_INT_D = -2.147483648E9D;
/*     */   
/*     */   static final double MAX_INT_D = 2.147483647E9D;
/*     */   
/*     */   protected static final int INT_0 = 48;
/*     */   
/*     */   protected static final int INT_1 = 49;
/*     */   
/*     */   protected static final int INT_2 = 50;
/*     */   
/*     */   protected static final int INT_3 = 51;
/*     */   
/*     */   protected static final int INT_4 = 52;
/*     */   
/*     */   protected static final int INT_5 = 53;
/*     */   
/*     */   protected static final int INT_6 = 54;
/*     */   
/*     */   protected static final int INT_7 = 55;
/*     */   
/*     */   protected static final int INT_8 = 56;
/*     */   
/*     */   protected static final int INT_9 = 57;
/*     */   
/*     */   protected static final int INT_MINUS = 45;
/*     */   
/*     */   protected static final int INT_PLUS = 43;
/*     */   
/*     */   protected static final int INT_DECIMAL_POINT = 46;
/*     */   
/*     */   protected static final int INT_e = 101;
/*     */   
/*     */   protected static final int INT_E = 69;
/*     */   
/*     */   protected static final char CHAR_NULL = '\000';
/*     */   
/*  91 */   protected int _numTypesValid = 0;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected int _numberInt;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected long _numberLong;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected double _numberDouble;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected BigInteger _numberBigInt;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected BigDecimal _numberBigDecimal;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected boolean _numberNegative;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected int _intLength;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected int _fractLength;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected int _expLength;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonNumericParserBase(IOContext ctxt, int features)
/*     */   {
/* 143 */     super(ctxt, features);
/*     */   }
/*     */   
/*     */   protected final JsonToken reset(boolean negative, int intLen, int fractLen, int expLen)
/*     */   {
/* 148 */     if ((fractLen < 1) && (expLen < 1)) {
/* 149 */       return resetInt(negative, intLen);
/*     */     }
/* 151 */     return resetFloat(negative, intLen, fractLen, expLen);
/*     */   }
/*     */   
/*     */   protected final JsonToken resetInt(boolean negative, int intLen)
/*     */   {
/* 156 */     this._numberNegative = negative;
/* 157 */     this._intLength = intLen;
/* 158 */     this._fractLength = 0;
/* 159 */     this._expLength = 0;
/* 160 */     this._numTypesValid = 0;
/* 161 */     return JsonToken.VALUE_NUMBER_INT;
/*     */   }
/*     */   
/*     */   protected final JsonToken resetFloat(boolean negative, int intLen, int fractLen, int expLen)
/*     */   {
/* 166 */     this._numberNegative = negative;
/* 167 */     this._intLength = intLen;
/* 168 */     this._fractLength = fractLen;
/* 169 */     this._expLength = expLen;
/* 170 */     this._numTypesValid = 0;
/* 171 */     return JsonToken.VALUE_NUMBER_FLOAT;
/*     */   }
/*     */   
/*     */   protected final JsonToken resetAsNaN(String valueStr, double value)
/*     */   {
/* 176 */     this._textBuffer.resetWithString(valueStr);
/* 177 */     this._numberDouble = value;
/* 178 */     this._numTypesValid = 8;
/* 179 */     return JsonToken.VALUE_NUMBER_FLOAT;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Number getNumberValue()
/*     */     throws IOException, JsonParseException
/*     */   {
/* 191 */     if (this._numTypesValid == 0) {
/* 192 */       _parseNumericValue(0);
/*     */     }
/*     */     
/* 195 */     if (this._currToken == JsonToken.VALUE_NUMBER_INT) {
/* 196 */       if ((this._numTypesValid & 0x1) != 0) {
/* 197 */         return Integer.valueOf(this._numberInt);
/*     */       }
/* 199 */       if ((this._numTypesValid & 0x2) != 0) {
/* 200 */         return Long.valueOf(this._numberLong);
/*     */       }
/* 202 */       if ((this._numTypesValid & 0x4) != 0) {
/* 203 */         return this._numberBigInt;
/*     */       }
/*     */       
/* 206 */       return this._numberBigDecimal;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 212 */     if ((this._numTypesValid & 0x10) != 0) {
/* 213 */       return this._numberBigDecimal;
/*     */     }
/* 215 */     if ((this._numTypesValid & 0x8) == 0) {
/* 216 */       _throwInternal();
/*     */     }
/* 218 */     return Double.valueOf(this._numberDouble);
/*     */   }
/*     */   
/*     */   public JsonParser.NumberType getNumberType()
/*     */     throws IOException, JsonParseException
/*     */   {
/* 224 */     if (this._numTypesValid == 0) {
/* 225 */       _parseNumericValue(0);
/*     */     }
/* 227 */     if (this._currToken == JsonToken.VALUE_NUMBER_INT) {
/* 228 */       if ((this._numTypesValid & 0x1) != 0) {
/* 229 */         return JsonParser.NumberType.INT;
/*     */       }
/* 231 */       if ((this._numTypesValid & 0x2) != 0) {
/* 232 */         return JsonParser.NumberType.LONG;
/*     */       }
/* 234 */       return JsonParser.NumberType.BIG_INTEGER;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 243 */     if ((this._numTypesValid & 0x10) != 0) {
/* 244 */       return JsonParser.NumberType.BIG_DECIMAL;
/*     */     }
/* 246 */     return JsonParser.NumberType.DOUBLE;
/*     */   }
/*     */   
/*     */   public int getIntValue()
/*     */     throws IOException, JsonParseException
/*     */   {
/* 252 */     if ((this._numTypesValid & 0x1) == 0) {
/* 253 */       if (this._numTypesValid == 0) {
/* 254 */         _parseNumericValue(1);
/*     */       }
/* 256 */       if ((this._numTypesValid & 0x1) == 0) {
/* 257 */         convertNumberToInt();
/*     */       }
/*     */     }
/* 260 */     return this._numberInt;
/*     */   }
/*     */   
/*     */   public long getLongValue()
/*     */     throws IOException, JsonParseException
/*     */   {
/* 266 */     if ((this._numTypesValid & 0x2) == 0) {
/* 267 */       if (this._numTypesValid == 0) {
/* 268 */         _parseNumericValue(2);
/*     */       }
/* 270 */       if ((this._numTypesValid & 0x2) == 0) {
/* 271 */         convertNumberToLong();
/*     */       }
/*     */     }
/* 274 */     return this._numberLong;
/*     */   }
/*     */   
/*     */   public BigInteger getBigIntegerValue()
/*     */     throws IOException, JsonParseException
/*     */   {
/* 280 */     if ((this._numTypesValid & 0x4) == 0) {
/* 281 */       if (this._numTypesValid == 0) {
/* 282 */         _parseNumericValue(4);
/*     */       }
/* 284 */       if ((this._numTypesValid & 0x4) == 0) {
/* 285 */         convertNumberToBigInteger();
/*     */       }
/*     */     }
/* 288 */     return this._numberBigInt;
/*     */   }
/*     */   
/*     */   public float getFloatValue()
/*     */     throws IOException, JsonParseException
/*     */   {
/* 294 */     double value = getDoubleValue();
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 303 */     return (float)value;
/*     */   }
/*     */   
/*     */   public double getDoubleValue()
/*     */     throws IOException, JsonParseException
/*     */   {
/* 309 */     if ((this._numTypesValid & 0x8) == 0) {
/* 310 */       if (this._numTypesValid == 0) {
/* 311 */         _parseNumericValue(8);
/*     */       }
/* 313 */       if ((this._numTypesValid & 0x8) == 0) {
/* 314 */         convertNumberToDouble();
/*     */       }
/*     */     }
/* 317 */     return this._numberDouble;
/*     */   }
/*     */   
/*     */   public BigDecimal getDecimalValue()
/*     */     throws IOException, JsonParseException
/*     */   {
/* 323 */     if ((this._numTypesValid & 0x10) == 0) {
/* 324 */       if (this._numTypesValid == 0) {
/* 325 */         _parseNumericValue(16);
/*     */       }
/* 327 */       if ((this._numTypesValid & 0x10) == 0) {
/* 328 */         convertNumberToBigDecimal();
/*     */       }
/*     */     }
/* 331 */     return this._numberBigDecimal;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void _parseNumericValue(int expType)
/*     */     throws IOException, JsonParseException
/*     */   {
/* 354 */     if (this._currToken == JsonToken.VALUE_NUMBER_INT) {
/* 355 */       char[] buf = this._textBuffer.getTextBuffer();
/* 356 */       int offset = this._textBuffer.getTextOffset();
/* 357 */       int len = this._intLength;
/* 358 */       if (this._numberNegative) {
/* 359 */         offset++;
/*     */       }
/* 361 */       if (len <= 9) {
/* 362 */         int i = NumberInput.parseInt(buf, offset, len);
/* 363 */         this._numberInt = (this._numberNegative ? -i : i);
/* 364 */         this._numTypesValid = 1;
/* 365 */         return;
/*     */       }
/* 367 */       if (len <= 18) {
/* 368 */         long l = NumberInput.parseLong(buf, offset, len);
/* 369 */         if (this._numberNegative) {
/* 370 */           l = -l;
/*     */         }
/*     */         
/* 373 */         if (len == 10) {
/* 374 */           if (this._numberNegative) {
/* 375 */             if (l >= -2147483648L) {
/* 376 */               this._numberInt = ((int)l);
/* 377 */               this._numTypesValid = 1;
/*     */             }
/*     */             
/*     */           }
/* 381 */           else if (l <= 2147483647L) {
/* 382 */             this._numberInt = ((int)l);
/* 383 */             this._numTypesValid = 1;
/* 384 */             return;
/*     */           }
/*     */         }
/*     */         
/* 388 */         this._numberLong = l;
/* 389 */         this._numTypesValid = 2;
/* 390 */         return;
/*     */       }
/* 392 */       _parseSlowIntValue(expType, buf, offset, len);
/* 393 */       return;
/*     */     }
/* 395 */     if (this._currToken == JsonToken.VALUE_NUMBER_FLOAT) {
/* 396 */       _parseSlowFloatValue(expType);
/* 397 */       return;
/*     */     }
/* 399 */     _reportError("Current token (" + this._currToken + ") not numeric, can not use numeric value accessors");
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private final void _parseSlowFloatValue(int expType)
/*     */     throws IOException, JsonParseException
/*     */   {
/*     */     try
/*     */     {
/* 413 */       if (expType == 16) {
/* 414 */         this._numberBigDecimal = this._textBuffer.contentsAsDecimal();
/* 415 */         this._numTypesValid = 16;
/*     */       }
/*     */       else {
/* 418 */         this._numberDouble = this._textBuffer.contentsAsDouble();
/* 419 */         this._numTypesValid = 8;
/*     */       }
/*     */     }
/*     */     catch (NumberFormatException nex) {
/* 423 */       _wrapError("Malformed numeric value '" + this._textBuffer.contentsAsString() + "'", nex);
/*     */     }
/*     */   }
/*     */   
/*     */   private final void _parseSlowIntValue(int expType, char[] buf, int offset, int len)
/*     */     throws IOException, JsonParseException
/*     */   {
/* 430 */     String numStr = this._textBuffer.contentsAsString();
/*     */     try
/*     */     {
/* 433 */       if (NumberInput.inLongRange(buf, offset, len, this._numberNegative))
/*     */       {
/* 435 */         this._numberLong = Long.parseLong(numStr);
/* 436 */         this._numTypesValid = 2;
/*     */       }
/*     */       else {
/* 439 */         this._numberBigInt = new BigInteger(numStr);
/* 440 */         this._numTypesValid = 4;
/*     */       }
/*     */     }
/*     */     catch (NumberFormatException nex) {
/* 444 */       _wrapError("Malformed numeric value '" + numStr + "'", nex);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void convertNumberToInt()
/*     */     throws IOException, JsonParseException
/*     */   {
/* 458 */     if ((this._numTypesValid & 0x2) != 0)
/*     */     {
/* 460 */       int result = (int)this._numberLong;
/* 461 */       if (result != this._numberLong) {
/* 462 */         _reportError("Numeric value (" + getText() + ") out of range of int");
/*     */       }
/* 464 */       this._numberInt = result;
/* 465 */     } else if ((this._numTypesValid & 0x4) != 0)
/*     */     {
/* 467 */       this._numberInt = this._numberBigInt.intValue();
/* 468 */     } else if ((this._numTypesValid & 0x8) != 0)
/*     */     {
/* 470 */       if ((this._numberDouble < -2.147483648E9D) || (this._numberDouble > 2.147483647E9D)) {
/* 471 */         reportOverflowInt();
/*     */       }
/* 473 */       this._numberInt = ((int)this._numberDouble);
/* 474 */     } else if ((this._numTypesValid & 0x10) != 0) {
/* 475 */       if ((BD_MIN_INT.compareTo(this._numberBigDecimal) > 0) || (BD_MAX_INT.compareTo(this._numberBigDecimal) < 0))
/*     */       {
/* 477 */         reportOverflowInt();
/*     */       }
/* 479 */       this._numberInt = this._numberBigDecimal.intValue();
/*     */     } else {
/* 481 */       _throwInternal();
/*     */     }
/*     */     
/* 484 */     this._numTypesValid |= 0x1;
/*     */   }
/*     */   
/*     */   protected void convertNumberToLong()
/*     */     throws IOException, JsonParseException
/*     */   {
/* 490 */     if ((this._numTypesValid & 0x1) != 0) {
/* 491 */       this._numberLong = this._numberInt;
/* 492 */     } else if ((this._numTypesValid & 0x4) != 0)
/*     */     {
/* 494 */       this._numberLong = this._numberBigInt.longValue();
/* 495 */     } else if ((this._numTypesValid & 0x8) != 0)
/*     */     {
/* 497 */       if ((this._numberDouble < -9.223372036854776E18D) || (this._numberDouble > 9.223372036854776E18D)) {
/* 498 */         reportOverflowLong();
/*     */       }
/* 500 */       this._numberLong = (this._numberDouble);
/* 501 */     } else if ((this._numTypesValid & 0x10) != 0) {
/* 502 */       if ((BD_MIN_LONG.compareTo(this._numberBigDecimal) > 0) || (BD_MAX_LONG.compareTo(this._numberBigDecimal) < 0))
/*     */       {
/* 504 */         reportOverflowLong();
/*     */       }
/* 506 */       this._numberLong = this._numberBigDecimal.longValue();
/*     */     } else {
/* 508 */       _throwInternal();
/*     */     }
/*     */     
/* 511 */     this._numTypesValid |= 0x2;
/*     */   }
/*     */   
/*     */   protected void convertNumberToBigInteger()
/*     */     throws IOException, JsonParseException
/*     */   {
/* 517 */     if ((this._numTypesValid & 0x10) != 0)
/*     */     {
/* 519 */       this._numberBigInt = this._numberBigDecimal.toBigInteger();
/* 520 */     } else if ((this._numTypesValid & 0x2) != 0) {
/* 521 */       this._numberBigInt = BigInteger.valueOf(this._numberLong);
/* 522 */     } else if ((this._numTypesValid & 0x1) != 0) {
/* 523 */       this._numberBigInt = BigInteger.valueOf(this._numberInt);
/* 524 */     } else if ((this._numTypesValid & 0x8) != 0) {
/* 525 */       this._numberBigInt = BigDecimal.valueOf(this._numberDouble).toBigInteger();
/*     */     } else {
/* 527 */       _throwInternal();
/*     */     }
/* 529 */     this._numTypesValid |= 0x4;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void convertNumberToDouble()
/*     */     throws IOException, JsonParseException
/*     */   {
/* 541 */     if ((this._numTypesValid & 0x10) != 0) {
/* 542 */       this._numberDouble = this._numberBigDecimal.doubleValue();
/* 543 */     } else if ((this._numTypesValid & 0x4) != 0) {
/* 544 */       this._numberDouble = this._numberBigInt.doubleValue();
/* 545 */     } else if ((this._numTypesValid & 0x2) != 0) {
/* 546 */       this._numberDouble = this._numberLong;
/* 547 */     } else if ((this._numTypesValid & 0x1) != 0) {
/* 548 */       this._numberDouble = this._numberInt;
/*     */     } else {
/* 550 */       _throwInternal();
/*     */     }
/*     */     
/* 553 */     this._numTypesValid |= 0x8;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void convertNumberToBigDecimal()
/*     */     throws IOException, JsonParseException
/*     */   {
/* 565 */     if ((this._numTypesValid & 0x8) != 0)
/*     */     {
/*     */ 
/*     */ 
/*     */ 
/* 570 */       this._numberBigDecimal = new BigDecimal(getText());
/* 571 */     } else if ((this._numTypesValid & 0x4) != 0) {
/* 572 */       this._numberBigDecimal = new BigDecimal(this._numberBigInt);
/* 573 */     } else if ((this._numTypesValid & 0x2) != 0) {
/* 574 */       this._numberBigDecimal = BigDecimal.valueOf(this._numberLong);
/* 575 */     } else if ((this._numTypesValid & 0x1) != 0) {
/* 576 */       this._numberBigDecimal = BigDecimal.valueOf(this._numberInt);
/*     */     } else {
/* 578 */       _throwInternal();
/*     */     }
/* 580 */     this._numTypesValid |= 0x10;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void reportUnexpectedNumberChar(int ch, String comment)
/*     */     throws JsonParseException
/*     */   {
/* 592 */     String msg = "Unexpected character (" + _getCharDesc(ch) + ") in numeric value";
/* 593 */     if (comment != null) {
/* 594 */       msg = msg + ": " + comment;
/*     */     }
/* 596 */     _reportError(msg);
/*     */   }
/*     */   
/*     */   protected void reportInvalidNumber(String msg)
/*     */     throws JsonParseException
/*     */   {
/* 602 */     _reportError("Invalid numeric value: " + msg);
/*     */   }
/*     */   
/*     */ 
/*     */   protected void reportOverflowInt()
/*     */     throws IOException, JsonParseException
/*     */   {
/* 609 */     _reportError("Numeric value (" + getText() + ") out of range of int (" + Integer.MIN_VALUE + " - " + Integer.MAX_VALUE + ")");
/*     */   }
/*     */   
/*     */   protected void reportOverflowLong()
/*     */     throws IOException, JsonParseException
/*     */   {
/* 615 */     _reportError("Numeric value (" + getText() + ") out of range of long (" + Long.MIN_VALUE + " - " + Long.MAX_VALUE + ")");
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\impl\JsonNumericParserBase.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */