/*     */ package org.codehaus.jackson.map;
/*     */ 
/*     */ import java.text.DateFormat;
/*     */ import java.util.HashMap;
/*     */ import java.util.Map;
/*     */ import java.util.Map.Entry;
/*     */ import org.codehaus.jackson.map.introspect.Annotated;
/*     */ import org.codehaus.jackson.map.introspect.VisibilityChecker;
/*     */ import org.codehaus.jackson.map.jsontype.SubtypeResolver;
/*     */ import org.codehaus.jackson.map.jsontype.TypeIdResolver;
/*     */ import org.codehaus.jackson.map.jsontype.TypeResolverBuilder;
/*     */ import org.codehaus.jackson.map.jsontype.impl.StdSubtypeResolver;
/*     */ import org.codehaus.jackson.map.type.ClassKey;
/*     */ import org.codehaus.jackson.map.type.TypeFactory;
/*     */ import org.codehaus.jackson.map.util.ClassUtil;
/*     */ import org.codehaus.jackson.map.util.StdDateFormat;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class MapperConfig<T extends MapperConfig<T>>
/*     */   implements ClassIntrospector.MixInResolver
/*     */ {
/*  42 */   protected static final DateFormat DEFAULT_DATE_FORMAT = StdDateFormat.instance;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected Base _base;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected HashMap<ClassKey, Class<?>> _mixInAnnotations;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected boolean _mixInAnnotationsShared;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected SubtypeResolver _subtypeResolver;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected MapperConfig(ClassIntrospector<? extends BeanDescription> ci, AnnotationIntrospector ai, VisibilityChecker<?> vc, SubtypeResolver str, PropertyNamingStrategy pns, TypeFactory tf, HandlerInstantiator hi)
/*     */   {
/* 119 */     this._base = new Base(ci, ai, vc, pns, tf, null, DEFAULT_DATE_FORMAT, hi);
/* 120 */     this._subtypeResolver = str;
/*     */     
/* 122 */     this._mixInAnnotationsShared = true;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected MapperConfig(MapperConfig<?> src)
/*     */   {
/* 131 */     this(src, src._base, src._subtypeResolver);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected MapperConfig(MapperConfig<?> src, Base base, SubtypeResolver str)
/*     */   {
/* 139 */     this._base = base;
/* 140 */     this._subtypeResolver = str;
/*     */     
/* 142 */     this._mixInAnnotationsShared = true;
/* 143 */     this._mixInAnnotations = src._mixInAnnotations;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public abstract void fromAnnotations(Class<?> paramClass);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public abstract T createUnshared(SubtypeResolver paramSubtypeResolver);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public abstract T withClassIntrospector(ClassIntrospector<? extends BeanDescription> paramClassIntrospector);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public abstract T withAnnotationIntrospector(AnnotationIntrospector paramAnnotationIntrospector);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public abstract T withVisibilityChecker(VisibilityChecker<?> paramVisibilityChecker);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public abstract T withTypeResolverBuilder(TypeResolverBuilder<?> paramTypeResolverBuilder);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public abstract T withSubtypeResolver(SubtypeResolver paramSubtypeResolver);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public abstract T withPropertyNamingStrategy(PropertyNamingStrategy paramPropertyNamingStrategy);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public abstract T withTypeFactory(TypeFactory paramTypeFactory);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public abstract T withDateFormat(DateFormat paramDateFormat);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public abstract T withHandlerInstantiator(HandlerInstantiator paramHandlerInstantiator);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public ClassIntrospector<? extends BeanDescription> getClassIntrospector()
/*     */   {
/* 287 */     return this._base.getClassIntrospector();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public AnnotationIntrospector getAnnotationIntrospector()
/*     */   {
/* 297 */     return this._base.getAnnotationIntrospector();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final void insertAnnotationIntrospector(AnnotationIntrospector introspector)
/*     */   {
/* 310 */     this._base = this._base.withAnnotationIntrospector(AnnotationIntrospector.Pair.create(introspector, getAnnotationIntrospector()));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final void appendAnnotationIntrospector(AnnotationIntrospector introspector)
/*     */   {
/* 324 */     this._base = this._base.withAnnotationIntrospector(AnnotationIntrospector.Pair.create(getAnnotationIntrospector(), introspector));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final VisibilityChecker<?> getDefaultVisibilityChecker()
/*     */   {
/* 340 */     return this._base.getVisibilityChecker();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public final PropertyNamingStrategy getPropertyNamingStrategy()
/*     */   {
/* 347 */     return this._base.getPropertyNamingStrategy();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public final HandlerInstantiator getHandlerInstantiator()
/*     */   {
/* 354 */     return this._base.getHandlerInstantiator();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final void setMixInAnnotations(Map<Class<?>, Class<?>> sourceMixins)
/*     */   {
/* 379 */     HashMap<ClassKey, Class<?>> mixins = null;
/* 380 */     if ((sourceMixins != null) && (sourceMixins.size() > 0)) {
/* 381 */       mixins = new HashMap(sourceMixins.size());
/* 382 */       for (Map.Entry<Class<?>, Class<?>> en : sourceMixins.entrySet()) {
/* 383 */         mixins.put(new ClassKey((Class)en.getKey()), en.getValue());
/*     */       }
/*     */     }
/* 386 */     this._mixInAnnotationsShared = false;
/* 387 */     this._mixInAnnotations = mixins;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final void addMixInAnnotations(Class<?> target, Class<?> mixinSource)
/*     */   {
/* 404 */     if (this._mixInAnnotations == null) {
/* 405 */       this._mixInAnnotationsShared = false;
/* 406 */       this._mixInAnnotations = new HashMap();
/* 407 */     } else if (this._mixInAnnotationsShared) {
/* 408 */       this._mixInAnnotationsShared = false;
/* 409 */       this._mixInAnnotations = new HashMap(this._mixInAnnotations);
/*     */     }
/* 411 */     this._mixInAnnotations.put(new ClassKey(target), mixinSource);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final Class<?> findMixInClassFor(Class<?> cls)
/*     */   {
/* 423 */     return this._mixInAnnotations == null ? null : (Class)this._mixInAnnotations.get(new ClassKey(cls));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public final int mixInCount()
/*     */   {
/* 430 */     return this._mixInAnnotations == null ? 0 : this._mixInAnnotations.size();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final TypeResolverBuilder<?> getDefaultTyper(JavaType baseType)
/*     */   {
/* 448 */     return this._base.getTypeResolverBuilder();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final SubtypeResolver getSubtypeResolver()
/*     */   {
/* 459 */     if (this._subtypeResolver == null) {
/* 460 */       this._subtypeResolver = new StdSubtypeResolver();
/*     */     }
/* 462 */     return this._subtypeResolver;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public final TypeFactory getTypeFactory()
/*     */   {
/* 469 */     return this._base.getTypeFactory();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final JavaType constructType(Class<?> cls)
/*     */   {
/* 483 */     return getTypeFactory().constructType(cls);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final DateFormat getDateFormat()
/*     */   {
/* 508 */     return this._base.getDateFormat();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public abstract <DESC extends BeanDescription> DESC introspectClassAnnotations(Class<?> paramClass);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public abstract <DESC extends BeanDescription> DESC introspectDirectClassAnnotations(Class<?> paramClass);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public abstract boolean isAnnotationProcessingEnabled();
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public abstract boolean canOverrideAccessModifiers();
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public TypeResolverBuilder<?> typeResolverBuilderInstance(Annotated annotated, Class<? extends TypeResolverBuilder<?>> builderClass)
/*     */   {
/* 565 */     HandlerInstantiator hi = getHandlerInstantiator();
/* 566 */     if (hi != null) {
/* 567 */       TypeResolverBuilder<?> builder = hi.typeResolverBuilderInstance(this, annotated, builderClass);
/* 568 */       if (builder != null) {
/* 569 */         return builder;
/*     */       }
/*     */     }
/* 572 */     return (TypeResolverBuilder)ClassUtil.createInstance(builderClass, canOverrideAccessModifiers());
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public TypeIdResolver typeIdResolverInstance(Annotated annotated, Class<? extends TypeIdResolver> resolverClass)
/*     */   {
/* 584 */     HandlerInstantiator hi = getHandlerInstantiator();
/* 585 */     if (hi != null) {
/* 586 */       TypeIdResolver builder = hi.typeIdResolverInstance(this, annotated, resolverClass);
/* 587 */       if (builder != null) {
/* 588 */         return builder;
/*     */       }
/*     */     }
/* 591 */     return (TypeIdResolver)ClassUtil.createInstance(resolverClass, canOverrideAccessModifiers());
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @Deprecated
/*     */   public abstract T createUnshared(TypeResolverBuilder<?> paramTypeResolverBuilder, VisibilityChecker<?> paramVisibilityChecker, SubtypeResolver paramSubtypeResolver);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @Deprecated
/*     */   public final void setIntrospector(ClassIntrospector<? extends BeanDescription> ci)
/*     */   {
/* 615 */     this._base = this._base.withClassIntrospector(ci);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @Deprecated
/*     */   public final void setAnnotationIntrospector(AnnotationIntrospector ai)
/*     */   {
/* 628 */     this._base = this._base.withAnnotationIntrospector(ai);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @Deprecated
/*     */   public void setDateFormat(DateFormat df)
/*     */   {
/* 644 */     if (df == null) {
/* 645 */       df = StdDateFormat.instance;
/*     */     }
/* 647 */     this._base = this._base.withDateFormat(df);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @Deprecated
/*     */   public final void setSubtypeResolver(SubtypeResolver str)
/*     */   {
/* 659 */     this._subtypeResolver = str;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static class Base
/*     */   {
/*     */     protected final ClassIntrospector<? extends BeanDescription> _classIntrospector;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     protected final AnnotationIntrospector _annotationIntrospector;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     protected final VisibilityChecker<?> _visibilityChecker;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     protected final PropertyNamingStrategy _propertyNamingStrategy;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     protected final TypeFactory _typeFactory;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     protected final TypeResolverBuilder<?> _typeResolverBuilder;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     protected final DateFormat _dateFormat;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     protected final HandlerInstantiator _handlerInstantiator;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public Base(ClassIntrospector<? extends BeanDescription> ci, AnnotationIntrospector ai, VisibilityChecker<?> vc, PropertyNamingStrategy pns, TypeFactory tf, TypeResolverBuilder<?> typer, DateFormat dateFormat, HandlerInstantiator hi)
/*     */     {
/* 768 */       this._classIntrospector = ci;
/* 769 */       this._annotationIntrospector = ai;
/* 770 */       this._visibilityChecker = vc;
/* 771 */       this._propertyNamingStrategy = pns;
/* 772 */       this._typeFactory = tf;
/* 773 */       this._typeResolverBuilder = typer;
/* 774 */       this._dateFormat = dateFormat;
/* 775 */       this._handlerInstantiator = hi;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public Base withClassIntrospector(ClassIntrospector<? extends BeanDescription> ci)
/*     */     {
/* 785 */       return new Base(ci, this._annotationIntrospector, this._visibilityChecker, this._propertyNamingStrategy, this._typeFactory, this._typeResolverBuilder, this._dateFormat, this._handlerInstantiator);
/*     */     }
/*     */     
/*     */     public Base withAnnotationIntrospector(AnnotationIntrospector ai)
/*     */     {
/* 790 */       return new Base(this._classIntrospector, ai, this._visibilityChecker, this._propertyNamingStrategy, this._typeFactory, this._typeResolverBuilder, this._dateFormat, this._handlerInstantiator);
/*     */     }
/*     */     
/*     */     public Base withVisibilityChecker(VisibilityChecker<?> vc)
/*     */     {
/* 795 */       return new Base(this._classIntrospector, this._annotationIntrospector, vc, this._propertyNamingStrategy, this._typeFactory, this._typeResolverBuilder, this._dateFormat, this._handlerInstantiator);
/*     */     }
/*     */     
/*     */     public Base withPropertyNamingStrategy(PropertyNamingStrategy pns)
/*     */     {
/* 800 */       return new Base(this._classIntrospector, this._annotationIntrospector, this._visibilityChecker, pns, this._typeFactory, this._typeResolverBuilder, this._dateFormat, this._handlerInstantiator);
/*     */     }
/*     */     
/*     */     public Base withTypeFactory(TypeFactory tf)
/*     */     {
/* 805 */       return new Base(this._classIntrospector, this._annotationIntrospector, this._visibilityChecker, this._propertyNamingStrategy, tf, this._typeResolverBuilder, this._dateFormat, this._handlerInstantiator);
/*     */     }
/*     */     
/*     */     public Base withTypeResolverBuilder(TypeResolverBuilder<?> typer)
/*     */     {
/* 810 */       return new Base(this._classIntrospector, this._annotationIntrospector, this._visibilityChecker, this._propertyNamingStrategy, this._typeFactory, typer, this._dateFormat, this._handlerInstantiator);
/*     */     }
/*     */     
/*     */     public Base withDateFormat(DateFormat df)
/*     */     {
/* 815 */       return new Base(this._classIntrospector, this._annotationIntrospector, this._visibilityChecker, this._propertyNamingStrategy, this._typeFactory, this._typeResolverBuilder, df, this._handlerInstantiator);
/*     */     }
/*     */     
/*     */     public Base withHandlerInstantiator(HandlerInstantiator hi)
/*     */     {
/* 820 */       return new Base(this._classIntrospector, this._annotationIntrospector, this._visibilityChecker, this._propertyNamingStrategy, this._typeFactory, this._typeResolverBuilder, this._dateFormat, hi);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public ClassIntrospector<? extends BeanDescription> getClassIntrospector()
/*     */     {
/* 831 */       return this._classIntrospector;
/*     */     }
/*     */     
/*     */     public AnnotationIntrospector getAnnotationIntrospector() {
/* 835 */       return this._annotationIntrospector;
/*     */     }
/*     */     
/*     */     public VisibilityChecker<?> getVisibilityChecker()
/*     */     {
/* 840 */       return this._visibilityChecker;
/*     */     }
/*     */     
/*     */     public PropertyNamingStrategy getPropertyNamingStrategy() {
/* 844 */       return this._propertyNamingStrategy;
/*     */     }
/*     */     
/*     */     public TypeFactory getTypeFactory() {
/* 848 */       return this._typeFactory;
/*     */     }
/*     */     
/*     */     public TypeResolverBuilder<?> getTypeResolverBuilder() {
/* 852 */       return this._typeResolverBuilder;
/*     */     }
/*     */     
/*     */     public DateFormat getDateFormat() {
/* 856 */       return this._dateFormat;
/*     */     }
/*     */     
/*     */     public HandlerInstantiator getHandlerInstantiator() {
/* 860 */       return this._handlerInstantiator;
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\MapperConfig.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */