/*      */ package org.codehaus.jackson.map;
/*      */ 
/*      */ import java.io.Closeable;
/*      */ import java.io.EOFException;
/*      */ import java.io.File;
/*      */ import java.io.IOException;
/*      */ import java.io.InputStream;
/*      */ import java.io.OutputStream;
/*      */ import java.io.Reader;
/*      */ import java.io.Writer;
/*      */ import java.lang.reflect.Type;
/*      */ import java.net.URL;
/*      */ import java.text.DateFormat;
/*      */ import java.util.Collection;
/*      */ import java.util.concurrent.ConcurrentHashMap;
/*      */ import org.codehaus.jackson.FormatSchema;
/*      */ import org.codehaus.jackson.JsonEncoding;
/*      */ import org.codehaus.jackson.JsonFactory;
/*      */ import org.codehaus.jackson.JsonGenerationException;
/*      */ import org.codehaus.jackson.JsonGenerator;
/*      */ import org.codehaus.jackson.JsonGenerator.Feature;
/*      */ import org.codehaus.jackson.JsonNode;
/*      */ import org.codehaus.jackson.JsonParseException;
/*      */ import org.codehaus.jackson.JsonParser;
/*      */ import org.codehaus.jackson.JsonParser.Feature;
/*      */ import org.codehaus.jackson.JsonProcessingException;
/*      */ import org.codehaus.jackson.JsonToken;
/*      */ import org.codehaus.jackson.ObjectCodec;
/*      */ import org.codehaus.jackson.PrettyPrinter;
/*      */ import org.codehaus.jackson.Version;
/*      */ import org.codehaus.jackson.Versioned;
/*      */ import org.codehaus.jackson.annotate.JsonTypeInfo.As;
/*      */ import org.codehaus.jackson.annotate.JsonTypeInfo.Id;
/*      */ import org.codehaus.jackson.io.SegmentedStringWriter;
/*      */ import org.codehaus.jackson.map.deser.BeanDeserializerModifier;
/*      */ import org.codehaus.jackson.map.deser.StdDeserializationContext;
/*      */ import org.codehaus.jackson.map.deser.StdDeserializerProvider;
/*      */ import org.codehaus.jackson.map.introspect.BasicClassIntrospector;
/*      */ import org.codehaus.jackson.map.introspect.JacksonAnnotationIntrospector;
/*      */ import org.codehaus.jackson.map.introspect.VisibilityChecker;
/*      */ import org.codehaus.jackson.map.introspect.VisibilityChecker.Std;
/*      */ import org.codehaus.jackson.map.jsontype.NamedType;
/*      */ import org.codehaus.jackson.map.jsontype.SubtypeResolver;
/*      */ import org.codehaus.jackson.map.jsontype.TypeResolverBuilder;
/*      */ import org.codehaus.jackson.map.jsontype.impl.StdSubtypeResolver;
/*      */ import org.codehaus.jackson.map.jsontype.impl.StdTypeResolverBuilder;
/*      */ import org.codehaus.jackson.map.ser.BeanSerializerFactory;
/*      */ import org.codehaus.jackson.map.ser.BeanSerializerModifier;
/*      */ import org.codehaus.jackson.map.ser.FilterProvider;
/*      */ import org.codehaus.jackson.map.ser.StdSerializerProvider;
/*      */ import org.codehaus.jackson.map.type.SimpleType;
/*      */ import org.codehaus.jackson.map.type.TypeFactory;
/*      */ import org.codehaus.jackson.map.type.TypeModifier;
/*      */ import org.codehaus.jackson.node.ArrayNode;
/*      */ import org.codehaus.jackson.node.JsonNodeFactory;
/*      */ import org.codehaus.jackson.node.NullNode;
/*      */ import org.codehaus.jackson.node.ObjectNode;
/*      */ import org.codehaus.jackson.node.TreeTraversingParser;
/*      */ import org.codehaus.jackson.schema.JsonSchema;
/*      */ import org.codehaus.jackson.type.JavaType;
/*      */ import org.codehaus.jackson.type.TypeReference;
/*      */ import org.codehaus.jackson.util.ByteArrayBuilder;
/*      */ import org.codehaus.jackson.util.DefaultPrettyPrinter;
/*      */ import org.codehaus.jackson.util.TokenBuffer;
/*      */ import org.codehaus.jackson.util.VersionUtil;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class ObjectMapper
/*      */   extends ObjectCodec
/*      */   implements Versioned
/*      */ {
/*      */   public static enum DefaultTyping
/*      */   {
/*   88 */     JAVA_LANG_OBJECT, 
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*   96 */     OBJECT_AND_NON_CONCRETE, 
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  103 */     NON_CONCRETE_AND_ARRAYS, 
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  112 */     NON_FINAL;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     private DefaultTyping() {}
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public static class DefaultTypeResolverBuilder
/*      */     extends StdTypeResolverBuilder
/*      */   {
/*      */     protected final ObjectMapper.DefaultTyping _appliesFor;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */     public DefaultTypeResolverBuilder(ObjectMapper.DefaultTyping t)
/*      */     {
/*  134 */       this._appliesFor = t;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */     public TypeDeserializer buildTypeDeserializer(DeserializationConfig config, JavaType baseType, Collection<NamedType> subtypes, BeanProperty property)
/*      */     {
/*  141 */       return useForType(baseType) ? super.buildTypeDeserializer(config, baseType, subtypes, property) : null;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */     public TypeSerializer buildTypeSerializer(SerializationConfig config, JavaType baseType, Collection<NamedType> subtypes, BeanProperty property)
/*      */     {
/*  148 */       return useForType(baseType) ? super.buildTypeSerializer(config, baseType, subtypes, property) : null;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     public boolean useForType(JavaType t)
/*      */     {
/*  161 */       switch (ObjectMapper.2.$SwitchMap$org$codehaus$jackson$map$ObjectMapper$DefaultTyping[this._appliesFor.ordinal()]) {
/*      */       case 1: 
/*  163 */         if (t.isArrayType()) {
/*  164 */           t = t.getContentType();
/*      */         }
/*      */       
/*      */       case 2: 
/*  168 */         return (t.getRawClass() == Object.class) || (!t.isConcrete());
/*      */       case 3: 
/*  170 */         if (t.isArrayType()) {
/*  171 */           t = t.getContentType();
/*      */         }
/*  173 */         return !t.isFinal();
/*      */       }
/*      */       
/*  176 */       return t.getRawClass() == Object.class;
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  188 */   private static final JavaType JSON_NODE_TYPE = SimpleType.constructUnsafe(JsonNode.class);
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*  193 */   protected static final ClassIntrospector<? extends BeanDescription> DEFAULT_INTROSPECTOR = BasicClassIntrospector.instance;
/*      */   
/*      */ 
/*  196 */   protected static final AnnotationIntrospector DEFAULT_ANNOTATION_INTROSPECTOR = new JacksonAnnotationIntrospector();
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*  201 */   protected static final VisibilityChecker<?> STD_VISIBILITY_CHECKER = VisibilityChecker.Std.defaultInstance();
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected final JsonFactory _jsonFactory;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected SubtypeResolver _subtypeResolver;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected TypeFactory _typeFactory;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected SerializationConfig _serializationConfig;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected SerializerProvider _serializerProvider;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected SerializerFactory _serializerFactory;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected DeserializationConfig _deserializationConfig;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected DeserializerProvider _deserializerProvider;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  303 */   protected final ConcurrentHashMap<JavaType, JsonDeserializer<Object>> _rootDeserializers = new ConcurrentHashMap(64, 0.6F, 2);
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper()
/*      */   {
/*  326 */     this(null, null, null);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper(JsonFactory jf)
/*      */   {
/*  336 */     this(jf, null, null);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   @Deprecated
/*      */   public ObjectMapper(SerializerFactory sf)
/*      */   {
/*  349 */     this(null, null, null);
/*  350 */     setSerializerFactory(sf);
/*      */   }
/*      */   
/*      */ 
/*      */   public ObjectMapper(JsonFactory jf, SerializerProvider sp, DeserializerProvider dp)
/*      */   {
/*  356 */     this(jf, sp, dp, null, null);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper(JsonFactory jf, SerializerProvider sp, DeserializerProvider dp, SerializationConfig sconfig, DeserializationConfig dconfig)
/*      */   {
/*  379 */     if (jf == null) {
/*  380 */       this._jsonFactory = new MappingJsonFactory(this);
/*      */     } else {
/*  382 */       this._jsonFactory = jf;
/*  383 */       if (jf.getCodec() == null) {
/*  384 */         this._jsonFactory.setCodec(this);
/*      */       }
/*      */     }
/*      */     
/*  388 */     this._typeFactory = TypeFactory.defaultInstance();
/*  389 */     this._serializationConfig = (sconfig != null ? sconfig : new SerializationConfig(DEFAULT_INTROSPECTOR, DEFAULT_ANNOTATION_INTROSPECTOR, STD_VISIBILITY_CHECKER, null, null, this._typeFactory, null));
/*      */     
/*      */ 
/*  392 */     this._deserializationConfig = (dconfig != null ? dconfig : new DeserializationConfig(DEFAULT_INTROSPECTOR, DEFAULT_ANNOTATION_INTROSPECTOR, STD_VISIBILITY_CHECKER, null, null, this._typeFactory, null));
/*      */     
/*      */ 
/*  395 */     this._serializerProvider = (sp == null ? new StdSerializerProvider() : sp);
/*  396 */     this._deserializerProvider = (dp == null ? new StdDeserializerProvider() : dp);
/*      */     
/*      */ 
/*  399 */     this._serializerFactory = BeanSerializerFactory.instance;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Version version()
/*      */   {
/*  415 */     return VersionUtil.versionFor(getClass());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void registerModule(Module module)
/*      */   {
/*  439 */     String name = module.getModuleName();
/*  440 */     if (name == null) {
/*  441 */       throw new IllegalArgumentException("Module without defined name");
/*      */     }
/*  443 */     Version version = module.version();
/*  444 */     if (version == null) {
/*  445 */       throw new IllegalArgumentException("Module without defined version");
/*      */     }
/*      */     
/*  448 */     final ObjectMapper mapper = this;
/*      */     
/*      */ 
/*  451 */     module.setupModule(new Module.SetupContext()
/*      */     {
/*      */ 
/*      */       public Version getMapperVersion()
/*      */       {
/*      */ 
/*  457 */         return ObjectMapper.this.version();
/*      */       }
/*      */       
/*      */       public DeserializationConfig getDeserializationConfig() {
/*  461 */         return mapper.getDeserializationConfig();
/*      */       }
/*      */       
/*      */       public SerializationConfig getSerializationConfig() {
/*  465 */         return mapper.getSerializationConfig();
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */       public void addDeserializers(Deserializers d)
/*      */       {
/*  472 */         mapper._deserializerProvider = mapper._deserializerProvider.withAdditionalDeserializers(d);
/*      */       }
/*      */       
/*      */       public void addKeyDeserializers(KeyDeserializers d)
/*      */       {
/*  477 */         mapper._deserializerProvider = mapper._deserializerProvider.withAdditionalKeyDeserializers(d);
/*      */       }
/*      */       
/*      */       public void addSerializers(Serializers s)
/*      */       {
/*  482 */         mapper._serializerFactory = mapper._serializerFactory.withAdditionalSerializers(s);
/*      */       }
/*      */       
/*      */       public void addKeySerializers(Serializers s)
/*      */       {
/*  487 */         mapper._serializerFactory = mapper._serializerFactory.withAdditionalKeySerializers(s);
/*      */       }
/*      */       
/*      */       public void addBeanSerializerModifier(BeanSerializerModifier modifier)
/*      */       {
/*  492 */         mapper._serializerFactory = mapper._serializerFactory.withSerializerModifier(modifier);
/*      */       }
/*      */       
/*      */       public void addBeanDeserializerModifier(BeanDeserializerModifier modifier)
/*      */       {
/*  497 */         mapper._deserializerProvider = mapper._deserializerProvider.withDeserializerModifier(modifier);
/*      */       }
/*      */       
/*      */       public void addAbstractTypeResolver(AbstractTypeResolver resolver)
/*      */       {
/*  502 */         mapper._deserializerProvider = mapper._deserializerProvider.withAbstractTypeResolver(resolver);
/*      */       }
/*      */       
/*      */       public void addTypeModifier(TypeModifier modifier)
/*      */       {
/*  507 */         TypeFactory f = mapper._typeFactory;
/*  508 */         f = f.withModifier(modifier);
/*  509 */         mapper.setTypeFactory(f);
/*      */       }
/*      */       
/*      */       public void insertAnnotationIntrospector(AnnotationIntrospector ai)
/*      */       {
/*  514 */         mapper._deserializationConfig.insertAnnotationIntrospector(ai);
/*  515 */         mapper._serializationConfig.insertAnnotationIntrospector(ai);
/*      */       }
/*      */       
/*      */       public void appendAnnotationIntrospector(AnnotationIntrospector ai)
/*      */       {
/*  520 */         mapper._deserializationConfig.appendAnnotationIntrospector(ai);
/*  521 */         mapper._serializationConfig.appendAnnotationIntrospector(ai);
/*      */       }
/*      */       
/*      */       public void setMixInAnnotations(Class<?> target, Class<?> mixinSource)
/*      */       {
/*  526 */         mapper._deserializationConfig.addMixInAnnotations(target, mixinSource);
/*  527 */         mapper._serializationConfig.addMixInAnnotations(target, mixinSource);
/*      */       }
/*      */     });
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper withModule(Module module)
/*      */   {
/*  543 */     registerModule(module);
/*  544 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public SerializationConfig getSerializationConfig()
/*      */   {
/*  561 */     return this._serializationConfig;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public SerializationConfig copySerializationConfig()
/*      */   {
/*  578 */     return this._serializationConfig.createUnshared(this._subtypeResolver);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper setSerializationConfig(SerializationConfig cfg)
/*      */   {
/*  586 */     this._serializationConfig = cfg;
/*  587 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public DeserializationConfig getDeserializationConfig()
/*      */   {
/*  599 */     return this._deserializationConfig;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public DeserializationConfig copyDeserializationConfig()
/*      */   {
/*  617 */     return this._deserializationConfig.createUnshared(this._subtypeResolver);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper setDeserializationConfig(DeserializationConfig cfg)
/*      */   {
/*  625 */     this._deserializationConfig = cfg;
/*  626 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper setSerializerFactory(SerializerFactory f)
/*      */   {
/*  640 */     this._serializerFactory = f;
/*  641 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper setSerializerProvider(SerializerProvider p)
/*      */   {
/*  649 */     this._serializerProvider = p;
/*  650 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public SerializerProvider getSerializerProvider()
/*      */   {
/*  657 */     return this._serializerProvider;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper setDeserializerProvider(DeserializerProvider p)
/*      */   {
/*  665 */     this._deserializerProvider = p;
/*  666 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public DeserializerProvider getDeserializerProvider()
/*      */   {
/*  673 */     return this._deserializerProvider;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public VisibilityChecker<?> getVisibilityChecker()
/*      */   {
/*  690 */     return this._serializationConfig.getDefaultVisibilityChecker();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void setVisibilityChecker(VisibilityChecker<?> vc)
/*      */   {
/*  703 */     this._deserializationConfig = this._deserializationConfig.withVisibilityChecker(vc);
/*  704 */     this._serializationConfig = this._serializationConfig.withVisibilityChecker(vc);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public SubtypeResolver getSubtypeResolver()
/*      */   {
/*  711 */     if (this._subtypeResolver == null) {
/*  712 */       this._subtypeResolver = new StdSubtypeResolver();
/*      */     }
/*  714 */     return this._subtypeResolver;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void setSubtypeResolver(SubtypeResolver r)
/*      */   {
/*  723 */     this._subtypeResolver = r;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper setAnnotationIntrospector(AnnotationIntrospector ai)
/*      */   {
/*  733 */     this._serializationConfig = this._serializationConfig.withAnnotationIntrospector(ai);
/*  734 */     this._deserializationConfig = this._deserializationConfig.withAnnotationIntrospector(ai);
/*  735 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper setPropertyNamingStrategy(PropertyNamingStrategy s)
/*      */   {
/*  744 */     this._serializationConfig = this._serializationConfig.withPropertyNamingStrategy(s);
/*  745 */     this._deserializationConfig = this._deserializationConfig.withPropertyNamingStrategy(s);
/*  746 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper enableDefaultTyping()
/*      */   {
/*  762 */     return enableDefaultTyping(DefaultTyping.OBJECT_AND_NON_CONCRETE);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper enableDefaultTyping(DefaultTyping dti)
/*      */   {
/*  772 */     return enableDefaultTyping(dti, JsonTypeInfo.As.WRAPPER_ARRAY);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper enableDefaultTyping(DefaultTyping applicability, JsonTypeInfo.As includeAs)
/*      */   {
/*  785 */     TypeResolverBuilder<?> typer = new DefaultTypeResolverBuilder(applicability);
/*      */     
/*  787 */     typer = typer.init(JsonTypeInfo.Id.CLASS, null);
/*  788 */     typer = typer.inclusion(includeAs);
/*  789 */     return setDefaultTyping(typer);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper enableDefaultTypingAsProperty(DefaultTyping applicability, String propertyName)
/*      */   {
/*  804 */     TypeResolverBuilder<?> typer = new DefaultTypeResolverBuilder(applicability);
/*      */     
/*  806 */     typer = typer.init(JsonTypeInfo.Id.CLASS, null);
/*  807 */     typer = typer.inclusion(JsonTypeInfo.As.PROPERTY);
/*  808 */     typer = typer.typeProperty(propertyName);
/*  809 */     return setDefaultTyping(typer);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper disableDefaultTyping()
/*      */   {
/*  819 */     return setDefaultTyping(null);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper setDefaultTyping(TypeResolverBuilder<?> typer)
/*      */   {
/*  832 */     this._deserializationConfig = this._deserializationConfig.withTypeResolverBuilder(typer);
/*  833 */     this._serializationConfig = this._serializationConfig.withTypeResolverBuilder(typer);
/*  834 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void registerSubtypes(Class<?>... classes)
/*      */   {
/*  847 */     getSubtypeResolver().registerSubtypes(classes);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void registerSubtypes(NamedType... types)
/*      */   {
/*  861 */     getSubtypeResolver().registerSubtypes(types);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public TypeFactory getTypeFactory()
/*      */   {
/*  876 */     return this._typeFactory;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper setTypeFactory(TypeFactory f)
/*      */   {
/*  888 */     this._typeFactory = f;
/*  889 */     this._deserializationConfig = this._deserializationConfig.withTypeFactory(f);
/*  890 */     this._serializationConfig = this._serializationConfig.withTypeFactory(f);
/*  891 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public JavaType constructType(Type t)
/*      */   {
/*  902 */     return this._typeFactory.constructType(t);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper setNodeFactory(JsonNodeFactory f)
/*      */   {
/*  919 */     this._deserializationConfig = this._deserializationConfig.withNodeFactory(f);
/*  920 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void setFilters(FilterProvider filterProvider)
/*      */   {
/*  942 */     this._serializationConfig = this._serializationConfig.withFilters(filterProvider);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public JsonFactory getJsonFactory()
/*      */   {
/*  959 */     return this._jsonFactory;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void setDateFormat(DateFormat dateFormat)
/*      */   {
/*  971 */     this._deserializationConfig = this._deserializationConfig.withDateFormat(dateFormat);
/*  972 */     this._serializationConfig = this._serializationConfig.withDateFormat(dateFormat);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void setHandlerInstantiator(HandlerInstantiator hi)
/*      */   {
/*  984 */     this._deserializationConfig = this._deserializationConfig.withHandlerInstantiator(hi);
/*  985 */     this._serializationConfig = this._serializationConfig.withHandlerInstantiator(hi);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper configure(SerializationConfig.Feature f, boolean state)
/*      */   {
/* 1003 */     this._serializationConfig.set(f, state);
/* 1004 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper configure(DeserializationConfig.Feature f, boolean state)
/*      */   {
/* 1016 */     this._deserializationConfig.set(f, state);
/* 1017 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper configure(JsonParser.Feature f, boolean state)
/*      */   {
/* 1032 */     this._jsonFactory.configure(f, state);
/* 1033 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectMapper configure(JsonGenerator.Feature f, boolean state)
/*      */   {
/* 1048 */     this._jsonFactory.configure(f, state);
/* 1049 */     return this;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public JsonNodeFactory getNodeFactory()
/*      */   {
/* 1065 */     return this._deserializationConfig.getNodeFactory();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> T readValue(JsonParser jp, Class<T> valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1094 */     return (T)_readValue(copyDeserializationConfig(), jp, this._typeFactory.constructType(valueType));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> T readValue(JsonParser jp, Class<T> valueType, DeserializationConfig cfg)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1121 */     return (T)_readValue(cfg, jp, this._typeFactory.constructType(valueType));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> T readValue(JsonParser jp, TypeReference<?> valueTypeRef)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1136 */     return (T)_readValue(copyDeserializationConfig(), jp, this._typeFactory.constructType(valueTypeRef));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> T readValue(JsonParser jp, TypeReference<?> valueTypeRef, DeserializationConfig cfg)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1159 */     return (T)_readValue(cfg, jp, this._typeFactory.constructType(valueTypeRef));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> T readValue(JsonParser jp, JavaType valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1173 */     return (T)_readValue(copyDeserializationConfig(), jp, valueType);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> T readValue(JsonParser jp, JavaType valueType, DeserializationConfig cfg)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1195 */     return (T)_readValue(cfg, jp, valueType);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public JsonNode readTree(JsonParser jp)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/* 1209 */     return readTree(jp, copyDeserializationConfig());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public JsonNode readTree(JsonParser jp, DeserializationConfig cfg)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/* 1234 */     JsonNode n = (JsonNode)_readValue(cfg, jp, JSON_NODE_TYPE);
/* 1235 */     return n == null ? NullNode.instance : n;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public JsonNode readTree(InputStream in)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/* 1253 */     JsonNode n = (JsonNode)readValue(in, JSON_NODE_TYPE);
/* 1254 */     return n == null ? NullNode.instance : n;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public JsonNode readTree(Reader r)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/* 1272 */     JsonNode n = (JsonNode)readValue(r, JSON_NODE_TYPE);
/* 1273 */     return n == null ? NullNode.instance : n;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public JsonNode readTree(String content)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/* 1291 */     JsonNode n = (JsonNode)readValue(content, JSON_NODE_TYPE);
/* 1292 */     return n == null ? NullNode.instance : n;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> MappingIterator<T> readValues(JsonParser jp, JavaType valueType)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/* 1305 */     DeserializationConfig config = copyDeserializationConfig();
/* 1306 */     DeserializationContext ctxt = _createDeserializationContext(jp, config);
/* 1307 */     JsonDeserializer<?> deser = _findRootDeserializer(config, valueType);
/* 1308 */     return new MappingIterator(valueType, jp, ctxt, deser);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> MappingIterator<T> readValues(JsonParser jp, Class<?> valueType)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/* 1319 */     return readValues(jp, this._typeFactory.constructType(valueType));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> MappingIterator<T> readValues(JsonParser jp, TypeReference<?> valueTypeRef)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/* 1330 */     return readValues(jp, this._typeFactory.constructType(valueTypeRef));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void writeValue(JsonGenerator jgen, Object value)
/*      */     throws IOException, JsonGenerationException, JsonMappingException
/*      */   {
/* 1348 */     SerializationConfig config = copySerializationConfig();
/* 1349 */     if ((config.isEnabled(SerializationConfig.Feature.CLOSE_CLOSEABLE)) && ((value instanceof Closeable))) {
/* 1350 */       _writeCloseableValue(jgen, value, config);
/*      */     } else {
/* 1352 */       this._serializerProvider.serializeValue(config, jgen, value, this._serializerFactory);
/* 1353 */       if (config.isEnabled(SerializationConfig.Feature.FLUSH_AFTER_WRITE_VALUE)) {
/* 1354 */         jgen.flush();
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void writeValue(JsonGenerator jgen, Object value, SerializationConfig config)
/*      */     throws IOException, JsonGenerationException, JsonMappingException
/*      */   {
/* 1370 */     if ((config.isEnabled(SerializationConfig.Feature.CLOSE_CLOSEABLE)) && ((value instanceof Closeable))) {
/* 1371 */       _writeCloseableValue(jgen, value, config);
/*      */     } else {
/* 1373 */       this._serializerProvider.serializeValue(config, jgen, value, this._serializerFactory);
/* 1374 */       if (config.isEnabled(SerializationConfig.Feature.FLUSH_AFTER_WRITE_VALUE)) {
/* 1375 */         jgen.flush();
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void writeTree(JsonGenerator jgen, JsonNode rootNode)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/* 1388 */     SerializationConfig config = copySerializationConfig();
/* 1389 */     this._serializerProvider.serializeValue(config, jgen, rootNode, this._serializerFactory);
/* 1390 */     if (config.isEnabled(SerializationConfig.Feature.FLUSH_AFTER_WRITE_VALUE)) {
/* 1391 */       jgen.flush();
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void writeTree(JsonGenerator jgen, JsonNode rootNode, SerializationConfig cfg)
/*      */     throws IOException, JsonProcessingException
/*      */   {
/* 1405 */     this._serializerProvider.serializeValue(cfg, jgen, rootNode, this._serializerFactory);
/* 1406 */     if (cfg.isEnabled(SerializationConfig.Feature.FLUSH_AFTER_WRITE_VALUE)) {
/* 1407 */       jgen.flush();
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectNode createObjectNode()
/*      */   {
/* 1428 */     return this._deserializationConfig.getNodeFactory().objectNode();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ArrayNode createArrayNode()
/*      */   {
/* 1442 */     return this._deserializationConfig.getNodeFactory().arrayNode();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public JsonParser treeAsTokens(JsonNode n)
/*      */   {
/* 1456 */     return new TreeTraversingParser(n, this);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> T treeToValue(JsonNode n, Class<T> valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1472 */     return (T)readValue(treeAsTokens(n), valueType);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T extends JsonNode> T valueToTree(Object fromValue)
/*      */     throws IllegalArgumentException
/*      */   {
/* 1492 */     if (fromValue == null) return null;
/* 1493 */     TokenBuffer buf = new TokenBuffer(this);
/*      */     JsonNode result;
/*      */     try {
/* 1496 */       writeValue(buf, fromValue);
/* 1497 */       JsonParser jp = buf.asParser();
/* 1498 */       result = readTree(jp);
/* 1499 */       jp.close();
/*      */     } catch (IOException e) {
/* 1501 */       throw new IllegalArgumentException(e.getMessage(), e);
/*      */     }
/* 1503 */     return result;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public boolean canSerialize(Class<?> type)
/*      */   {
/* 1524 */     return this._serializerProvider.hasSerializerFor(copySerializationConfig(), type, this._serializerFactory);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public boolean canDeserialize(JavaType type)
/*      */   {
/* 1540 */     return this._deserializerProvider.hasValueDeserializerFor(copyDeserializationConfig(), type);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> T readValue(File src, Class<T> valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1556 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(src), this._typeFactory.constructType(valueType));
/*      */   }
/*      */   
/*      */ 
/*      */   public <T> T readValue(File src, TypeReference valueTypeRef)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1563 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(src), this._typeFactory.constructType(valueTypeRef));
/*      */   }
/*      */   
/*      */ 
/*      */   public <T> T readValue(File src, JavaType valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1570 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(src), valueType);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> T readValue(URL src, Class<T> valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1579 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(src), this._typeFactory.constructType(valueType));
/*      */   }
/*      */   
/*      */ 
/*      */   public <T> T readValue(URL src, TypeReference valueTypeRef)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1586 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(src), this._typeFactory.constructType(valueTypeRef));
/*      */   }
/*      */   
/*      */ 
/*      */   public <T> T readValue(URL src, JavaType valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1593 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(src), valueType);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> T readValue(String content, Class<T> valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1602 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(content), this._typeFactory.constructType(valueType));
/*      */   }
/*      */   
/*      */ 
/*      */   public <T> T readValue(String content, TypeReference valueTypeRef)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1609 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(content), this._typeFactory.constructType(valueTypeRef));
/*      */   }
/*      */   
/*      */ 
/*      */   public <T> T readValue(String content, JavaType valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1616 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(content), valueType);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> T readValue(Reader src, Class<T> valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1625 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(src), this._typeFactory.constructType(valueType));
/*      */   }
/*      */   
/*      */ 
/*      */   public <T> T readValue(Reader src, TypeReference valueTypeRef)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1632 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(src), this._typeFactory.constructType(valueTypeRef));
/*      */   }
/*      */   
/*      */ 
/*      */   public <T> T readValue(Reader src, JavaType valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1639 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(src), valueType);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> T readValue(InputStream src, Class<T> valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1648 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(src), this._typeFactory.constructType(valueType));
/*      */   }
/*      */   
/*      */ 
/*      */   public <T> T readValue(InputStream src, TypeReference valueTypeRef)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1655 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(src), this._typeFactory.constructType(valueTypeRef));
/*      */   }
/*      */   
/*      */ 
/*      */   public <T> T readValue(InputStream src, JavaType valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1662 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(src), valueType);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> T readValue(byte[] src, Class<T> valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1674 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(src), this._typeFactory.constructType(valueType));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> T readValue(byte[] src, int offset, int len, Class<T> valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1684 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(src, offset, len), this._typeFactory.constructType(valueType));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> T readValue(byte[] src, TypeReference valueTypeRef)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1694 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(src), this._typeFactory.constructType(valueTypeRef));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public <T> T readValue(byte[] src, int offset, int len, TypeReference valueTypeRef)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1702 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(src, offset, len), this._typeFactory.constructType(valueTypeRef));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> T readValue(byte[] src, JavaType valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1712 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(src), valueType);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public <T> T readValue(byte[] src, int offset, int len, JavaType valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1720 */     return (T)_readMapAndClose(this._jsonFactory.createJsonParser(src, offset, len), valueType);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> T readValue(JsonNode root, Class<T> valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1738 */     return (T)_readValue(copyDeserializationConfig(), treeAsTokens(root), this._typeFactory.constructType(valueType));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> T readValue(JsonNode root, TypeReference valueTypeRef)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1754 */     return (T)_readValue(copyDeserializationConfig(), treeAsTokens(root), this._typeFactory.constructType(valueTypeRef));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> T readValue(JsonNode root, JavaType valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 1770 */     return (T)_readValue(copyDeserializationConfig(), treeAsTokens(root), valueType);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void writeValue(File resultFile, Object value)
/*      */     throws IOException, JsonGenerationException, JsonMappingException
/*      */   {
/* 1787 */     _configAndWriteValue(this._jsonFactory.createJsonGenerator(resultFile, JsonEncoding.UTF8), value);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void writeValue(OutputStream out, Object value)
/*      */     throws IOException, JsonGenerationException, JsonMappingException
/*      */   {
/* 1804 */     _configAndWriteValue(this._jsonFactory.createJsonGenerator(out, JsonEncoding.UTF8), value);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void writeValue(Writer w, Object value)
/*      */     throws IOException, JsonGenerationException, JsonMappingException
/*      */   {
/* 1820 */     _configAndWriteValue(this._jsonFactory.createJsonGenerator(w), value);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public String writeValueAsString(Object value)
/*      */     throws IOException, JsonGenerationException, JsonMappingException
/*      */   {
/* 1835 */     SegmentedStringWriter sw = new SegmentedStringWriter(this._jsonFactory._getBufferRecycler());
/* 1836 */     _configAndWriteValue(this._jsonFactory.createJsonGenerator(sw), value);
/* 1837 */     return sw.getAndClear();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public byte[] writeValueAsBytes(Object value)
/*      */     throws IOException, JsonGenerationException, JsonMappingException
/*      */   {
/* 1852 */     ByteArrayBuilder bb = new ByteArrayBuilder(this._jsonFactory._getBufferRecycler());
/* 1853 */     _configAndWriteValue(this._jsonFactory.createJsonGenerator(bb, JsonEncoding.UTF8), value);
/* 1854 */     byte[] result = bb.toByteArray();
/* 1855 */     bb.release();
/* 1856 */     return result;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   @Deprecated
/*      */   public void writeValueUsingView(JsonGenerator jgen, Object value, Class<?> viewClass)
/*      */     throws IOException, JsonGenerationException, JsonMappingException
/*      */   {
/* 1891 */     _configAndWriteValue(jgen, value, viewClass);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   @Deprecated
/*      */   public void writeValueUsingView(Writer w, Object value, Class<?> viewClass)
/*      */     throws IOException, JsonGenerationException, JsonMappingException
/*      */   {
/* 1911 */     _configAndWriteValue(this._jsonFactory.createJsonGenerator(w), value, viewClass);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   @Deprecated
/*      */   public void writeValueUsingView(OutputStream out, Object value, Class<?> viewClass)
/*      */     throws IOException, JsonGenerationException, JsonMappingException
/*      */   {
/* 1931 */     _configAndWriteValue(this._jsonFactory.createJsonGenerator(out, JsonEncoding.UTF8), value, viewClass);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectWriter writer()
/*      */   {
/* 1948 */     return new ObjectWriter(this, copySerializationConfig());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectWriter viewWriter(Class<?> serializationView)
/*      */   {
/* 1958 */     return new ObjectWriter(this, copySerializationConfig().withView(serializationView));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectWriter typedWriter(Class<?> rootType)
/*      */   {
/* 1970 */     JavaType t = rootType == null ? null : this._typeFactory.constructType(rootType);
/* 1971 */     return new ObjectWriter(this, copySerializationConfig(), t, null);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectWriter typedWriter(JavaType rootType)
/*      */   {
/* 1982 */     return new ObjectWriter(this, copySerializationConfig(), rootType, null);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectWriter typedWriter(TypeReference<?> rootType)
/*      */   {
/* 1993 */     JavaType t = rootType == null ? null : this._typeFactory.constructType(rootType);
/* 1994 */     return new ObjectWriter(this, copySerializationConfig(), t, null);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectWriter prettyPrintingWriter(PrettyPrinter pp)
/*      */   {
/* 2005 */     if (pp == null) {
/* 2006 */       pp = ObjectWriter.NULL_PRETTY_PRINTER;
/*      */     }
/* 2008 */     return new ObjectWriter(this, copySerializationConfig(), null, pp);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectWriter defaultPrettyPrintingWriter()
/*      */   {
/* 2018 */     return new ObjectWriter(this, copySerializationConfig(), null, _defaultPrettyPrinter());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectWriter filteredWriter(FilterProvider filterProvider)
/*      */   {
/* 2029 */     return new ObjectWriter(this, copySerializationConfig().withFilters(filterProvider));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectWriter schemaBasedWriter(FormatSchema schema)
/*      */   {
/* 2043 */     return new ObjectWriter(this, copySerializationConfig(), schema);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectReader reader()
/*      */   {
/* 2060 */     return new ObjectReader(this, copyDeserializationConfig());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectReader updatingReader(Object valueToUpdate)
/*      */   {
/* 2077 */     JavaType t = this._typeFactory.constructType(valueToUpdate.getClass());
/* 2078 */     return new ObjectReader(this, copyDeserializationConfig(), t, valueToUpdate, null);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectReader reader(JavaType type)
/*      */   {
/* 2089 */     return new ObjectReader(this, copyDeserializationConfig(), type, null, null);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectReader reader(Class<?> type)
/*      */   {
/* 2100 */     return reader(this._typeFactory.constructType(type));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectReader reader(TypeReference<?> type)
/*      */   {
/* 2111 */     return reader(this._typeFactory.constructType(type));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectReader reader(JsonNodeFactory f)
/*      */   {
/* 2122 */     return new ObjectReader(this, copyDeserializationConfig()).withNodeFactory(f);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public ObjectReader schemaBasedReader(FormatSchema schema)
/*      */   {
/* 2135 */     return new ObjectReader(this, copyDeserializationConfig(), null, null, schema);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public <T> T convertValue(Object fromValue, Class<T> toValueType)
/*      */     throws IllegalArgumentException
/*      */   {
/* 2160 */     return (T)_convert(fromValue, this._typeFactory.constructType(toValueType));
/*      */   }
/*      */   
/*      */ 
/*      */   public <T> T convertValue(Object fromValue, TypeReference toValueTypeRef)
/*      */     throws IllegalArgumentException
/*      */   {
/* 2167 */     return (T)_convert(fromValue, this._typeFactory.constructType(toValueTypeRef));
/*      */   }
/*      */   
/*      */ 
/*      */   public <T> T convertValue(Object fromValue, JavaType toValueType)
/*      */     throws IllegalArgumentException
/*      */   {
/* 2174 */     return (T)_convert(fromValue, toValueType);
/*      */   }
/*      */   
/*      */ 
/*      */   protected Object _convert(Object fromValue, JavaType toValueType)
/*      */     throws IllegalArgumentException
/*      */   {
/* 2181 */     if (fromValue == null) { return null;
/*      */     }
/*      */     
/*      */ 
/* 2185 */     TokenBuffer buf = new TokenBuffer(this);
/*      */     try {
/* 2187 */       writeValue(buf, fromValue);
/*      */       
/* 2189 */       JsonParser jp = buf.asParser();
/* 2190 */       Object result = readValue(jp, toValueType);
/* 2191 */       jp.close();
/* 2192 */       return result;
/*      */     } catch (IOException e) {
/* 2194 */       throw new IllegalArgumentException(e.getMessage(), e);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public JsonSchema generateJsonSchema(Class<?> t)
/*      */     throws JsonMappingException
/*      */   {
/* 2214 */     return generateJsonSchema(t, copySerializationConfig());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public JsonSchema generateJsonSchema(Class<?> t, SerializationConfig cfg)
/*      */     throws JsonMappingException
/*      */   {
/* 2228 */     return this._serializerProvider.generateJsonSchema(t, cfg, this._serializerFactory);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected PrettyPrinter _defaultPrettyPrinter()
/*      */   {
/* 2245 */     return new DefaultPrettyPrinter();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected final void _configAndWriteValue(JsonGenerator jgen, Object value)
/*      */     throws IOException, JsonGenerationException, JsonMappingException
/*      */   {
/* 2255 */     SerializationConfig cfg = copySerializationConfig();
/*      */     
/* 2257 */     if (cfg.isEnabled(SerializationConfig.Feature.INDENT_OUTPUT)) {
/* 2258 */       jgen.useDefaultPrettyPrinter();
/*      */     }
/*      */     
/* 2261 */     if ((cfg.isEnabled(SerializationConfig.Feature.CLOSE_CLOSEABLE)) && ((value instanceof Closeable))) {
/* 2262 */       _configAndWriteCloseable(jgen, value, cfg);
/* 2263 */       return;
/*      */     }
/* 2265 */     boolean closed = false;
/*      */     try {
/* 2267 */       this._serializerProvider.serializeValue(cfg, jgen, value, this._serializerFactory);
/* 2268 */       closed = true;
/* 2269 */       jgen.close(); return;
/*      */ 
/*      */     }
/*      */     finally
/*      */     {
/* 2274 */       if (!closed) {
/*      */         try {
/* 2276 */           jgen.close();
/*      */         }
/*      */         catch (IOException ioe) {}
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   protected final void _configAndWriteValue(JsonGenerator jgen, Object value, Class<?> viewClass) throws IOException, JsonGenerationException, JsonMappingException
/*      */   {
/* 2285 */     SerializationConfig cfg = copySerializationConfig().withView(viewClass);
/* 2286 */     if (cfg.isEnabled(SerializationConfig.Feature.INDENT_OUTPUT)) {
/* 2287 */       jgen.useDefaultPrettyPrinter();
/*      */     }
/*      */     
/* 2290 */     if ((cfg.isEnabled(SerializationConfig.Feature.CLOSE_CLOSEABLE)) && ((value instanceof Closeable))) {
/* 2291 */       _configAndWriteCloseable(jgen, value, cfg);
/* 2292 */       return;
/*      */     }
/* 2294 */     boolean closed = false;
/*      */     try {
/* 2296 */       this._serializerProvider.serializeValue(cfg, jgen, value, this._serializerFactory);
/* 2297 */       closed = true;
/* 2298 */       jgen.close(); return;
/*      */     } finally {
/* 2300 */       if (!closed) {
/*      */         try {
/* 2302 */           jgen.close();
/*      */         }
/*      */         catch (IOException ioe) {}
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private final void _configAndWriteCloseable(JsonGenerator jgen, Object value, SerializationConfig cfg)
/*      */     throws IOException, JsonGenerationException, JsonMappingException
/*      */   {
/* 2315 */     Closeable toClose = (Closeable)value;
/*      */     try {
/* 2317 */       this._serializerProvider.serializeValue(cfg, jgen, value, this._serializerFactory);
/* 2318 */       JsonGenerator tmpJgen = jgen;
/* 2319 */       jgen = null;
/* 2320 */       tmpJgen.close();
/* 2321 */       Closeable tmpToClose = toClose;
/* 2322 */       toClose = null;
/* 2323 */       tmpToClose.close(); return;
/*      */ 
/*      */     }
/*      */     finally
/*      */     {
/* 2328 */       if (jgen != null) {
/*      */         try {
/* 2330 */           jgen.close();
/*      */         } catch (IOException ioe) {}
/*      */       }
/* 2333 */       if (toClose != null) {
/*      */         try {
/* 2335 */           toClose.close();
/*      */         }
/*      */         catch (IOException ioe) {}
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private final void _writeCloseableValue(JsonGenerator jgen, Object value, SerializationConfig cfg)
/*      */     throws IOException, JsonGenerationException, JsonMappingException
/*      */   {
/* 2348 */     Closeable toClose = (Closeable)value;
/*      */     try {
/* 2350 */       this._serializerProvider.serializeValue(cfg, jgen, value, this._serializerFactory);
/* 2351 */       if (cfg.isEnabled(SerializationConfig.Feature.FLUSH_AFTER_WRITE_VALUE)) {
/* 2352 */         jgen.flush();
/*      */       }
/* 2354 */       Closeable tmpToClose = toClose;
/* 2355 */       toClose = null;
/* 2356 */       tmpToClose.close(); return;
/*      */     } finally {
/* 2358 */       if (toClose != null) {
/*      */         try {
/* 2360 */           toClose.close();
/*      */         }
/*      */         catch (IOException ioe) {}
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected Object _readValue(DeserializationConfig cfg, JsonParser jp, JavaType valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 2377 */     JsonToken t = _initForReading(jp);
/* 2378 */     Object result; Object result; if ((t == JsonToken.VALUE_NULL) || (t == JsonToken.END_ARRAY) || (t == JsonToken.END_OBJECT)) {
/* 2379 */       result = null;
/*      */     } else {
/* 2381 */       DeserializationContext ctxt = _createDeserializationContext(jp, cfg);
/*      */       
/* 2383 */       result = _findRootDeserializer(cfg, valueType).deserialize(jp, ctxt);
/*      */     }
/*      */     
/* 2386 */     jp.clearCurrentToken();
/* 2387 */     return result;
/*      */   }
/*      */   
/*      */ 
/*      */   protected Object _readMapAndClose(JsonParser jp, JavaType valueType)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/*      */     try
/*      */     {
/* 2396 */       JsonToken t = _initForReading(jp);
/* 2397 */       Object result; DeserializationConfig cfg; Object result; if ((t == JsonToken.VALUE_NULL) || (t == JsonToken.END_ARRAY) || (t == JsonToken.END_OBJECT)) {
/* 2398 */         result = null;
/*      */       } else {
/* 2400 */         cfg = copyDeserializationConfig();
/* 2401 */         DeserializationContext ctxt = _createDeserializationContext(jp, cfg);
/* 2402 */         result = _findRootDeserializer(cfg, valueType).deserialize(jp, ctxt);
/*      */       }
/*      */       
/* 2405 */       jp.clearCurrentToken();
/* 2406 */       return (DeserializationConfig)result;
/*      */     } finally {
/*      */       try {
/* 2409 */         jp.close();
/*      */       }
/*      */       catch (IOException ioe) {}
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected JsonToken _initForReading(JsonParser jp)
/*      */     throws IOException, JsonParseException, JsonMappingException
/*      */   {
/* 2436 */     JsonToken t = jp.getCurrentToken();
/* 2437 */     if (t == null)
/*      */     {
/* 2439 */       t = jp.nextToken();
/* 2440 */       if (t == null)
/*      */       {
/*      */ 
/*      */ 
/* 2444 */         throw new EOFException("No content to map to Object due to end of input");
/*      */       }
/*      */     }
/* 2447 */     return t;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected JsonDeserializer<Object> _findRootDeserializer(DeserializationConfig cfg, JavaType valueType)
/*      */     throws JsonMappingException
/*      */   {
/* 2463 */     JsonDeserializer<Object> deser = (JsonDeserializer)this._rootDeserializers.get(valueType);
/* 2464 */     if (deser != null) {
/* 2465 */       return deser;
/*      */     }
/*      */     
/* 2468 */     deser = this._deserializerProvider.findTypedValueDeserializer(cfg, valueType, null);
/* 2469 */     if (deser == null) {
/* 2470 */       throw new JsonMappingException("Can not find a deserializer for type " + valueType);
/*      */     }
/* 2472 */     this._rootDeserializers.put(valueType, deser);
/* 2473 */     return deser;
/*      */   }
/*      */   
/*      */ 
/*      */   protected DeserializationContext _createDeserializationContext(JsonParser jp, DeserializationConfig cfg)
/*      */   {
/* 2479 */     return new StdDeserializationContext(cfg, jp, this._deserializerProvider);
/*      */   }
/*      */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\ObjectMapper.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */