/*     */ package org.codehaus.jackson.map.ser;
/*     */ 
/*     */ import java.util.ArrayList;
/*     */ import java.util.Collection;
/*     */ import java.util.Collections;
/*     */ import java.util.HashMap;
/*     */ import java.util.HashSet;
/*     */ import java.util.Iterator;
/*     */ import java.util.LinkedHashMap;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.Map.Entry;
/*     */ import java.util.Set;
/*     */ import java.util.TreeMap;
/*     */ import org.codehaus.jackson.annotate.JsonAutoDetect.Visibility;
/*     */ import org.codehaus.jackson.map.AnnotationIntrospector;
/*     */ import org.codehaus.jackson.map.AnnotationIntrospector.ReferenceProperty;
/*     */ import org.codehaus.jackson.map.BeanProperty;
/*     */ import org.codehaus.jackson.map.BeanProperty.Std;
/*     */ import org.codehaus.jackson.map.JsonMappingException;
/*     */ import org.codehaus.jackson.map.JsonSerializer;
/*     */ import org.codehaus.jackson.map.SerializationConfig;
/*     */ import org.codehaus.jackson.map.SerializationConfig.Feature;
/*     */ import org.codehaus.jackson.map.SerializerFactory;
/*     */ import org.codehaus.jackson.map.SerializerFactory.Config;
/*     */ import org.codehaus.jackson.map.Serializers;
/*     */ import org.codehaus.jackson.map.TypeSerializer;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedClass;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedField;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedMember;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedMethod;
/*     */ import org.codehaus.jackson.map.introspect.BasicBeanDescription;
/*     */ import org.codehaus.jackson.map.introspect.VisibilityChecker;
/*     */ import org.codehaus.jackson.map.jsontype.NamedType;
/*     */ import org.codehaus.jackson.map.jsontype.SubtypeResolver;
/*     */ import org.codehaus.jackson.map.jsontype.TypeResolverBuilder;
/*     */ import org.codehaus.jackson.map.type.TypeBindings;
/*     */ import org.codehaus.jackson.map.util.ArrayBuilders;
/*     */ import org.codehaus.jackson.map.util.ClassUtil;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class BeanSerializerFactory
/*     */   extends BasicSerializerFactory
/*     */ {
/*  56 */   public static final BeanSerializerFactory instance = new BeanSerializerFactory(null);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final SerializerFactory.Config _factoryConfig;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static class ConfigImpl
/*     */     extends SerializerFactory.Config
/*     */   {
/*  83 */     protected static final Serializers[] NO_SERIALIZERS = new Serializers[0];
/*     */     
/*  85 */     protected static final BeanSerializerModifier[] NO_MODIFIERS = new BeanSerializerModifier[0];
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     protected final Serializers[] _additionalSerializers;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     protected final Serializers[] _additionalKeySerializers;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     protected final BeanSerializerModifier[] _modifiers;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     public ConfigImpl()
/*     */     {
/* 107 */       this(null, null, null);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */     protected ConfigImpl(Serializers[] allAdditionalSerializers, Serializers[] allAdditionalKeySerializers, BeanSerializerModifier[] modifiers)
/*     */     {
/* 114 */       this._additionalSerializers = (allAdditionalSerializers == null ? NO_SERIALIZERS : allAdditionalSerializers);
/*     */       
/* 116 */       this._additionalKeySerializers = (allAdditionalKeySerializers == null ? NO_SERIALIZERS : allAdditionalKeySerializers);
/*     */       
/* 118 */       this._modifiers = (modifiers == null ? NO_MODIFIERS : modifiers);
/*     */     }
/*     */     
/*     */ 
/*     */     public SerializerFactory.Config withAdditionalSerializers(Serializers additional)
/*     */     {
/* 124 */       if (additional == null) {
/* 125 */         throw new IllegalArgumentException("Can not pass null Serializers");
/*     */       }
/* 127 */       Serializers[] all = (Serializers[])ArrayBuilders.insertInListNoDup(this._additionalSerializers, additional);
/* 128 */       return new ConfigImpl(all, this._additionalKeySerializers, this._modifiers);
/*     */     }
/*     */     
/*     */ 
/*     */     public SerializerFactory.Config withAdditionalKeySerializers(Serializers additional)
/*     */     {
/* 134 */       if (additional == null) {
/* 135 */         throw new IllegalArgumentException("Can not pass null Serializers");
/*     */       }
/* 137 */       Serializers[] all = (Serializers[])ArrayBuilders.insertInListNoDup(this._additionalKeySerializers, additional);
/* 138 */       return new ConfigImpl(this._additionalSerializers, all, this._modifiers);
/*     */     }
/*     */     
/*     */ 
/*     */     public SerializerFactory.Config withSerializerModifier(BeanSerializerModifier modifier)
/*     */     {
/* 144 */       if (modifier == null) {
/* 145 */         throw new IllegalArgumentException("Can not pass null modifier");
/*     */       }
/* 147 */       BeanSerializerModifier[] modifiers = (BeanSerializerModifier[])ArrayBuilders.insertInListNoDup(this._modifiers, modifier);
/* 148 */       return new ConfigImpl(this._additionalSerializers, this._additionalKeySerializers, modifiers);
/*     */     }
/*     */     
/*     */     public boolean hasSerializers() {
/* 152 */       return this._additionalSerializers.length > 0;
/*     */     }
/*     */     
/* 155 */     public boolean hasKeySerializers() { return this._additionalKeySerializers.length > 0; }
/*     */     
/*     */     public boolean hasSerializerModifiers() {
/* 158 */       return this._modifiers.length > 0;
/*     */     }
/*     */     
/*     */     public Iterable<Serializers> serializers() {
/* 162 */       return ArrayBuilders.arrayAsIterable(this._additionalSerializers);
/*     */     }
/*     */     
/*     */     public Iterable<Serializers> keySerializers()
/*     */     {
/* 167 */       return ArrayBuilders.arrayAsIterable(this._additionalKeySerializers);
/*     */     }
/*     */     
/*     */     public Iterable<BeanSerializerModifier> serializerModifiers()
/*     */     {
/* 172 */       return ArrayBuilders.arrayAsIterable(this._modifiers);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   @Deprecated
/*     */   protected BeanSerializerFactory()
/*     */   {
/* 183 */     this(null);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   protected BeanSerializerFactory(SerializerFactory.Config config)
/*     */   {
/* 190 */     if (config == null) {
/* 191 */       config = new ConfigImpl();
/*     */     }
/* 193 */     this._factoryConfig = config;
/*     */   }
/*     */   
/* 196 */   public SerializerFactory.Config getConfig() { return this._factoryConfig; }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public SerializerFactory withConfig(SerializerFactory.Config config)
/*     */   {
/* 209 */     if (this._factoryConfig == config) {
/* 210 */       return this;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 218 */     if (getClass() != BeanSerializerFactory.class) {
/* 219 */       throw new IllegalStateException("Subtype of BeanSerializerFactory (" + getClass().getName() + ") has not properly overridden method 'withAdditionalSerializers': can not instantiate subtype with " + "additional serializer definitions");
/*     */     }
/*     */     
/*     */ 
/* 223 */     return new BeanSerializerFactory(config);
/*     */   }
/*     */   
/*     */   protected Iterable<Serializers> customSerializers()
/*     */   {
/* 228 */     return this._factoryConfig.serializers();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonSerializer<Object> createSerializer(SerializationConfig config, JavaType origType, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 254 */     BasicBeanDescription beanDesc = (BasicBeanDescription)config.introspect(origType);
/* 255 */     JsonSerializer<?> ser = findSerializerFromAnnotation(config, beanDesc.getClassInfo(), property);
/* 256 */     if (ser != null) {
/* 257 */       return ser;
/*     */     }
/*     */     
/*     */ 
/* 261 */     JavaType type = modifyTypeByAnnotation(config, beanDesc.getClassInfo(), origType);
/*     */     
/* 263 */     boolean staticTyping = type != origType;
/*     */     
/*     */ 
/* 266 */     if (origType.isContainerType()) {
/* 267 */       return buildContainerSerializer(config, type, beanDesc, property, staticTyping);
/*     */     }
/*     */     
/*     */ 
/* 271 */     for (Serializers serializers : this._factoryConfig.serializers()) {
/* 272 */       ser = serializers.findSerializer(config, type, beanDesc, property);
/* 273 */       if (ser != null) {
/* 274 */         return ser;
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 282 */     ser = findSerializerByLookup(type, config, beanDesc, property, staticTyping);
/* 283 */     if (ser != null) {
/* 284 */       return ser;
/*     */     }
/* 286 */     ser = findSerializerByPrimaryType(type, config, beanDesc, property, staticTyping);
/* 287 */     if (ser != null) {
/* 288 */       return ser;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 295 */     ser = findBeanSerializer(config, type, beanDesc, property);
/*     */     
/*     */ 
/*     */ 
/* 299 */     if (ser == null) {
/* 300 */       ser = super.findSerializerByAddonType(config, type, beanDesc, property, staticTyping);
/*     */     }
/* 302 */     return ser;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonSerializer<Object> createKeySerializer(SerializationConfig config, JavaType type, BeanProperty property)
/*     */   {
/* 311 */     if (!this._factoryConfig.hasKeySerializers()) {
/* 312 */       return null;
/*     */     }
/*     */     
/*     */ 
/* 316 */     BasicBeanDescription beanDesc = (BasicBeanDescription)config.introspectClassAnnotations(type.getRawClass());
/* 317 */     JsonSerializer<?> ser = null;
/*     */     
/*     */ 
/* 320 */     for (Serializers serializers : this._factoryConfig.keySerializers()) {
/* 321 */       ser = serializers.findSerializer(config, type, beanDesc, property);
/* 322 */       if (ser != null) {
/*     */         break;
/*     */       }
/*     */     }
/* 326 */     return ser;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonSerializer<Object> findBeanSerializer(SerializationConfig config, JavaType type, BasicBeanDescription beanDesc, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 346 */     if (!isPotentialBeanType(type.getRawClass())) {
/* 347 */       return null;
/*     */     }
/* 349 */     JsonSerializer<Object> serializer = constructBeanSerializer(config, beanDesc, property);
/*     */     
/* 351 */     if (this._factoryConfig.hasSerializerModifiers()) {
/* 352 */       for (BeanSerializerModifier mod : this._factoryConfig.serializerModifiers()) {
/* 353 */         serializer = mod.modifySerializer(config, beanDesc, serializer);
/*     */       }
/*     */     }
/* 356 */     return serializer;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public TypeSerializer findPropertyTypeSerializer(JavaType baseType, SerializationConfig config, AnnotatedMember accessor, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 375 */     AnnotationIntrospector ai = config.getAnnotationIntrospector();
/* 376 */     TypeResolverBuilder<?> b = ai.findPropertyTypeResolver(config, accessor, baseType);
/*     */     
/* 378 */     if (b == null) {
/* 379 */       return createTypeSerializer(config, baseType, property);
/*     */     }
/* 381 */     Collection<NamedType> subtypes = config.getSubtypeResolver().collectAndResolveSubtypes(accessor, config, ai);
/* 382 */     return b.buildTypeSerializer(config, baseType, subtypes, property);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public TypeSerializer findPropertyContentTypeSerializer(JavaType containerType, SerializationConfig config, AnnotatedMember accessor, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 401 */     JavaType contentType = containerType.getContentType();
/* 402 */     AnnotationIntrospector ai = config.getAnnotationIntrospector();
/* 403 */     TypeResolverBuilder<?> b = ai.findPropertyContentTypeResolver(config, accessor, containerType);
/*     */     
/* 405 */     if (b == null) {
/* 406 */       return createTypeSerializer(config, contentType, property);
/*     */     }
/* 408 */     Collection<NamedType> subtypes = config.getSubtypeResolver().collectAndResolveSubtypes(accessor, config, ai);
/* 409 */     return b.buildTypeSerializer(config, contentType, subtypes, property);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonSerializer<Object> constructBeanSerializer(SerializationConfig config, BasicBeanDescription beanDesc, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 429 */     if (beanDesc.getBeanClass() == Object.class) {
/* 430 */       throw new IllegalArgumentException("Can not create bean serializer for Object.class");
/*     */     }
/*     */     
/* 433 */     BeanSerializerBuilder builder = constructBeanSerializerBuilder(beanDesc);
/*     */     
/*     */ 
/* 436 */     List<BeanPropertyWriter> props = findBeanProperties(config, beanDesc);
/*     */     
/*     */ 
/* 439 */     if (this._factoryConfig.hasSerializerModifiers()) {
/* 440 */       if (props == null) {
/* 441 */         props = new ArrayList();
/*     */       }
/* 443 */       for (BeanSerializerModifier mod : this._factoryConfig.serializerModifiers()) {
/* 444 */         props = mod.changeProperties(config, beanDesc, props);
/*     */       }
/*     */     }
/*     */     
/* 448 */     AnnotatedMethod anyGetter = beanDesc.findAnyGetter();
/*     */     
/*     */ 
/* 451 */     if ((props == null) || (props.size() == 0)) {
/* 452 */       if (anyGetter == null)
/*     */       {
/*     */ 
/*     */ 
/*     */ 
/* 457 */         if (beanDesc.hasKnownClassAnnotations()) {
/* 458 */           return builder.createDummy();
/*     */         }
/* 460 */         return null;
/*     */       }
/* 462 */       props = Collections.emptyList();
/*     */     }
/*     */     else {
/* 465 */       props = filterBeanProperties(config, beanDesc, props);
/*     */       
/* 467 */       props = sortBeanProperties(config, beanDesc, props);
/*     */     }
/*     */     
/* 470 */     if (this._factoryConfig.hasSerializerModifiers()) {
/* 471 */       for (BeanSerializerModifier mod : this._factoryConfig.serializerModifiers()) {
/* 472 */         props = mod.orderProperties(config, beanDesc, props);
/*     */       }
/*     */     }
/* 475 */     builder.setProperties(props);
/* 476 */     builder.setFilterId(findFilterId(config, beanDesc));
/*     */     
/* 478 */     if (anyGetter != null) {
/* 479 */       if (config.isEnabled(SerializationConfig.Feature.CAN_OVERRIDE_ACCESS_MODIFIERS)) {
/* 480 */         anyGetter.fixAccess();
/*     */       }
/* 482 */       JavaType type = anyGetter.getType(beanDesc.bindingsForBeanType());
/*     */       
/* 484 */       boolean staticTyping = config.isEnabled(SerializationConfig.Feature.USE_STATIC_TYPING);
/* 485 */       JavaType valueType = type.getContentType();
/* 486 */       TypeSerializer typeSer = createTypeSerializer(config, valueType, property);
/*     */       
/* 488 */       MapSerializer mapSer = MapSerializer.construct(null, type, staticTyping, typeSer, property, null, null);
/*     */       
/* 490 */       builder.setAnyGetter(new AnyGetterWriter(anyGetter, mapSer));
/*     */     }
/*     */     
/* 493 */     processViews(config, builder);
/*     */     
/* 495 */     if (this._factoryConfig.hasSerializerModifiers()) {
/* 496 */       for (BeanSerializerModifier mod : this._factoryConfig.serializerModifiers()) {
/* 497 */         builder = mod.updateBuilder(config, beanDesc, builder);
/*     */       }
/*     */     }
/*     */     
/* 501 */     return builder.build();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected BeanPropertyWriter constructFilteredBeanWriter(BeanPropertyWriter writer, Class<?>[] inViews)
/*     */   {
/* 511 */     return FilteredBeanPropertyWriter.constructViewBased(writer, inViews);
/*     */   }
/*     */   
/*     */ 
/*     */   protected PropertyBuilder constructPropertyBuilder(SerializationConfig config, BasicBeanDescription beanDesc)
/*     */   {
/* 517 */     return new PropertyBuilder(config, beanDesc);
/*     */   }
/*     */   
/*     */   protected BeanSerializerBuilder constructBeanSerializerBuilder(BasicBeanDescription beanDesc) {
/* 521 */     return new BeanSerializerBuilder(beanDesc);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected Object findFilterId(SerializationConfig config, BasicBeanDescription beanDesc)
/*     */   {
/* 532 */     return config.getAnnotationIntrospector().findFilterId(beanDesc.getClassInfo());
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected boolean isPotentialBeanType(Class<?> type)
/*     */   {
/* 551 */     return (ClassUtil.canBeABeanType(type) == null) && (!ClassUtil.isProxyType(type));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected List<BeanPropertyWriter> findBeanProperties(SerializationConfig config, BasicBeanDescription beanDesc)
/*     */     throws JsonMappingException
/*     */   {
/* 562 */     VisibilityChecker<?> vchecker = config.getDefaultVisibilityChecker();
/* 563 */     if (!config.isEnabled(SerializationConfig.Feature.AUTO_DETECT_GETTERS)) {
/* 564 */       vchecker = vchecker.withGetterVisibility(JsonAutoDetect.Visibility.NONE);
/*     */     }
/*     */     
/* 567 */     if (!config.isEnabled(SerializationConfig.Feature.AUTO_DETECT_IS_GETTERS)) {
/* 568 */       vchecker = vchecker.withIsGetterVisibility(JsonAutoDetect.Visibility.NONE);
/*     */     }
/* 570 */     if (!config.isEnabled(SerializationConfig.Feature.AUTO_DETECT_FIELDS)) {
/* 571 */       vchecker = vchecker.withFieldVisibility(JsonAutoDetect.Visibility.NONE);
/*     */     }
/*     */     
/* 574 */     AnnotationIntrospector intr = config.getAnnotationIntrospector();
/* 575 */     vchecker = intr.findAutoDetectVisibility(beanDesc.getClassInfo(), vchecker);
/*     */     
/* 577 */     LinkedHashMap<String, AnnotatedMethod> methodsByProp = beanDesc.findGetters(vchecker, null);
/* 578 */     LinkedHashMap<String, AnnotatedField> fieldsByProp = beanDesc.findSerializableFields(vchecker, methodsByProp.keySet());
/*     */     
/*     */ 
/* 581 */     removeIgnorableTypes(config, beanDesc, methodsByProp);
/* 582 */     removeIgnorableTypes(config, beanDesc, fieldsByProp);
/*     */     
/*     */ 
/* 585 */     if ((methodsByProp.isEmpty()) && (fieldsByProp.isEmpty())) {
/* 586 */       return null;
/*     */     }
/*     */     
/*     */ 
/* 590 */     boolean staticTyping = usesStaticTyping(config, beanDesc, null, null);
/* 591 */     PropertyBuilder pb = constructPropertyBuilder(config, beanDesc);
/*     */     
/* 593 */     ArrayList<BeanPropertyWriter> props = new ArrayList(methodsByProp.size());
/* 594 */     TypeBindings typeBind = beanDesc.bindingsForBeanType();
/*     */     
/* 596 */     for (Map.Entry<String, AnnotatedField> en : fieldsByProp.entrySet())
/*     */     {
/* 598 */       AnnotationIntrospector.ReferenceProperty prop = intr.findReferenceType((AnnotatedMember)en.getValue());
/* 599 */       if ((prop == null) || (!prop.isBackReference()))
/*     */       {
/*     */ 
/* 602 */         props.add(_constructWriter(config, typeBind, pb, staticTyping, (String)en.getKey(), (AnnotatedMember)en.getValue()));
/*     */       }
/*     */     }
/* 605 */     for (Map.Entry<String, AnnotatedMethod> en : methodsByProp.entrySet())
/*     */     {
/* 607 */       AnnotationIntrospector.ReferenceProperty prop = intr.findReferenceType((AnnotatedMember)en.getValue());
/* 608 */       if ((prop == null) || (!prop.isBackReference()))
/*     */       {
/*     */ 
/* 611 */         props.add(_constructWriter(config, typeBind, pb, staticTyping, (String)en.getKey(), (AnnotatedMember)en.getValue())); }
/*     */     }
/* 613 */     return props;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected List<BeanPropertyWriter> filterBeanProperties(SerializationConfig config, BasicBeanDescription beanDesc, List<BeanPropertyWriter> props)
/*     */   {
/* 629 */     AnnotationIntrospector intr = config.getAnnotationIntrospector();
/* 630 */     AnnotatedClass ac = beanDesc.getClassInfo();
/* 631 */     String[] ignored = intr.findPropertiesToIgnore(ac);
/* 632 */     if ((ignored != null) && (ignored.length > 0)) {
/* 633 */       HashSet<String> ignoredSet = ArrayBuilders.arrayToSet(ignored);
/* 634 */       Iterator<BeanPropertyWriter> it = props.iterator();
/* 635 */       while (it.hasNext()) {
/* 636 */         if (ignoredSet.contains(((BeanPropertyWriter)it.next()).getName())) {
/* 637 */           it.remove();
/*     */         }
/*     */       }
/*     */     }
/* 641 */     return props;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected List<BeanPropertyWriter> sortBeanProperties(SerializationConfig config, BasicBeanDescription beanDesc, List<BeanPropertyWriter> props)
/*     */   {
/* 662 */     List<String> creatorProps = beanDesc.findCreatorPropertyNames();
/*     */     
/* 664 */     AnnotationIntrospector intr = config.getAnnotationIntrospector();
/* 665 */     AnnotatedClass ac = beanDesc.getClassInfo();
/* 666 */     String[] propOrder = intr.findSerializationPropertyOrder(ac);
/* 667 */     Boolean alpha = intr.findSerializationSortAlphabetically(ac);
/*     */     boolean sort;
/*     */     boolean sort;
/* 670 */     if (alpha == null) {
/* 671 */       sort = config.isEnabled(SerializationConfig.Feature.SORT_PROPERTIES_ALPHABETICALLY);
/*     */     } else {
/* 673 */       sort = alpha.booleanValue();
/*     */     }
/* 675 */     if ((sort) || (!creatorProps.isEmpty()) || (propOrder != null)) {
/* 676 */       props = _sortBeanProperties(props, creatorProps, propOrder, sort);
/*     */     }
/* 678 */     return props;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void processViews(SerializationConfig config, BeanSerializerBuilder builder)
/*     */   {
/* 696 */     List<BeanPropertyWriter> props = builder.getProperties();
/* 697 */     boolean includeByDefault = config.isEnabled(SerializationConfig.Feature.DEFAULT_VIEW_INCLUSION);
/* 698 */     int propCount = props.size();
/* 699 */     int viewsFound = 0;
/* 700 */     BeanPropertyWriter[] filtered = new BeanPropertyWriter[propCount];
/*     */     
/* 702 */     for (int i = 0; i < propCount; i++) {
/* 703 */       BeanPropertyWriter bpw = (BeanPropertyWriter)props.get(i);
/* 704 */       Class<?>[] views = bpw.getViews();
/* 705 */       if (views == null) {
/* 706 */         if (includeByDefault) {
/* 707 */           filtered[i] = bpw;
/*     */         }
/*     */       } else {
/* 710 */         viewsFound++;
/* 711 */         filtered[i] = constructFilteredBeanWriter(bpw, views);
/*     */       }
/*     */     }
/*     */     
/* 715 */     if ((includeByDefault) && (viewsFound == 0)) {
/* 716 */       return;
/*     */     }
/* 718 */     builder.setFilteredProperties(filtered);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected <T extends AnnotatedMember> void removeIgnorableTypes(SerializationConfig config, BasicBeanDescription beanDesc, Map<String, T> props)
/*     */   {
/* 729 */     if (props.isEmpty()) {
/* 730 */       return;
/*     */     }
/* 732 */     AnnotationIntrospector intr = config.getAnnotationIntrospector();
/* 733 */     Iterator<Map.Entry<String, T>> it = props.entrySet().iterator();
/* 734 */     HashMap<Class<?>, Boolean> ignores = new HashMap();
/* 735 */     while (it.hasNext()) {
/* 736 */       Map.Entry<String, T> entry = (Map.Entry)it.next();
/* 737 */       Class<?> type = ((AnnotatedMember)entry.getValue()).getRawType();
/* 738 */       Boolean result = (Boolean)ignores.get(type);
/* 739 */       if (result == null) {
/* 740 */         BasicBeanDescription desc = (BasicBeanDescription)config.introspectClassAnnotations(type);
/* 741 */         AnnotatedClass ac = desc.getClassInfo();
/* 742 */         result = intr.isIgnorableType(ac);
/*     */         
/* 744 */         if (result == null) {
/* 745 */           result = Boolean.FALSE;
/*     */         }
/* 747 */         ignores.put(type, result);
/*     */       }
/*     */       
/* 750 */       if (result.booleanValue()) {
/* 751 */         it.remove();
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected BeanPropertyWriter _constructWriter(SerializationConfig config, TypeBindings typeContext, PropertyBuilder pb, boolean staticTyping, String name, AnnotatedMember accessor)
/*     */     throws JsonMappingException
/*     */   {
/* 770 */     if (config.isEnabled(SerializationConfig.Feature.CAN_OVERRIDE_ACCESS_MODIFIERS)) {
/* 771 */       accessor.fixAccess();
/*     */     }
/* 773 */     JavaType type = accessor.getType(typeContext);
/* 774 */     BeanProperty.Std property = new BeanProperty.Std(name, type, pb.getClassAnnotations(), accessor);
/*     */     
/*     */ 
/* 777 */     JsonSerializer<Object> annotatedSerializer = findSerializerFromAnnotation(config, accessor, property);
/*     */     
/* 779 */     TypeSerializer contentTypeSer = null;
/* 780 */     if (ClassUtil.isCollectionMapOrArray(type.getRawClass())) {
/* 781 */       contentTypeSer = findPropertyContentTypeSerializer(type, config, accessor, property);
/*     */     }
/*     */     
/*     */ 
/* 785 */     TypeSerializer typeSer = findPropertyTypeSerializer(type, config, accessor, property);
/* 786 */     BeanPropertyWriter pbw = pb.buildWriter(name, type, annotatedSerializer, typeSer, contentTypeSer, accessor, staticTyping);
/*     */     
/*     */ 
/* 789 */     AnnotationIntrospector intr = config.getAnnotationIntrospector();
/* 790 */     pbw.setViews(intr.findSerializationViews(accessor));
/* 791 */     return pbw;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected List<BeanPropertyWriter> _sortBeanProperties(List<BeanPropertyWriter> props, List<String> creatorProps, String[] propertyOrder, boolean sort)
/*     */   {
/* 801 */     int size = props.size();
/*     */     Map<String, BeanPropertyWriter> all;
/*     */     Map<String, BeanPropertyWriter> all;
/* 804 */     if (sort) {
/* 805 */       all = new TreeMap();
/*     */     } else {
/* 807 */       all = new LinkedHashMap(size * 2);
/*     */     }
/*     */     
/* 810 */     for (BeanPropertyWriter w : props) {
/* 811 */       all.put(w.getName(), w);
/*     */     }
/* 813 */     Map<String, BeanPropertyWriter> ordered = new LinkedHashMap(size * 2);
/*     */     
/* 815 */     if (propertyOrder != null) {
/* 816 */       for (String name : propertyOrder) {
/* 817 */         BeanPropertyWriter w = (BeanPropertyWriter)all.get(name);
/* 818 */         if (w != null) {
/* 819 */           ordered.put(name, w);
/*     */         }
/*     */       }
/*     */     }
/*     */     
/* 824 */     for (String name : creatorProps) {
/* 825 */       BeanPropertyWriter w = (BeanPropertyWriter)all.get(name);
/* 826 */       if (w != null) {
/* 827 */         ordered.put(name, w);
/*     */       }
/*     */     }
/*     */     
/* 831 */     ordered.putAll(all);
/* 832 */     return new ArrayList(ordered.values());
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\ser\BeanSerializerFactory.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */