/*     */ package org.codehaus.janino;
/*     */ 
/*     */ import java.io.BufferedReader;
/*     */ import java.io.BufferedWriter;
/*     */ import java.io.FileReader;
/*     */ import java.io.IOException;
/*     */ import java.io.InputStreamReader;
/*     */ import java.io.OutputStreamWriter;
/*     */ import java.io.PrintWriter;
/*     */ import java.io.StringReader;
/*     */ import java.io.Writer;
/*     */ import java.util.Collection;
/*     */ import java.util.HashMap;
/*     */ import java.util.HashSet;
/*     */ import java.util.Iterator;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.Set;
/*     */ import org.codehaus.janino.util.AutoIndentWriter;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class UnparseVisitor
/*     */   implements Visitor.ComprehensiveVisitor
/*     */ {
/*     */   protected final AutoIndentWriter aiw;
/*     */   protected final PrintWriter pw;
/*     */   
/*     */   public static void main(String[] args)
/*     */     throws Exception
/*     */   {
/*  61 */     Writer w = new BufferedWriter(new OutputStreamWriter(System.out));
/*  62 */     for (int i = 0; i < args.length; i++) {
/*  63 */       String fileName = args[i];
/*     */       
/*     */ 
/*  66 */       FileReader r = new FileReader(fileName);
/*     */       Java.CompilationUnit cu;
/*     */       try {
/*  69 */         cu = new Parser(new Scanner(fileName, r)).parseCompilationUnit();
/*     */       } finally {
/*  71 */         r.close();
/*     */       }
/*     */       
/*     */ 
/*  75 */       unparse(cu, w);
/*     */     }
/*  77 */     w.flush();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public static void unparse(Java.CompilationUnit cu, Writer w)
/*     */   {
/*  84 */     UnparseVisitor uv = new UnparseVisitor(w);
/*  85 */     uv.unparseCompilationUnit(cu);
/*  86 */     uv.close();
/*     */   }
/*     */   
/*     */   public UnparseVisitor(Writer w) {
/*  90 */     this.aiw = new AutoIndentWriter(w);
/*  91 */     this.pw = new PrintWriter(this.aiw, true);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public void close()
/*     */   {
/*  99 */     this.pw.close();
/*     */   }
/*     */   
/*     */   public void unparseCompilationUnit(Java.CompilationUnit cu) {
/* 103 */     if (cu.optionalPackageDeclaration != null) {
/* 104 */       this.pw.println();
/* 105 */       this.pw.println("package " + cu.optionalPackageDeclaration.packageName + ';'); }
/*     */     Iterator it;
/* 107 */     if (!cu.importDeclarations.isEmpty()) {
/* 108 */       this.pw.println();
/* 109 */       for (it = cu.importDeclarations.iterator(); it.hasNext();) {
/* 110 */         ((Java.CompilationUnit.ImportDeclaration)it.next()).accept(this);
/*     */       }
/*     */     }
/* 113 */     for (Iterator it = cu.packageMemberTypeDeclarations.iterator(); it.hasNext();) {
/* 114 */       this.pw.println();
/* 115 */       unparseTypeDeclaration((Java.PackageMemberTypeDeclaration)it.next());
/* 116 */       this.pw.println();
/*     */     }
/*     */   }
/*     */   
/*     */   public void visitSingleTypeImportDeclaration(Java.CompilationUnit.SingleTypeImportDeclaration stid) {
/* 121 */     this.pw.println("import " + Java.join(stid.identifiers, ".") + ';');
/*     */   }
/*     */   
/* 124 */   public void visitTypeImportOnDemandDeclaration(Java.CompilationUnit.TypeImportOnDemandDeclaration tiodd) { this.pw.println("import " + Java.join(tiodd.identifiers, ".") + ".*;"); }
/*     */   
/*     */   public void visitSingleStaticImportDeclaration(Java.CompilationUnit.SingleStaticImportDeclaration ssid) {
/* 127 */     this.pw.println("import static " + Java.join(ssid.identifiers, ".") + ';');
/*     */   }
/*     */   
/* 130 */   public void visitStaticImportOnDemandDeclaration(Java.CompilationUnit.StaticImportOnDemandDeclaration siodd) { this.pw.println("import static " + Java.join(siodd.identifiers, ".") + ".*;"); }
/*     */   
/*     */   public void visitLocalClassDeclaration(Java.LocalClassDeclaration lcd)
/*     */   {
/* 134 */     unparseNamedClassDeclaration(lcd);
/*     */   }
/*     */   
/* 137 */   public void visitMemberClassDeclaration(Java.MemberClassDeclaration mcd) { unparseNamedClassDeclaration(mcd); }
/*     */   
/*     */   public void visitMemberInterfaceDeclaration(Java.MemberInterfaceDeclaration mid) {
/* 140 */     unparseInterfaceDeclaration(mid);
/*     */   }
/*     */   
/* 143 */   public void visitPackageMemberClassDeclaration(Java.PackageMemberClassDeclaration pmcd) { unparseNamedClassDeclaration(pmcd); }
/*     */   
/*     */ 
/* 146 */   public void visitPackageMemberInterfaceDeclaration(Java.PackageMemberInterfaceDeclaration pmid) { unparseInterfaceDeclaration(pmid); }
/*     */   
/*     */   public void visitConstructorDeclarator(Java.ConstructorDeclarator cd) {
/* 149 */     unparseDocComment(cd);
/* 150 */     unparseModifiers(cd.modifiers);
/* 151 */     Java.ClassDeclaration declaringClass = cd.getDeclaringClass();
/* 152 */     this.pw.print((declaringClass instanceof Java.NamedClassDeclaration) ? ((Java.NamedClassDeclaration)declaringClass).name : "UNNAMED");
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 157 */     unparseFunctionDeclaratorRest(cd);
/* 158 */     this.pw.print(' ');
/* 159 */     if (cd.optionalConstructorInvocation != null) {
/* 160 */       this.pw.println('{');
/* 161 */       this.pw.print(65533);
/* 162 */       unparseBlockStatement(cd.optionalConstructorInvocation);
/* 163 */       this.pw.println(';');
/*     */       
/* 165 */       if (!cd.optionalStatements.isEmpty()) {
/* 166 */         this.pw.println();
/* 167 */         unparseStatements(cd.optionalStatements);
/*     */       }
/* 169 */       this.pw.print("￼}");
/*     */     }
/* 171 */     else if (cd.optionalStatements.isEmpty()) {
/* 172 */       this.pw.print("{}");
/*     */     }
/*     */     else {
/* 175 */       this.pw.println('{');
/* 176 */       this.pw.print(65533);
/* 177 */       unparseStatements(cd.optionalStatements);
/* 178 */       this.pw.print("￼}");
/*     */     }
/*     */   }
/*     */   
/* 182 */   public void visitMethodDeclarator(Java.MethodDeclarator md) { unparseDocComment(md);
/* 183 */     unparseModifiers(md.modifiers);
/* 184 */     unparseType(md.type);
/* 185 */     this.pw.print(' ' + md.name);
/* 186 */     unparseFunctionDeclaratorRest(md);
/* 187 */     if (md.optionalStatements == null) {
/* 188 */       this.pw.print(';');
/*     */     }
/* 190 */     else if (md.optionalStatements.isEmpty()) {
/* 191 */       this.pw.print(" {}");
/*     */     }
/*     */     else {
/* 194 */       this.pw.println(" {");
/* 195 */       this.pw.print(65533);
/* 196 */       unparseStatements(md.optionalStatements);
/* 197 */       this.pw.print(65532);
/* 198 */       this.pw.print('}');
/*     */     }
/*     */   }
/*     */   
/* 202 */   public void visitFieldDeclaration(Java.FieldDeclaration fd) { unparseDocComment(fd);
/* 203 */     unparseModifiers(fd.modifiers);
/* 204 */     unparseType(fd.type);
/* 205 */     this.pw.print(' ');
/* 206 */     for (int i = 0; i < fd.variableDeclarators.length; i++) {
/* 207 */       if (i > 0) this.pw.print(", ");
/* 208 */       unparseVariableDeclarator(fd.variableDeclarators[i]);
/*     */     }
/* 210 */     this.pw.print(';');
/*     */   }
/*     */   
/* 213 */   public void visitInitializer(Java.Initializer i) { if (i.statiC) this.pw.print("static ");
/* 214 */     unparseBlockStatement(i.block);
/*     */   }
/*     */   
/* 217 */   public void visitBlock(Java.Block b) { if (b.statements.isEmpty()) {
/* 218 */       this.pw.print("{}");
/* 219 */       return;
/*     */     }
/* 221 */     this.pw.println('{');
/* 222 */     this.pw.print(65533);
/* 223 */     unparseStatements(b.statements);
/* 224 */     this.pw.print("￼}");
/*     */   }
/*     */   
/*     */   private void unparseStatements(List statements) {
/* 228 */     int state = -1;
/* 229 */     for (Iterator it = statements.iterator(); it.hasNext();) {
/* 230 */       Java.BlockStatement bs = (Java.BlockStatement)it.next();
/* 231 */       int x = (bs instanceof Java.SynchronizedStatement) ? 4 : (bs instanceof Java.LocalVariableDeclarationStatement) ? 3 : (bs instanceof Java.LocalClassDeclarationStatement) ? 2 : (bs instanceof Java.Block) ? 1 : 99;
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 238 */       if ((state != -1) && (state != x)) this.pw.println(65534);
/* 239 */       state = x;
/*     */       
/* 241 */       unparseBlockStatement(bs);
/* 242 */       this.pw.println();
/*     */     }
/*     */   }
/*     */   
/* 246 */   public void visitBreakStatement(Java.BreakStatement bs) { this.pw.print("break");
/* 247 */     if (bs.optionalLabel != null) this.pw.print(' ' + bs.optionalLabel);
/* 248 */     this.pw.print(';');
/*     */   }
/*     */   
/* 251 */   public void visitContinueStatement(Java.ContinueStatement cs) { this.pw.print("continue");
/* 252 */     if (cs.optionalLabel != null) this.pw.print(' ' + cs.optionalLabel);
/* 253 */     this.pw.print(';');
/*     */   }
/*     */   
/* 256 */   public void visitDoStatement(Java.DoStatement ds) { this.pw.print("do ");
/* 257 */     unparseBlockStatement(ds.body);
/* 258 */     this.pw.print("while (");
/* 259 */     unparse(ds.condition);
/* 260 */     this.pw.print(");");
/*     */   }
/*     */   
/* 263 */   public void visitEmptyStatement(Java.EmptyStatement es) { this.pw.print(';'); }
/*     */   
/*     */   public void visitExpressionStatement(Java.ExpressionStatement es) {
/* 266 */     unparse(es.rvalue);
/* 267 */     this.pw.print(';');
/*     */   }
/*     */   
/* 270 */   public void visitForStatement(Java.ForStatement fs) { this.pw.print("for (");
/* 271 */     if (fs.optionalInit != null) {
/* 272 */       unparseBlockStatement(fs.optionalInit);
/*     */     } else {
/* 274 */       this.pw.print(';');
/*     */     }
/* 276 */     if (fs.optionalCondition != null) {
/* 277 */       this.pw.print(' ');
/* 278 */       unparse(fs.optionalCondition);
/*     */     }
/* 280 */     this.pw.print(';');
/* 281 */     if (fs.optionalUpdate != null) {
/* 282 */       this.pw.print(' ');
/* 283 */       for (int i = 0; i < fs.optionalUpdate.length; i++) {
/* 284 */         if (i > 0) this.pw.print(", ");
/* 285 */         unparse(fs.optionalUpdate[i]);
/*     */       }
/*     */     }
/* 288 */     this.pw.print(") ");
/* 289 */     unparseBlockStatement(fs.body);
/*     */   }
/*     */   
/* 292 */   public void visitIfStatement(Java.IfStatement is) { this.pw.print("if (");
/* 293 */     unparse(is.condition);
/* 294 */     this.pw.print(") ");
/* 295 */     unparseBlockStatement(is.thenStatement);
/* 296 */     if (is.optionalElseStatement != null) {
/* 297 */       this.pw.println(" else");
/* 298 */       unparseBlockStatement(is.optionalElseStatement);
/*     */     }
/*     */   }
/*     */   
/* 302 */   public void visitLabeledStatement(Java.LabeledStatement ls) { this.pw.println(ls.label + ':');
/* 303 */     unparseBlockStatement(ls.body);
/*     */   }
/*     */   
/* 306 */   public void visitLocalClassDeclarationStatement(Java.LocalClassDeclarationStatement lcds) { unparseTypeDeclaration(lcds.lcd); }
/*     */   
/*     */   public void visitLocalVariableDeclarationStatement(Java.LocalVariableDeclarationStatement lvds) {
/* 309 */     unparseModifiers(lvds.modifiers);
/* 310 */     unparseType(lvds.type);
/* 311 */     this.pw.print(' ');
/* 312 */     this.pw.print(65535);
/* 313 */     unparseVariableDeclarator(lvds.variableDeclarators[0]);
/* 314 */     for (int i = 1; i < lvds.variableDeclarators.length; i++) {
/* 315 */       this.pw.print(", ");
/* 316 */       unparseVariableDeclarator(lvds.variableDeclarators[i]);
/*     */     }
/* 318 */     this.pw.print(';');
/*     */   }
/*     */   
/* 321 */   public void visitReturnStatement(Java.ReturnStatement rs) { this.pw.print("return");
/* 322 */     if (rs.optionalReturnValue != null) {
/* 323 */       this.pw.print(' ');
/* 324 */       unparse(rs.optionalReturnValue);
/*     */     }
/* 326 */     this.pw.print(';');
/*     */   }
/*     */   
/* 329 */   public void visitSwitchStatement(Java.SwitchStatement ss) { this.pw.print("switch (");
/* 330 */     unparse(ss.condition);
/* 331 */     this.pw.println(") {");
/* 332 */     for (Iterator it = ss.sbsgs.iterator(); it.hasNext();) {
/* 333 */       Java.SwitchStatement.SwitchBlockStatementGroup sbgs = (Java.SwitchStatement.SwitchBlockStatementGroup)it.next();
/*     */       
/*     */ 
/* 336 */       this.pw.print(65532);
/*     */       try {
/* 338 */         for (Iterator it2 = sbgs.caseLabels.iterator(); it2.hasNext();) {
/* 339 */           Java.Rvalue rv = (Java.Rvalue)it2.next();
/* 340 */           this.pw.print("case ");
/* 341 */           unparse(rv);
/* 342 */           this.pw.println(':');
/*     */         }
/* 344 */         if (sbgs.hasDefaultLabel) this.pw.println("default:");
/*     */       } finally {
/* 346 */         this.pw.print(65533);
/*     */       }
/* 348 */       for (it2 = sbgs.blockStatements.iterator(); it2.hasNext();) {
/* 349 */         unparseBlockStatement((Java.BlockStatement)it2.next());
/* 350 */         this.pw.println();
/*     */       } }
/*     */     Iterator it2;
/* 353 */     this.pw.print('}');
/*     */   }
/*     */   
/* 356 */   public void visitSynchronizedStatement(Java.SynchronizedStatement ss) { this.pw.print("synchronized (");
/* 357 */     unparse(ss.expression);
/* 358 */     this.pw.print(") ");
/* 359 */     unparseBlockStatement(ss.body);
/*     */   }
/*     */   
/* 362 */   public void visitThrowStatement(Java.ThrowStatement ts) { this.pw.print("throw ");
/* 363 */     unparse(ts.expression);
/* 364 */     this.pw.print(';');
/*     */   }
/*     */   
/* 367 */   public void visitTryStatement(Java.TryStatement ts) { this.pw.print("try ");
/* 368 */     unparseBlockStatement(ts.body);
/* 369 */     for (Iterator it = ts.catchClauses.iterator(); it.hasNext();) {
/* 370 */       Java.CatchClause cc = (Java.CatchClause)it.next();
/* 371 */       this.pw.print(" catch (");
/* 372 */       unparseFormalParameter(cc.caughtException);
/* 373 */       this.pw.print(") ");
/* 374 */       unparseBlockStatement(cc.body);
/*     */     }
/* 376 */     if (ts.optionalFinally != null) {
/* 377 */       this.pw.print(" finally ");
/* 378 */       unparseBlockStatement(ts.optionalFinally);
/*     */     }
/*     */   }
/*     */   
/* 382 */   public void visitWhileStatement(Java.WhileStatement ws) { this.pw.print("while (");
/* 383 */     unparse(ws.condition);
/* 384 */     this.pw.print(") ");
/* 385 */     unparseBlockStatement(ws.body);
/*     */   }
/*     */   
/* 388 */   public void unparseVariableDeclarator(Java.VariableDeclarator vd) { this.pw.print(vd.name);
/* 389 */     for (int i = 0; i < vd.brackets; i++) this.pw.print("[]");
/* 390 */     if (vd.optionalInitializer != null) {
/* 391 */       this.pw.print(" = ");
/* 392 */       unparseArrayInitializerOrRvalue(vd.optionalInitializer);
/*     */     }
/*     */   }
/*     */   
/* 396 */   public void unparseFormalParameter(Java.FunctionDeclarator.FormalParameter fp) { if (fp.finaL) this.pw.print("final ");
/* 397 */     unparseType(fp.type);
/* 398 */     this.pw.print(" ￿" + fp.name);
/*     */   }
/*     */   
/* 401 */   public void visitMethodInvocation(Java.MethodInvocation mi) { if (mi.optionalTarget != null) {
/* 402 */       unparseLhs(mi.optionalTarget, ".");
/* 403 */       this.pw.print('.');
/*     */     }
/* 405 */     this.pw.print(mi.methodName);
/* 406 */     unparseFunctionInvocationArguments(mi.arguments);
/*     */   }
/*     */   
/* 409 */   public void visitAlternateConstructorInvocation(Java.AlternateConstructorInvocation aci) { this.pw.print("this");
/* 410 */     unparseFunctionInvocationArguments(aci.arguments);
/*     */   }
/*     */   
/* 413 */   public void visitSuperConstructorInvocation(Java.SuperConstructorInvocation sci) { if (sci.optionalQualification != null) {
/* 414 */       unparseLhs(sci.optionalQualification, ".");
/* 415 */       this.pw.print('.');
/*     */     }
/* 417 */     this.pw.print("super");
/* 418 */     unparseFunctionInvocationArguments(sci.arguments);
/*     */   }
/*     */   
/* 421 */   public void visitNewClassInstance(Java.NewClassInstance nci) { if (nci.optionalQualification != null) {
/* 422 */       unparseLhs(nci.optionalQualification, ".");
/* 423 */       this.pw.print('.');
/*     */     }
/* 425 */     this.pw.print("new " + nci.type.toString());
/* 426 */     unparseFunctionInvocationArguments(nci.arguments);
/*     */   }
/*     */   
/* 429 */   public void visitAssignment(Java.Assignment a) { unparseLhs(a.lhs, a.operator);
/* 430 */     this.pw.print(' ' + a.operator + ' ');
/* 431 */     unparseRhs(a.rhs, a.operator); }
/*     */   
/* 433 */   public void visitAmbiguousName(Java.AmbiguousName an) { this.pw.print(an.toString()); }
/*     */   
/* 435 */   public void visitArrayAccessExpression(Java.ArrayAccessExpression aae) { unparseLhs(aae.lhs, "[ ]");
/* 436 */     this.pw.print('[');
/* 437 */     unparse(aae.index);
/* 438 */     this.pw.print(']');
/*     */   }
/*     */   
/* 441 */   public void visitArrayLength(Java.ArrayLength al) { unparseLhs(al.lhs, ".");
/* 442 */     this.pw.print(".length");
/*     */   }
/*     */   
/* 445 */   public void visitArrayType(Java.ArrayType at) { unparseType(at.componentType);
/* 446 */     this.pw.print("[]");
/*     */   }
/*     */   
/* 449 */   public void visitBasicType(Java.BasicType bt) { this.pw.print(bt.toString()); }
/*     */   
/*     */   public void visitBinaryOperation(Java.BinaryOperation bo) {
/* 452 */     unparseLhs(bo.lhs, bo.op);
/* 453 */     this.pw.print(' ' + bo.op + ' ');
/* 454 */     unparseRhs(bo.rhs, bo.op);
/*     */   }
/*     */   
/* 457 */   public void visitCast(Java.Cast c) { this.pw.print('(');
/* 458 */     unparseType(c.targetType);
/* 459 */     this.pw.print(") ");
/* 460 */     unparseRhs(c.value, "cast");
/*     */   }
/*     */   
/* 463 */   public void visitClassLiteral(Java.ClassLiteral cl) { unparseType(cl.type);
/* 464 */     this.pw.print(".class");
/*     */   }
/*     */   
/* 467 */   public void visitConditionalExpression(Java.ConditionalExpression ce) { unparseLhs(ce.lhs, "?:");
/* 468 */     this.pw.print(" ? ");
/* 469 */     unparseLhs(ce.mhs, "?:");
/* 470 */     this.pw.print(" : ");
/* 471 */     unparseRhs(ce.rhs, "?:");
/*     */   }
/*     */   
/* 474 */   public void visitCrement(Java.Crement c) { if (c.pre) {
/* 475 */       this.pw.print(c.operator);
/* 476 */       unparseUnaryOperation(c.operand, c.operator + "x");
/*     */     }
/*     */     else {
/* 479 */       unparseUnaryOperation(c.operand, "x" + c.operator);
/* 480 */       this.pw.print(c.operator);
/*     */     }
/*     */   }
/*     */   
/* 484 */   public void visitFieldAccess(Java.FieldAccess fa) { unparseLhs(fa.lhs, ".");
/* 485 */     this.pw.print('.' + fa.field.getName());
/*     */   }
/*     */   
/* 488 */   public void visitFieldAccessExpression(Java.FieldAccessExpression fae) { unparseLhs(fae.lhs, ".");
/* 489 */     this.pw.print('.' + fae.fieldName);
/*     */   }
/*     */   
/* 492 */   public void visitSuperclassFieldAccessExpression(Java.SuperclassFieldAccessExpression scfae) { if (scfae.optionalQualification != null) {
/* 493 */       unparseType(scfae.optionalQualification);
/* 494 */       this.pw.print(".super." + scfae.fieldName);
/*     */     }
/*     */     else {
/* 497 */       this.pw.print("super." + scfae.fieldName);
/*     */     }
/*     */   }
/*     */   
/* 501 */   public void visitInstanceof(Java.Instanceof io) { unparseLhs(io.lhs, "instanceof");
/* 502 */     this.pw.print(" instanceof ");
/* 503 */     unparseType(io.rhs); }
/*     */   
/* 505 */   public void visitLiteral(Java.Literal l) { this.pw.print(l.toString()); }
/* 506 */   public void visitLocalVariableAccess(Java.LocalVariableAccess lva) { this.pw.print(lva.toString()); }
/*     */   
/* 508 */   public void visitNewArray(Java.NewArray na) { this.pw.print("new ");
/* 509 */     unparseType(na.type);
/* 510 */     for (int i = 0; i < na.dimExprs.length; i++) {
/* 511 */       this.pw.print('[');
/* 512 */       unparse(na.dimExprs[i]);
/* 513 */       this.pw.print(']');
/*     */     }
/* 515 */     for (int i = 0; i < na.dims; i++)
/* 516 */       this.pw.print("[]");
/*     */   }
/*     */   
/*     */   public void visitNewInitializedArray(Java.NewInitializedArray nai) {
/* 520 */     this.pw.print("new ");
/* 521 */     unparseType(nai.arrayType);
/* 522 */     this.pw.print(" ");
/* 523 */     unparseArrayInitializerOrRvalue(nai.arrayInitializer); }
/*     */   
/* 525 */   public void visitPackage(Java.Package p) { this.pw.print(p.toString()); }
/* 526 */   public void visitParameterAccess(Java.ParameterAccess pa) { this.pw.print(pa.toString()); }
/*     */   
/* 528 */   public void visitQualifiedThisReference(Java.QualifiedThisReference qtr) { unparseType(qtr.qualification);
/* 529 */     this.pw.print(".this"); }
/*     */   
/* 531 */   public void visitReferenceType(Java.ReferenceType rt) { this.pw.print(rt.toString()); }
/* 532 */   public void visitRvalueMemberType(Java.RvalueMemberType rmt) { this.pw.print(rmt.toString()); }
/* 533 */   public void visitSimpleType(Java.SimpleType st) { this.pw.print(st.toString()); }
/*     */   
/* 535 */   public void visitSuperclassMethodInvocation(Java.SuperclassMethodInvocation smi) { this.pw.print("super." + smi.methodName);
/* 536 */     unparseFunctionInvocationArguments(smi.arguments);
/*     */   }
/*     */   
/* 539 */   public void visitThisReference(Java.ThisReference tr) { this.pw.print("this"); }
/*     */   
/*     */   public void visitUnaryOperation(Java.UnaryOperation uo) {
/* 542 */     this.pw.print(uo.operator);
/* 543 */     unparseUnaryOperation(uo.operand, uo.operator + "x");
/*     */   }
/*     */   
/* 546 */   public void visitParenthesizedExpression(Java.ParenthesizedExpression pe) { this.pw.print('(');
/* 547 */     unparse(pe.value);
/* 548 */     this.pw.print(')');
/*     */   }
/*     */   
/*     */ 
/*     */   private void unparseBlockStatement(Java.BlockStatement blockStatement)
/*     */   {
/* 554 */     blockStatement.accept(this);
/*     */   }
/*     */   
/*     */   private void unparseTypeDeclaration(Java.TypeDeclaration typeDeclaration) {
/* 558 */     typeDeclaration.accept(this);
/*     */   }
/*     */   
/*     */   private void unparseType(Java.Type type) {
/* 562 */     type.accept(this);
/*     */   }
/*     */   
/*     */   private void unparse(Java.Atom operand) {
/* 566 */     operand.accept(this);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private void unparseUnaryOperation(Java.Rvalue operand, String unaryOperator)
/*     */   {
/* 576 */     int cmp = comparePrecedence(unaryOperator, operand);
/* 577 */     unparse(operand, cmp < 0);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private void unparseLhs(Java.Atom lhs, String binaryOperator)
/*     */   {
/* 588 */     int cmp = comparePrecedence(binaryOperator, lhs);
/* 589 */     unparse(lhs, (cmp < 0) || ((cmp == 0) && (isLeftAssociate(binaryOperator))));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private void unparseRhs(Java.Rvalue rhs, String binaryOperator)
/*     */   {
/* 598 */     int cmp = comparePrecedence(binaryOperator, rhs);
/* 599 */     unparse(rhs, (cmp < 0) || ((cmp == 0) && (isRightAssociate(binaryOperator))));
/*     */   }
/*     */   
/*     */   private void unparse(Java.Atom operand, boolean natural) {
/* 603 */     if (!natural) this.pw.print("((( ");
/* 604 */     unparse(operand);
/* 605 */     if (!natural) { this.pw.print(" )))");
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private static boolean isRightAssociate(String op)
/*     */   {
/* 615 */     return RIGHT_ASSOCIATIVE_OPERATORS.contains(op);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private static boolean isLeftAssociate(String op)
/*     */   {
/* 625 */     return LEFT_ASSOCIATIVE_OPERATORS.contains(op);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private static int comparePrecedence(String operator, Java.Atom operand)
/*     */   {
/* 637 */     if ((operand instanceof Java.BinaryOperation)) {
/* 638 */       return getOperatorPrecedence(operator) - getOperatorPrecedence(((Java.BinaryOperation)operand).op);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 643 */     if ((operand instanceof Java.UnaryOperation)) {
/* 644 */       return getOperatorPrecedence(operator) - getOperatorPrecedence(((Java.UnaryOperation)operand).operator + "x");
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 649 */     if ((operand instanceof Java.ConditionalExpression)) {
/* 650 */       return getOperatorPrecedence(operator) - getOperatorPrecedence("?:");
/*     */     }
/* 652 */     if ((operand instanceof Java.Instanceof)) {
/* 653 */       return getOperatorPrecedence(operator) - getOperatorPrecedence("instanceof");
/*     */     }
/* 655 */     if ((operand instanceof Java.Cast)) {
/* 656 */       return getOperatorPrecedence(operator) - getOperatorPrecedence("cast");
/*     */     }
/* 658 */     if (((operand instanceof Java.MethodInvocation)) || ((operand instanceof Java.FieldAccess))) {
/* 659 */       return getOperatorPrecedence(operator) - getOperatorPrecedence(".");
/*     */     }
/* 661 */     if ((operand instanceof Java.NewArray)) {
/* 662 */       return getOperatorPrecedence(operator) - getOperatorPrecedence("new");
/*     */     }
/* 664 */     if ((operand instanceof Java.Crement)) {
/* 665 */       Java.Crement c = (Java.Crement)operand;
/* 666 */       return getOperatorPrecedence(operator) - getOperatorPrecedence("x" + c.operator);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 673 */     return -1;
/*     */   }
/*     */   
/*     */ 
/* 677 */   private static int getOperatorPrecedence(String operator) { return ((Integer)OPERATOR_PRECEDENCE.get(operator)).intValue(); }
/*     */   
/* 679 */   private static final Set LEFT_ASSOCIATIVE_OPERATORS = new HashSet();
/* 680 */   private static final Set RIGHT_ASSOCIATIVE_OPERATORS = new HashSet();
/* 681 */   private static final Set UNARY_OPERATORS = new HashSet();
/* 682 */   private static final Map OPERATOR_PRECEDENCE = new HashMap();
/*     */   
/* 684 */   static { Object[] ops = { RIGHT_ASSOCIATIVE_OPERATORS, "=", "*=", "/=", "%=", "+=", "-=", "<<=", ">>=", ">>>=", "&=", "^=", "|=", RIGHT_ASSOCIATIVE_OPERATORS, "?:", LEFT_ASSOCIATIVE_OPERATORS, "||", LEFT_ASSOCIATIVE_OPERATORS, "&&", LEFT_ASSOCIATIVE_OPERATORS, "|", LEFT_ASSOCIATIVE_OPERATORS, "^", LEFT_ASSOCIATIVE_OPERATORS, "&", LEFT_ASSOCIATIVE_OPERATORS, "==", "!=", LEFT_ASSOCIATIVE_OPERATORS, "<", ">", "<=", ">=", "instanceof", LEFT_ASSOCIATIVE_OPERATORS, "<<", ">>", ">>>", LEFT_ASSOCIATIVE_OPERATORS, "+", "-", LEFT_ASSOCIATIVE_OPERATORS, "*", "/", "%", RIGHT_ASSOCIATIVE_OPERATORS, "cast", UNARY_OPERATORS, "++x", "--x", "+x", "-x", "~x", "!x", UNARY_OPERATORS, "x++", "x--", LEFT_ASSOCIATIVE_OPERATORS, "new", LEFT_ASSOCIATIVE_OPERATORS, ".", "[ ]" };
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 704 */     int precedence = 0;
/* 705 */     int i = 0;
/* 706 */     for (;;) { Set s = (Set)ops[(i++)];
/* 707 */       Integer pi = new Integer(++precedence);
/*     */       for (;;) {
/* 709 */         if (i == ops.length) return;
/* 710 */         if (!(ops[i] instanceof String)) break;
/* 711 */         String op = (String)ops[(i++)];
/* 712 */         s.add(op);
/* 713 */         OPERATOR_PRECEDENCE.put(op, pi);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   private void unparseNamedClassDeclaration(Java.NamedClassDeclaration ncd) {
/* 719 */     unparseDocComment(ncd);
/* 720 */     unparseModifiers(ncd.getModifiers());
/* 721 */     this.pw.print("class " + ncd.name);
/* 722 */     if (ncd.optionalExtendedType != null) {
/* 723 */       this.pw.print(" extends ");
/* 724 */       unparseType(ncd.optionalExtendedType);
/*     */     }
/* 726 */     if (ncd.implementedTypes.length > 0) this.pw.print(" implements " + Java.join(ncd.implementedTypes, ", "));
/* 727 */     this.pw.println(" {");
/* 728 */     this.pw.print(65533);
/* 729 */     unparseClassDeclarationBody(ncd);
/* 730 */     this.pw.print("￼}");
/*     */   }
/*     */   
/* 733 */   private void unparseArrayInitializerOrRvalue(Java.ArrayInitializerOrRvalue aiorv) { if ((aiorv instanceof Java.Rvalue)) {
/* 734 */       unparse((Java.Rvalue)aiorv);
/*     */     }
/* 736 */     else if ((aiorv instanceof Java.ArrayInitializer)) {
/* 737 */       Java.ArrayInitializer ai = (Java.ArrayInitializer)aiorv;
/* 738 */       if (ai.values.length == 0) {
/* 739 */         this.pw.print("{}");
/*     */       }
/*     */       else {
/* 742 */         this.pw.print("{ ");
/* 743 */         unparseArrayInitializerOrRvalue(ai.values[0]);
/* 744 */         for (int i = 1; i < ai.values.length; i++) {
/* 745 */           this.pw.print(", ");
/* 746 */           unparseArrayInitializerOrRvalue(ai.values[i]);
/*     */         }
/* 748 */         this.pw.print(" }");
/*     */       }
/*     */     }
/*     */     else {
/* 752 */       throw new JaninoRuntimeException("Unexpected array initializer or rvalue class " + aiorv.getClass().getName());
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public void visitAnonymousClassDeclaration(Java.AnonymousClassDeclaration acd)
/*     */   {
/* 760 */     unparseType(acd.baseType);
/* 761 */     this.pw.println(" {");
/* 762 */     this.pw.print(65533);
/* 763 */     unparseClassDeclarationBody(acd);
/* 764 */     this.pw.print("￼}");
/*     */   }
/*     */   
/* 767 */   public void visitNewAnonymousClassInstance(Java.NewAnonymousClassInstance naci) { if (naci.optionalQualification != null) {
/* 768 */       unparseLhs(naci.optionalQualification, ".");
/* 769 */       this.pw.print('.');
/*     */     }
/* 771 */     this.pw.print("new " + naci.anonymousClassDeclaration.baseType.toString() + '(');
/* 772 */     for (int i = 0; i < naci.arguments.length; i++) {
/* 773 */       if (i > 0) this.pw.print(", ");
/* 774 */       unparse(naci.arguments[i]);
/*     */     }
/* 776 */     this.pw.println(") {");
/* 777 */     this.pw.print(65533);
/* 778 */     unparseClassDeclarationBody(naci.anonymousClassDeclaration);
/* 779 */     this.pw.print("￼}");
/*     */   }
/*     */   
/*     */   private void unparseClassDeclarationBody(Java.ClassDeclaration cd) {
/* 783 */     for (Iterator it = cd.constructors.iterator(); it.hasNext();) {
/* 784 */       this.pw.println();
/* 785 */       ((Java.ConstructorDeclarator)it.next()).accept(this);
/* 786 */       this.pw.println();
/*     */     }
/* 788 */     unparseAbstractTypeDeclarationBody(cd);
/* 789 */     for (Iterator it = cd.variableDeclaratorsAndInitializers.iterator(); it.hasNext();) {
/* 790 */       this.pw.println();
/* 791 */       ((Java.TypeBodyDeclaration)it.next()).accept(this);
/* 792 */       this.pw.println();
/*     */     }
/*     */   }
/*     */   
/* 796 */   private void unparseInterfaceDeclaration(Java.InterfaceDeclaration id) { unparseDocComment(id);
/* 797 */     unparseModifiers(id.getModifiers());
/*     */     
/* 799 */     if ((id.getModifiers() & 0x200) == 0) {
/* 800 */       this.pw.print("interface ");
/*     */     }
/* 802 */     this.pw.print(id.name);
/* 803 */     if (id.extendedTypes.length > 0) this.pw.print(" extends " + Java.join(id.extendedTypes, ", "));
/* 804 */     this.pw.println(" {");
/* 805 */     this.pw.print(65533);
/* 806 */     unparseAbstractTypeDeclarationBody(id);
/* 807 */     for (Iterator it = id.constantDeclarations.iterator(); it.hasNext();) {
/* 808 */       ((Java.TypeBodyDeclaration)it.next()).accept(this);
/* 809 */       this.pw.println();
/*     */     }
/* 811 */     this.pw.print("￼}");
/*     */   }
/*     */   
/*     */   private void unparseAbstractTypeDeclarationBody(Java.AbstractTypeDeclaration atd) {
/* 815 */     for (Iterator it = atd.getMethodDeclarations().iterator(); it.hasNext();) {
/* 816 */       this.pw.println();
/* 817 */       ((Java.MethodDeclarator)it.next()).accept(this);
/* 818 */       this.pw.println();
/*     */     }
/* 820 */     for (Iterator it = atd.getMemberTypeDeclarations().iterator(); it.hasNext();) {
/* 821 */       this.pw.println();
/* 822 */       ((Java.TypeBodyDeclaration)it.next()).accept(this);
/* 823 */       this.pw.println();
/*     */     }
/*     */   }
/*     */   
/* 827 */   private void unparseFunctionDeclaratorRest(Java.FunctionDeclarator fd) { boolean big = fd.formalParameters.length >= 4;
/* 828 */     this.pw.print('(');
/* 829 */     if (big) { this.pw.println();this.pw.print(65533); }
/* 830 */     for (int i = 0; i < fd.formalParameters.length; i++) {
/* 831 */       if (i > 0) {
/* 832 */         if (big) {
/* 833 */           this.pw.println(',');
/*     */         }
/*     */         else {
/* 836 */           this.pw.print(", ");
/*     */         }
/*     */       }
/* 839 */       unparseFormalParameter(fd.formalParameters[i]);
/*     */     }
/* 841 */     if (big) { this.pw.println();this.pw.print(65532); }
/* 842 */     this.pw.print(')');
/* 843 */     if (fd.thrownExceptions.length > 0) this.pw.print(" throws " + Java.join(fd.thrownExceptions, ", "));
/*     */   }
/*     */   
/* 846 */   private void unparseDocComment(Java.DocCommentable dc) { String optionalDocComment = dc.getDocComment();
/* 847 */     if (optionalDocComment != null) {
/* 848 */       this.pw.print("/**");
/* 849 */       BufferedReader br = new BufferedReader(new StringReader(optionalDocComment));
/*     */       for (;;) {
/*     */         String line;
/*     */         try {
/* 853 */           line = br.readLine();
/*     */         } catch (IOException e) {
/* 855 */           throw new JaninoRuntimeException();
/*     */         }
/* 857 */         if (line == null) break;
/* 858 */         this.pw.println(line);
/* 859 */         this.pw.print(" *");
/*     */       }
/* 861 */       this.pw.println("/");
/*     */     }
/*     */   }
/*     */   
/* 865 */   private void unparseModifiers(short modifiers) { if (modifiers != 0)
/* 866 */       this.pw.print(Mod.shortToString(modifiers) + ' ');
/*     */   }
/*     */   
/*     */   private void unparseFunctionInvocationArguments(Java.Rvalue[] arguments) {
/* 870 */     boolean big = arguments.length >= 5;
/* 871 */     this.pw.print('(');
/* 872 */     if (big) { this.pw.println();this.pw.print(65533); }
/* 873 */     for (int i = 0; i < arguments.length; i++) {
/* 874 */       if (i > 0) {
/* 875 */         if (big) {
/* 876 */           this.pw.println(',');
/*     */         }
/*     */         else {
/* 879 */           this.pw.print(", ");
/*     */         }
/*     */       }
/* 882 */       unparse(arguments[i]);
/*     */     }
/* 884 */     if (big) { this.pw.println();this.pw.print(65532); }
/* 885 */     this.pw.print(')');
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\janino\UnparseVisitor.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       0.7.1
 */