package prefuse.data.io;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.data.Schema;
import prefuse.data.Table;
import prefuse.util.io.XMLWriter;

public class GraphMLWriter
  extends AbstractGraphWriter
{
  private static final HashMap TYPES = new HashMap();
  
  public void writeGraph(Graph paramGraph, OutputStream paramOutputStream)
    throws DataIOException
  {
    Schema localSchema1 = paramGraph.getNodeTable().getSchema();
    Schema localSchema2 = paramGraph.getEdgeTable().getSchema();
    checkGraphMLSchema(localSchema1);
    checkGraphMLSchema(localSchema2);
    XMLWriter localXMLWriter = new XMLWriter(new PrintWriter(paramOutputStream));
    localXMLWriter.begin("<graphml xmlns=\"http://graphml.graphdrawing.org/xmlns\"\n  xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n  xsi:schemaLocation=\"http://graphml.graphdrawing.org/xmlns\n  http://graphml.graphdrawing.org/xmlns/1.0/graphml.xsd\">\n\n", 2);
    localXMLWriter.comment("prefuse GraphML Writer | " + new Date(System.currentTimeMillis()));
    printSchema(localXMLWriter, "node", localSchema1, null);
    printSchema(localXMLWriter, "edge", localSchema2, new String[] { paramGraph.getEdgeSourceField(), paramGraph.getEdgeTargetField() });
    localXMLWriter.println();
    localXMLWriter.start("graph", "edgedefault", paramGraph.isDirected() ? "directed" : "undirected");
    localXMLWriter.comment("nodes");
    Iterator localIterator = paramGraph.nodes();
    while (localIterator.hasNext())
    {
      localObject1 = (Node)localIterator.next();
      if (localSchema1.getColumnCount() > 0)
      {
        localXMLWriter.start("node", "id", String.valueOf(((Node)localObject1).getRow()));
        for (int i = 0; i < localSchema1.getColumnCount(); i++)
        {
          localObject2 = localSchema1.getColumnName(i);
          localXMLWriter.contentTag("data", "key", (String)localObject2, ((Node)localObject1).getString((String)localObject2));
        }
        localXMLWriter.end();
      }
      else
      {
        localXMLWriter.tag("node", "id", String.valueOf(((Node)localObject1).getRow()));
      }
    }
    localXMLWriter.println();
    Object localObject1 = { "id", "source", "target" };
    String[] arrayOfString = new String[3];
    localXMLWriter.comment("edges");
    Object localObject2 = paramGraph.edges();
    while (((Iterator)localObject2).hasNext())
    {
      Edge localEdge = (Edge)((Iterator)localObject2).next();
      arrayOfString[0] = String.valueOf(localEdge.getRow());
      arrayOfString[1] = String.valueOf(localEdge.getSourceNode().getRow());
      arrayOfString[2] = String.valueOf(localEdge.getTargetNode().getRow());
      if (localSchema2.getColumnCount() > 2)
      {
        localXMLWriter.start("edge", (String[])localObject1, arrayOfString, 3);
        for (int j = 0; j < localSchema2.getColumnCount(); j++)
        {
          String str = localSchema2.getColumnName(j);
          if ((!str.equals(paramGraph.getEdgeSourceField())) && (!str.equals(paramGraph.getEdgeTargetField()))) {
            localXMLWriter.contentTag("data", "key", str, localEdge.getString(str));
          }
        }
        localXMLWriter.end();
      }
      else
      {
        localXMLWriter.tag("edge", (String[])localObject1, arrayOfString, 3);
      }
    }
    localXMLWriter.end();
    localXMLWriter.finish("</graphml>\n");
  }
  
  private void printSchema(XMLWriter paramXMLWriter, String paramString, Schema paramSchema, String[] paramArrayOfString)
  {
    String[] arrayOfString1 = { "id", "for", "attr.name", "attr.type" };
    String[] arrayOfString2 = new String[4];
    label179:
    for (int i = 0; i < paramSchema.getColumnCount(); i++)
    {
      arrayOfString2[0] = paramSchema.getColumnName(i);
      for (int j = 0; (paramArrayOfString != null) && (j < paramArrayOfString.length); j++) {
        if (arrayOfString2[0].equals(paramArrayOfString[j])) {
          break label179;
        }
      }
      arrayOfString2[1] = paramString;
      arrayOfString2[2] = arrayOfString2[0];
      arrayOfString2[3] = ((String)TYPES.get(paramSchema.getColumnType(i)));
      Object localObject = paramSchema.getDefault(i);
      if (localObject == null)
      {
        paramXMLWriter.tag("key", arrayOfString1, arrayOfString2, 4);
      }
      else
      {
        paramXMLWriter.start("key", arrayOfString1, arrayOfString2, 4);
        paramXMLWriter.contentTag("default", localObject.toString());
        paramXMLWriter.end();
      }
    }
  }
  
  private void checkGraphMLSchema(Schema paramSchema)
    throws DataIOException
  {
    for (int i = 0; i < paramSchema.getColumnCount(); i++)
    {
      Class localClass = paramSchema.getColumnType(i);
      if (TYPES.get(localClass) == null) {
        throw new DataIOException("Data type unsupported by the GraphML format: " + localClass.getName());
      }
    }
  }
  
  static
  {
    TYPES.put(Integer.TYPE, "int");
    TYPES.put(Long.TYPE, "long");
    TYPES.put(Float.TYPE, "float");
    TYPES.put(Double.TYPE, "double");
    TYPES.put(Boolean.TYPE, "boolean");
    TYPES.put(String.class, "string");
  }
  
  public static abstract interface Tokens
    extends GraphMLReader.Tokens
  {
    public static final String GRAPHML = "graphml";
    public static final String GRAPHML_HEADER = "<graphml xmlns=\"http://graphml.graphdrawing.org/xmlns\"\n  xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n  xsi:schemaLocation=\"http://graphml.graphdrawing.org/xmlns\n  http://graphml.graphdrawing.org/xmlns/1.0/graphml.xsd\">\n\n";
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\io\GraphMLWriter.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */