package prefuse.data.tuple;

import java.util.Iterator;
import java.util.logging.Logger;
import prefuse.data.Graph;
import prefuse.data.Table;
import prefuse.data.Tuple;
import prefuse.util.StringLib;
import prefuse.util.collections.IntIterator;

public class TupleManager
{
  protected Graph m_graph;
  protected Table m_table;
  protected Class m_tupleType;
  private TableTuple[] m_tuples;
  private int m_size;
  
  public TupleManager(Table paramTable, Graph paramGraph, Class paramClass)
  {
    init(paramTable, paramGraph, paramClass);
  }
  
  public void init(Table paramTable, Graph paramGraph, Class paramClass)
  {
    if (this.m_table != null) {
      throw new IllegalStateException("This TupleManager has already been initialized");
    }
    this.m_table = paramTable;
    this.m_graph = paramGraph;
    this.m_tupleType = paramClass;
    this.m_size = this.m_table.getRowCount();
    this.m_tuples = null;
  }
  
  public Class getTupleType()
  {
    return this.m_tupleType;
  }
  
  private void ensureTupleArray()
  {
    if (this.m_tuples == null) {
      this.m_tuples = new TableTuple[this.m_size];
    }
  }
  
  public void setMaximumRow(int paramInt)
  {
    if ((this.m_tuples != null) && (paramInt > this.m_tuples.length))
    {
      int i = Math.max(3 * this.m_tuples.length / 2 + 1, paramInt);
      TableTuple[] arrayOfTableTuple = new TableTuple[i];
      System.arraycopy(this.m_tuples, 0, arrayOfTableTuple, 0, this.m_size);
      this.m_tuples = arrayOfTableTuple;
    }
    this.m_size = paramInt;
  }
  
  public Tuple getTuple(int paramInt)
  {
    if (this.m_table.isValidRow(paramInt))
    {
      ensureTupleArray();
      if (this.m_tuples[paramInt] == null) {
        return this.m_tuples[paramInt] = newTuple(paramInt);
      }
      return this.m_tuples[paramInt];
    }
    throw new IllegalArgumentException("Invalid row index: " + paramInt);
  }
  
  protected TableTuple newTuple(int paramInt)
  {
    try
    {
      TableTuple localTableTuple = (TableTuple)this.m_tupleType.newInstance();
      localTableTuple.init(this.m_table, this.m_graph, paramInt);
      return localTableTuple;
    }
    catch (Exception localException)
    {
      Logger.getLogger(getClass().getName()).warning(localException.getMessage() + "\n" + StringLib.getStackTrace(localException));
    }
    return null;
  }
  
  public void invalidate(int paramInt)
  {
    if ((paramInt < 0) || (paramInt > this.m_size) || (this.m_tuples == null)) {
      return;
    }
    if (this.m_tuples[paramInt] != null)
    {
      this.m_tuples[paramInt].invalidate();
      this.m_tuples[paramInt] = null;
    }
  }
  
  public void invalidateAll()
  {
    if (this.m_tuples == null) {
      return;
    }
    for (int i = 0; i < this.m_tuples.length; i++) {
      invalidate(i);
    }
  }
  
  public Iterator iterator(IntIterator paramIntIterator)
  {
    return new TupleManagerIterator(this, paramIntIterator);
  }
  
  public class TupleManagerIterator
    implements Iterator
  {
    private TupleManager m_tuples;
    private IntIterator m_rows;
    
    public TupleManagerIterator(TupleManager paramTupleManager, IntIterator paramIntIterator)
    {
      this.m_tuples = paramTupleManager;
      this.m_rows = paramIntIterator;
    }
    
    public boolean hasNext()
    {
      return this.m_rows.hasNext();
    }
    
    public Object next()
    {
      return this.m_tuples.getTuple(this.m_rows.nextInt());
    }
    
    public void remove()
    {
      this.m_rows.remove();
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\tuple\TupleManager.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */