package prefuse.util.io;

import java.awt.Component;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.logging.Logger;
import java.util.zip.GZIPInputStream;
import javax.swing.JFileChooser;
import prefuse.data.Graph;
import prefuse.data.Table;
import prefuse.data.io.CSVTableReader;
import prefuse.data.io.DelimitedTextTableReader;
import prefuse.data.io.GraphMLReader;
import prefuse.data.io.GraphReader;
import prefuse.data.io.TableReader;
import prefuse.data.io.TreeMLReader;
import prefuse.util.StringLib;
import prefuse.util.collections.ByteArrayList;

public class IOLib
{
  public static boolean isUrlString(String paramString)
  {
    return (paramString.startsWith("http:/")) || (paramString.startsWith("ftp:/")) || (paramString.startsWith("file:/"));
  }
  
  public static URL urlFromString(String paramString)
  {
    return urlFromString(paramString, null, true);
  }
  
  public static URL urlFromString(String paramString, Class paramClass, boolean paramBoolean)
  {
    URL localURL = null;
    if (isUrlString(paramString))
    {
      try
      {
        localURL = new URL(paramString);
      }
      catch (Exception localException1)
      {
        localException1.printStackTrace();
      }
    }
    else
    {
      if (paramClass != null) {
        localURL = paramClass.getResource(paramString);
      } else {
        localURL = IOLib.class.getResource(paramString);
      }
      if ((localURL == null) && (!paramString.startsWith("/"))) {
        localURL = IOLib.class.getResource("/" + paramString);
      }
      if ((paramBoolean) && (localURL == null) && (new File(paramString).exists())) {
        try
        {
          localURL = new URL("file:///" + paramString);
        }
        catch (Exception localException2) {}
      }
    }
    return localURL;
  }
  
  public static InputStream streamFromString(String paramString)
    throws IOException
  {
    Object localObject = null;
    URL localURL = urlFromString(paramString, null, false);
    if (localURL != null)
    {
      localObject = localURL.openStream();
    }
    else
    {
      File localFile = new File(paramString);
      if (localFile.exists()) {
        localObject = new FileInputStream(localFile);
      }
    }
    if (localObject == null) {
      return null;
    }
    if (isGZipFile(paramString)) {
      return new GZIPInputStream((InputStream)localObject);
    }
    return (InputStream)localObject;
  }
  
  public static String getExtension(File paramFile)
  {
    return paramFile != null ? getExtension(paramFile.getName()) : null;
  }
  
  public static boolean isGZipFile(String paramString)
  {
    return "gz".equals(getExtension(paramString));
  }
  
  public static boolean isZipFile(String paramString)
  {
    return "zip".equals(getExtension(paramString));
  }
  
  public static String getExtension(String paramString)
  {
    int i = paramString.lastIndexOf('.');
    if ((i > 0) && (i < paramString.length() - 1)) {
      return paramString.substring(i + 1).toLowerCase();
    }
    return null;
  }
  
  public static ByteArrayList readAsBytes(InputStream paramInputStream)
    throws IOException
  {
    ByteArrayList localByteArrayList = new ByteArrayList();
    byte[] arrayOfByte = new byte[' '];
    int i = -1;
    while ((i = paramInputStream.read(arrayOfByte)) >= 0) {
      localByteArrayList.add(arrayOfByte, 0, i);
    }
    return localByteArrayList;
  }
  
  public static String readAsString(InputStream paramInputStream)
    throws IOException
  {
    StringBuffer localStringBuffer = new StringBuffer();
    byte[] arrayOfByte = new byte[' '];
    int i = -1;
    while ((i = paramInputStream.read(arrayOfByte)) >= 0)
    {
      String str = new String(arrayOfByte, 0, i);
      localStringBuffer.append(str);
    }
    return localStringBuffer.toString();
  }
  
  public static String readAsString(String paramString)
    throws IOException
  {
    return readAsString(streamFromString(paramString));
  }
  
  public static Table getTableFile(Component paramComponent)
  {
    JFileChooser localJFileChooser = new JFileChooser();
    localJFileChooser.setDialogType(0);
    localJFileChooser.setDialogTitle("Open Table File");
    localJFileChooser.setAcceptAllFileFilterUsed(false);
    SimpleFileFilter localSimpleFileFilter = new SimpleFileFilter("csv", "Comma Separated Values (CSV) File (*.csv)", new CSVTableReader());
    localSimpleFileFilter.addExtension("gz");
    localJFileChooser.setFileFilter(localSimpleFileFilter);
    localSimpleFileFilter = new SimpleFileFilter("txt", "Pipe-Delimited Text File (*.txt)", new DelimitedTextTableReader("|"));
    localSimpleFileFilter.addExtension("gz");
    localJFileChooser.setFileFilter(localSimpleFileFilter);
    localSimpleFileFilter = new SimpleFileFilter("txt", "Tab-Delimited Text File (*.txt)", new DelimitedTextTableReader());
    localSimpleFileFilter.addExtension("gz");
    localJFileChooser.setFileFilter(localSimpleFileFilter);
    int i = localJFileChooser.showOpenDialog(paramComponent);
    if (i != 0) {
      return null;
    }
    File localFile = localJFileChooser.getSelectedFile();
    localSimpleFileFilter = (SimpleFileFilter)localJFileChooser.getFileFilter();
    TableReader localTableReader = (TableReader)localSimpleFileFilter.getUserData();
    try
    {
      return localTableReader.readTable(streamFromString(localFile.getAbsolutePath()));
    }
    catch (Exception localException)
    {
      Logger.getLogger(IOLib.class.getName()).warning(localException.getMessage() + "\n" + StringLib.getStackTrace(localException));
    }
    return null;
  }
  
  public static Graph getGraphFile(Component paramComponent)
  {
    JFileChooser localJFileChooser = new JFileChooser();
    localJFileChooser.setDialogType(0);
    localJFileChooser.setDialogTitle("Open Graph or Tree File");
    localJFileChooser.setAcceptAllFileFilterUsed(false);
    SimpleFileFilter localSimpleFileFilter = new SimpleFileFilter("xml", "TreeML File (*.xml, *.treeml)", new TreeMLReader());
    localSimpleFileFilter.addExtension("treeml");
    localSimpleFileFilter.addExtension("gz");
    localJFileChooser.setFileFilter(localSimpleFileFilter);
    localSimpleFileFilter = new SimpleFileFilter("xml", "GraphML File (*.xml, *.graphml)", new GraphMLReader());
    localSimpleFileFilter.addExtension("graphml");
    localSimpleFileFilter.addExtension("gz");
    localJFileChooser.setFileFilter(localSimpleFileFilter);
    int i = localJFileChooser.showOpenDialog(paramComponent);
    if (i != 0) {
      return null;
    }
    File localFile = localJFileChooser.getSelectedFile();
    localSimpleFileFilter = (SimpleFileFilter)localJFileChooser.getFileFilter();
    GraphReader localGraphReader = (GraphReader)localSimpleFileFilter.getUserData();
    try
    {
      return localGraphReader.readGraph(streamFromString(localFile.getAbsolutePath()));
    }
    catch (Exception localException)
    {
      Logger.getLogger(IOLib.class.getName()).warning(localException.getMessage() + "\n" + StringLib.getStackTrace(localException));
    }
    return null;
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\io\IOLib.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */