package prefuse.util.ui;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.AbstractButton;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.DefaultListSelectionModel;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JToggleButton;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class JToggleGroup
  extends JPanel
{
  public static final int CHECKBOX = 0;
  public static final int RADIO = 1;
  protected final int m_type;
  protected int m_margin = 0;
  protected int m_spacing = 0;
  protected int m_axis = 0;
  protected ListModel m_data;
  protected ListSelectionModel m_sel;
  protected String[] m_labels;
  protected ButtonGroup m_group;
  private Listener m_lstnr;
  
  public JToggleGroup(int paramInt, Object[] paramArrayOfObject)
  {
    this(paramInt, new DefaultListModel(), new DefaultListSelectionModel());
    DefaultListModel localDefaultListModel = (DefaultListModel)this.m_data;
    for (int i = 0; i < paramArrayOfObject.length; i++) {
      localDefaultListModel.addElement(paramArrayOfObject[i]);
    }
    initUI();
  }
  
  public JToggleGroup(int paramInt, ListModel paramListModel)
  {
    this(paramInt, paramListModel, new DefaultListSelectionModel());
  }
  
  public JToggleGroup(int paramInt, ListModel paramListModel, ListSelectionModel paramListSelectionModel)
  {
    setLayout(new BoxLayout(this, this.m_axis));
    this.m_type = paramInt;
    this.m_data = paramListModel;
    this.m_sel = paramListSelectionModel;
    if (this.m_type == 1) {
      this.m_group = new ButtonGroup();
    }
    this.m_lstnr = new Listener(null);
    this.m_sel.addListSelectionListener(this.m_lstnr);
    initUI();
    setFocusable(false);
  }
  
  protected void initUI()
  {
    Object localObject1;
    Object localObject2;
    for (int i = 0; i < getComponentCount(); i++)
    {
      localObject1 = getComponent(i);
      if ((localObject1 instanceof JToggleButton))
      {
        localObject2 = (JToggleButton)localObject1;
        ((JToggleButton)localObject2).removeActionListener(this.m_lstnr);
        if (this.m_group != null) {
          this.m_group.remove((AbstractButton)localObject2);
        }
      }
    }
    removeAll();
    UILib.addStrut(this, this.m_axis, this.m_margin);
    for (i = 0; i < this.m_data.getSize(); i++)
    {
      if (i > 0) {
        UILib.addStrut(this, this.m_axis, this.m_spacing);
      }
      localObject1 = this.m_data.getElementAt(i);
      localObject2 = this.m_labels == null ? localObject1.toString() : this.m_labels[i];
      Object localObject3 = null;
      if (this.m_type == 0)
      {
        localObject3 = new JCheckBox((String)localObject2);
      }
      else
      {
        localObject3 = new JRadioButton((String)localObject2);
        this.m_group.add((AbstractButton)localObject3);
      }
      ((JToggleButton)localObject3).putClientProperty("idx", new Integer(i));
      ((JToggleButton)localObject3).addActionListener(this.m_lstnr);
      add((Component)localObject3);
    }
    UILib.addStrut(this, this.m_axis, this.m_margin);
    this.m_lstnr.valueChanged(null);
  }
  
  public void setAxisType(int paramInt)
  {
    setLayout(new BoxLayout(this, paramInt));
    this.m_axis = paramInt;
    initUI();
  }
  
  public int getAxisType()
  {
    return this.m_axis;
  }
  
  public void setMargin(int paramInt)
  {
    if (paramInt < 0) {
      throw new IllegalArgumentException("Margin is less than zero.");
    }
    this.m_margin = paramInt;
    initUI();
  }
  
  public int getMargin()
  {
    return this.m_margin;
  }
  
  public void setSpacing(int paramInt)
  {
    if (paramInt < 0) {
      throw new IllegalArgumentException("Spacing is less than zero.");
    }
    this.m_spacing = paramInt;
    initUI();
  }
  
  public int getSpacing()
  {
    return this.m_spacing;
  }
  
  public void setModel(ListModel paramListModel)
  {
    this.m_data = paramListModel;
    initUI();
  }
  
  public ListModel getModel()
  {
    return this.m_data;
  }
  
  public void setSelectionModel(ListSelectionModel paramListSelectionModel)
  {
    this.m_sel.removeListSelectionListener(this.m_lstnr);
    this.m_sel = paramListSelectionModel;
    this.m_sel.addListSelectionListener(this.m_lstnr);
    this.m_lstnr.valueChanged(null);
  }
  
  public ListSelectionModel getSelectionModel()
  {
    return this.m_sel;
  }
  
  public void setLabels(String[] paramArrayOfString)
  {
    if (paramArrayOfString.length < this.m_data.getSize()) {
      throw new IllegalArgumentException("Alias array is too short");
    }
    this.m_labels = paramArrayOfString;
    initUI();
  }
  
  public void setGroupFocusable(boolean paramBoolean)
  {
    for (int i = 0; i < getComponentCount(); i++)
    {
      Component localComponent = getComponent(i);
      if ((localComponent instanceof JToggleButton)) {
        localComponent.setFocusable(paramBoolean);
      }
    }
  }
  
  private class Listener
    implements ListSelectionListener, ActionListener
  {
    private boolean m_ignore = false;
    
    private Listener() {}
    
    public void valueChanged(ListSelectionEvent paramListSelectionEvent)
    {
      if (this.m_ignore) {
        return;
      }
      this.m_ignore = true;
      int i;
      boolean bool;
      Object localObject;
      if (JToggleGroup.this.m_type == 1)
      {
        i = JToggleGroup.this.m_sel.getMinSelectionIndex();
        bool = i >= 0;
        localObject = null;
        int j = 0;
        int k = 0;
        while (j < JToggleGroup.this.getComponentCount())
        {
          Component localComponent = JToggleGroup.this.getComponent(j);
          if ((localComponent instanceof JToggleButton))
          {
            localObject = (JToggleButton)localComponent;
            if (((!bool) && (((JToggleButton)localObject).isSelected())) || ((bool) && (i == k))) {
              break;
            }
            k++;
          }
          j++;
        }
        ((JToggleButton)localObject).setSelected(bool);
      }
      else
      {
        i = 0;
        bool = false;
        while (i < JToggleGroup.this.getComponentCount())
        {
          localObject = JToggleGroup.this.getComponent(i);
          if ((localObject instanceof JCheckBox)) {
            ((JCheckBox)localObject).setSelected(JToggleGroup.this.m_sel.isSelectedIndex(bool++));
          }
          i++;
        }
      }
      this.m_ignore = false;
    }
    
    public void actionPerformed(ActionEvent paramActionEvent)
    {
      if (this.m_ignore) {
        return;
      }
      this.m_ignore = true;
      JToggleButton localJToggleButton = (JToggleButton)paramActionEvent.getSource();
      boolean bool = localJToggleButton.isSelected();
      int i = ((Integer)localJToggleButton.getClientProperty("idx")).intValue();
      if (JToggleGroup.this.m_type == 1) {
        JToggleGroup.this.m_sel.setSelectionInterval(i, i);
      } else if (bool) {
        JToggleGroup.this.m_sel.addSelectionInterval(i, i);
      } else {
        JToggleGroup.this.m_sel.removeSelectionInterval(i, i);
      }
      this.m_ignore = false;
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\ui\JToggleGroup.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */