package prefuse.util.ui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import javax.swing.AbstractAction;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import prefuse.util.StringLib;

public class JValueSlider
  extends JComponent
{
  private Number m_min;
  private Number m_max;
  private Number m_value;
  private boolean m_ignore = false;
  private JLabel m_label;
  private JSlider m_slider;
  private JTextField m_field;
  private List m_listeners;
  private int m_smin = 0;
  private int m_srange = 100;
  
  public JValueSlider(String paramString, double paramDouble1, double paramDouble2, double paramDouble3)
  {
    this(paramString, new Double(paramDouble1), new Double(paramDouble2), new Double(paramDouble3));
  }
  
  public JValueSlider(String paramString, float paramFloat1, float paramFloat2, float paramFloat3)
  {
    this(paramString, new Float(paramFloat1), new Float(paramFloat2), new Float(paramFloat3));
  }
  
  public JValueSlider(String paramString, int paramInt1, int paramInt2, int paramInt3)
  {
    this(paramString, new Integer(paramInt1), new Integer(paramInt2), new Integer(paramInt3));
    this.m_smin = paramInt1;
    this.m_srange = (paramInt2 - paramInt1);
    this.m_slider.setMinimum(paramInt1);
    this.m_slider.setMaximum(paramInt2);
    setValue(new Integer(paramInt3));
  }
  
  public JValueSlider(String paramString, long paramLong1, long paramLong2, long paramLong3)
  {
    this(paramString, new Long(paramLong1), new Long(paramLong2), new Long(paramLong3));
  }
  
  public JValueSlider(String paramString, Number paramNumber1, Number paramNumber2, Number paramNumber3)
  {
    this.m_min = paramNumber1;
    this.m_max = paramNumber2;
    this.m_value = paramNumber3;
    this.m_slider = new JSlider();
    this.m_label = new JLabel(paramString);
    this.m_field = new JTextField();
    this.m_listeners = new ArrayList();
    this.m_field.setBorder(null);
    setSliderValue();
    setFieldValue();
    initUI();
  }
  
  protected void initUI()
  {
    this.m_slider.addChangeListener(new ChangeListener()
    {
      public void stateChanged(ChangeEvent paramAnonymousChangeEvent)
      {
        if (JValueSlider.this.m_ignore) {
          return;
        }
        JValueSlider.this.m_ignore = true;
        JValueSlider.this.m_value = JValueSlider.this.getSliderValue();
        JValueSlider.this.setFieldValue();
        JValueSlider.this.fireChangeEvent();
        JValueSlider.this.m_ignore = false;
      }
    });
    this.m_field.addActionListener(new AbstractAction()
    {
      public void actionPerformed(ActionEvent paramAnonymousActionEvent)
      {
        if (JValueSlider.this.m_ignore) {
          return;
        }
        JValueSlider.this.m_ignore = true;
        Number localNumber = JValueSlider.this.getFieldValue();
        if (localNumber != JValueSlider.this.m_value)
        {
          JValueSlider.this.m_value = localNumber;
          JValueSlider.this.setSliderValue();
        }
        JValueSlider.this.fireChangeEvent();
        JValueSlider.this.m_ignore = false;
      }
    });
    this.m_field.addMouseListener(new MouseAdapter()
    {
      public void mouseEntered(MouseEvent paramAnonymousMouseEvent)
      {
        String str = JValueSlider.this.m_field.getText();
        if (JValueSlider.isTextObscured(JValueSlider.this.m_field, str)) {
          JValueSlider.this.m_field.setToolTipText(str);
        }
      }
      
      public void mouseExited(MouseEvent paramAnonymousMouseEvent)
      {
        JValueSlider.this.m_field.setToolTipText(null);
      }
    });
    this.m_label.addMouseListener(new MouseAdapter()
    {
      public void mouseEntered(MouseEvent paramAnonymousMouseEvent)
      {
        String str = JValueSlider.this.m_label.getText();
        if (JValueSlider.isTextObscured(JValueSlider.this.m_label, str)) {
          JValueSlider.this.m_label.setToolTipText(str);
        }
      }
      
      public void mouseExited(MouseEvent paramAnonymousMouseEvent)
      {
        JValueSlider.this.m_label.setToolTipText(null);
      }
    });
    setLayout(new BoxLayout(this, 0));
    add(this.m_label);
    add(this.m_slider);
    add(this.m_field);
  }
  
  private static boolean isTextObscured(JComponent paramJComponent, String paramString)
  {
    Graphics localGraphics = paramJComponent.getGraphics();
    FontMetrics localFontMetrics = localGraphics.getFontMetrics(paramJComponent.getFont());
    int i = localFontMetrics.stringWidth(paramString);
    return i > paramJComponent.getWidth();
  }
  
  public Number getValue()
  {
    return this.m_value;
  }
  
  public void setValue(Number paramNumber)
  {
    this.m_value = paramNumber;
    setSliderValue();
    setFieldValue();
  }
  
  private Number getSliderValue()
  {
    int i;
    if ((this.m_value instanceof Integer))
    {
      i = this.m_slider.getValue();
      int j = this.m_min.intValue();
      int k = this.m_max.intValue();
      return new Integer(j + (i - this.m_smin) * (k - j) / this.m_srange);
    }
    if ((this.m_value instanceof Long))
    {
      i = this.m_slider.getValue();
      long l1 = this.m_min.longValue();
      long l2 = this.m_max.longValue();
      return new Long(l1 + (i - this.m_smin) * (l2 - l1) / this.m_srange);
    }
    double d1 = (this.m_slider.getValue() - this.m_smin) / this.m_srange;
    double d2 = this.m_min.doubleValue();
    double d3 = this.m_max.doubleValue();
    double d4 = d2 + d1 * (d3 - d2);
    return (this.m_value instanceof Double) ? new Double(d4) : new Float((float)d4);
  }
  
  private void setSliderValue()
  {
    int i;
    if (((this.m_value instanceof Double)) || ((this.m_value instanceof Float)))
    {
      double d1 = this.m_value.doubleValue();
      double d2 = this.m_min.doubleValue();
      double d3 = this.m_max.doubleValue();
      i = this.m_smin + (int)Math.round(this.m_srange * ((d1 - d2) / (d3 - d2)));
    }
    else
    {
      long l1 = this.m_value.longValue();
      long l2 = this.m_min.longValue();
      long l3 = this.m_max.longValue();
      i = this.m_smin + (int)(this.m_srange * (l1 - l2) / (l3 - l2));
    }
    this.m_slider.setValue(i);
  }
  
  private Number getFieldValue()
  {
    if (((this.m_value instanceof Double)) || ((this.m_value instanceof Float)))
    {
      double d;
      try
      {
        d = Double.parseDouble(this.m_field.getText());
      }
      catch (Exception localException1)
      {
        return this.m_value;
      }
      if ((d < this.m_min.doubleValue()) || (d > this.m_max.doubleValue())) {
        return this.m_value;
      }
      return (this.m_value instanceof Double) ? new Double(d) : new Float((float)d);
    }
    long l;
    try
    {
      l = Long.parseLong(this.m_field.getText());
    }
    catch (Exception localException2)
    {
      return this.m_value;
    }
    if ((l < this.m_min.longValue()) || (l > this.m_max.longValue())) {
      return this.m_value;
    }
    return (this.m_value instanceof Long) ? new Long(l) : new Integer((int)l);
  }
  
  private void setFieldValue()
  {
    String str;
    if (((this.m_value instanceof Double)) || ((this.m_value instanceof Float))) {
      str = StringLib.formatNumber(this.m_value.doubleValue(), 3);
    } else {
      str = String.valueOf(this.m_value.longValue());
    }
    this.m_field.setText(str);
  }
  
  public void addChangeListener(ChangeListener paramChangeListener)
  {
    if (!this.m_listeners.contains(paramChangeListener)) {
      this.m_listeners.add(paramChangeListener);
    }
  }
  
  public void removeChangeListener(ChangeListener paramChangeListener)
  {
    this.m_listeners.remove(paramChangeListener);
  }
  
  protected void fireChangeEvent()
  {
    Iterator localIterator = this.m_listeners.iterator();
    ChangeEvent localChangeEvent = new ChangeEvent(this);
    while (localIterator.hasNext())
    {
      ChangeListener localChangeListener = (ChangeListener)localIterator.next();
      localChangeListener.stateChanged(localChangeEvent);
    }
  }
  
  public void setBackground(Color paramColor)
  {
    this.m_field.setBackground(paramColor);
    this.m_label.setBackground(paramColor);
    this.m_slider.setBackground(paramColor);
    super.setBackground(paramColor);
  }
  
  public void setForeground(Color paramColor)
  {
    this.m_field.setForeground(paramColor);
    this.m_label.setForeground(paramColor);
    this.m_slider.setForeground(paramColor);
    super.setForeground(paramColor);
  }
  
  public void setFont(Font paramFont)
  {
    this.m_field.setFont(paramFont);
    this.m_label.setFont(paramFont);
    this.m_slider.setFont(paramFont);
    super.setFont(paramFont);
  }
  
  public void setPreferredSize(Dimension paramDimension)
  {
    int i = Math.min(40, paramDimension.width / 5);
    int j = Math.min(100, (paramDimension.width - i) / 2);
    int k = paramDimension.width - i - j;
    super.setPreferredSize(paramDimension);
    Dimension localDimension = new Dimension(j, paramDimension.height);
    this.m_label.setPreferredSize(localDimension);
    localDimension = new Dimension(k, paramDimension.height);
    this.m_slider.setPreferredSize(localDimension);
    localDimension = new Dimension(i, paramDimension.height);
    this.m_field.setPreferredSize(localDimension);
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\ui\JValueSlider.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */